#!python

import os,sys, getopt, re
from qblib import process_dir, Clone, Fasta, DNA, align

from qblib.config import *
# for all sequences within a list of directories, assemble them by clone

def report_mutation(outfile, alignfile, reffile='',start=0, end=951):
	F = open(outfile, 'w')

	refd = {}
	'''
	for a in Fasta.all(reffile):
		refd[a.name] = a.sequence
		refname = a.name
		refseq = a.sequence
		refseq = re.sub('\*$', '', refseq)
		refseq = refseq[:end]

		reflist = list(refseq)
	'''
	name2muts = {}
	

	mutpos2count = {}
	seqs = Fasta.all(alignfile)
	r = seqs[0]
	refname = r.name
	refseq = r.sequence
	reflist = list(refseq)
	#print refname, refseq

	refbase = re.sub('[\.\,\~\-]', '', refseq)
	refbcnt = len(refbase)
	print >>F, 'Name\tInfo\tTag\tMutations'
	for a in seqs[1:]:
		seq = re.sub('\*$', '', a.sequence)
		tag = ''
		muts = []
		alist = list(seq)
		# remove trailing *
	
		myseq = re.sub('[\.\,\~\-]', '', seq)
		mycnt = len(myseq)
		print a.name, mycnt, refbcnt
		if mycnt > refbcnt:
			tag = 'insertion'
		elif mycnt < refbcnt:
			tag = 'deletion'
		#print alist

		fs = '' 
		for i in range(0, len(refseq)):
			mutpos2count.setdefault(i+1, 0)

			r = reflist[i].upper()
			if len(alist) < len(refseq):
				break
			s = alist[i].upper()

			if s == 'X':
				fs = 'Frameshift'
			if r != s:
				muts.append('%s%s%s' % (r, i+1,s))
				mutpos2count[i+1] += 1

		if not muts:
			tag ='WT'

		#print '%s\t%s\t%s\t%s' % ( a.name, fs, tag, "\t".join(muts))
		print >>F, '%s\t%s\t%s\t%s' % ( a.name,  fs, tag, "\t".join(muts))
		name2muts[a.name] = muts
	F.close()

	# need to wait until the end to write all mutation for a particular position
	# in a column

	poss = []
	for i,v in mutpos2count.items():
		if v == 0:
			continue
		poss.append(i)
	poss.sort()
	#print poss

	F = open('x.txt', 'w')
	names = name2muts.keys()
	names.sort()
	for name in names:
		muts = name2muts[name]
		tmp = [name]

		myp2m = {}
		for mut in muts:
			pos = re.findall('\d+', mut)
			if not pos:
				continue
			pos, = pos
			pos = int(pos)
			myp2m[pos] = mut

		for i in poss:
			if myp2m.has_key(i):
				tmp.append(myp2m[i])
			else:
				tmp.append('')
		print >>F, "\t".join(tmp)
	F.close()		


def do_assembly(tmpdir, dirlist, fprimer='', rprimer='', min_distance='', max_distance='',  outdir='', platename='', cleanstart='', assembly_prog="cap3", score=0, length=0):
	print "tmpdir: %s" % tmpdir
	print "outdir:", outdir

	if cleanstart:
		os.system('rm -rf %s' % tmpdir)
		os.system('rm -rf %s' % outdir)

	if not os.path.isdir(tmpdir):
		os.mkdir(tmpdir)

	cloned = process_dir.get_allclones('_', dirlist, score=score, length=length)
	for c, v in cloned.items():
		if v is None:
			continue
		print c, len(v.reads)

	curdir = os.path.abspath(os.path.curdir)
	if not outdir:
		outdir = curdir
	if not os.path.isdir(outdir):
		os.mkdir(outdir)

	contig_file = os.path.join(outdir, '%s_contigs.fas' % platename)
	fout = open(contig_file, 'w')

	clonenames = cloned.keys()
	clonenames.sort()
	
	for name in clonenames:
		clone = cloned[name]
		clone.set_dir(tmpdir, outdir)
		print "Clone Name:", name
		x=clone.do_assembly(fprimer, rprimer, notrim=0, assembly_prog=assembly_prog)
		try:
			contigseqs = clone.get_contig_seqs()
		except:
			continue
		for s in contigseqs:
			#print s.name, s.sequence
			print >>fout, s
	fout.close()

	print "Contig Sequence File: %s" % contig_file
	#html=align.do_muscle(contig_file, format='html')
	#os.system('mv -f %s .' % contig_file)
	#os.system('mv -f %s .' % html)

	return contig_file

def align_reads(tmpdir, dirlist, outdir, reffile, refname):
	# get all reads
	cloned = process_dir.get_clones(dirlist, outdir)

	# write all reads into a fasta file
	refs = Fasta.all(reffile)
	refd = {}
	for r in refs:
		refd[r.name] = r.sequence

def trim2ref(reffile, refname='', contig_file='', outdir='', reftype='CDS', platename=''):
	if not outdir:
		outdir = os.getcwd()
	if refname:
		# get all reference sequences
		refs = Fasta.all(os.path.abspath(reffile) )
		refd = {}
		for s in refs:
			refd[s.name] = s.sequence

		my_refseq = refd[refname]
		print "REF: %s" % refname

		myreffile = os.path.join(outdir, 'my_tmpref.fas' )
		F = open(myreffile, 'w')
		print >>F, '>%s\n%s' % (refname, my_refseq)
		F.close()
	else:
		myreffile = reffile
	

	outfile = os.path.join(outdir, '%s_nt.fas' % platename)
	if reftype == 'CDS':	
		outfile = os.path.join(outdir, '%s_cds.fas' % platename)

	print "REFFILE:", myreffile
	print "CONTIG:",  contig_file
	print "OUTFILE:", outfile

	
	align.trim2ref(myreffile, contig_file, outfile, debug=1)
	
	print "TRIM FASTA:", outfile
	return outfile
	# generate some alignment files

def usage():
	print """
	CloneAssembler.py [option] <dir1> <dir2> ...
	options:
	-a: do assembly and alignment
	-A: align reads to reference
	-o <optional>, output directory, default is cwd
	-r <optional>, reference file, optional, if present, trim contig sequence 
	   to the start and end of reference
	-F <optional>, forward primer
	-R <optional>, reverse primer
	-p <platename>
	-M <alignment> do_mutation report
	-N nucleotide reference
	-G cds reference, will translate and align
	-S <cap3 or cap>: default is cap3
	-s <score cutoff>
	-l <length cutoff>
	<dirlist> required

	ex: CloneAssembler.py -r ref.fas -o report 012309_DW 012509DW
	"""
	sys.exit()


def do(args):
	curdir = os.path.abspath(os.path.curdir)
	opts, ars = getopt.getopt(args, 'aco:r:F:R:G:N:p:M:S:s:l:')
	optd = dict([(x,y) for (x,y) in opts])
	if optd.has_key('-M'):
		alignfile = optd['-M']
		platename = optd['-p']
		reffile = optd['-r']
		return report_mutation('%s_mutation_summary.txt' % platename, alignfile, reffile)


	print opts
	if not ars:
		ars = [os.path.abspath(os.path.curdir)]
	dirlist = ars		

	outdir = os.path.abspath(os.path.curdir)
	reffile = refname = ''
	fprimer = rprimer = ''

	cleanup = 0

	dont =  docds = 0
	do_alignment = 0
	nt_refname = ''
	cds_refname = ''
	align_read = 0
	platename = 'all'

	do_mutreport =0

	assembly_prog = "cap3"
	score = 0
	length = 0	
	for opt in  opts:
		o, v = opt
		if o == '-s':
			score = optd['-s']
		elif o == '-l':
			length = optd['-l']

		if o  == '-p':
			platename = v
		if o == '-A':
			align_read = 1

		if o == '-c':
			cleanup = 1
		if o == '-o':
			outdir = v
		if o == '-r':
			reffile = v
			dont = 1
		if o == '-F':
			fprimer = v
		if o == '-R':
			rprimer = v
		if o == '-N':
			dont = 1
			nt_refname = v
		if o == '-G':
			docds = 1
			cds_refname = v
			print "DO CDS", cds_refname
		if o == '-a':
			do_alignment = 1

		if o== '-M':
			do_mutreport = 1

		if o == '-S':
			assembly_prog = v

	dname = os.path.basename(curdir)
	tmpdir = os.path.join(curdir, 'tmp_%s' % dname)
	if not os.path.isdir(tmpdir):
		os.mkdir(tmpdir)
	outdir = os.path.join(curdir, 'out_%s' % dname)
	if not os.path.isdir(outdir):
		os.mkdir(outdir)

	if cds_refname:
		refname = cds_refname
	elif nt_refname:
		refname = nt_refname
	else:
		refname = ''
	if align_read:
		return align_reads(tmpdir, dirlist, outdir, reffile, refname)

	contig_file = do_assembly(tmpdir, dirlist, fprimer, rprimer, outdir=outdir, platename=platename, cleanstart=cleanup, assembly_prog=assembly_prog, score=score, length=length)
	print "DO CDS2:", cds_refname

	if not reffile and not docds:
		if do_alignment:
			print "doing alignment ...."
			align.do_muscle(contig_file, "html")

		# cleanup
		if cleanup:
			os.system('rm -rf %s' % tmpdir)
		return


	print "DO CDS3:", cds_refname
	print "DO NT: %s, DO CDS: %s" % (dont, docds)
	if dont:
		print "REFNAME: ", nt_refname
		reftype = 'NT'
		if not nt_refname and cds_refname:
			nt_refname = cds_refname
		nt_contigfile=trim2ref(reffile, refname=nt_refname, contig_file = contig_file, outdir=outdir, reftype=reftype, platename=platename)
		print "contigfile:", nt_contigfile
		align.do_muscle(nt_contigfile, 'html')
		#align.do_muscle(nt_contigfile, 'fas')
		
	if docds:
		print "DO CDS4:", cds_refname
		cds_contigfile=trim2ref(reffile, refname=cds_refname, contig_file = contig_file, outdir=outdir, reftype='CDS', platename=platename)
		cds_html=align.do_muscle(cds_contigfile, 'html')
		cds_align_fasta=align.do_muscle(cds_contigfile, 'fas')

		# do translation
		aafile = cds_contigfile.replace('_cds.', '_aa.')
		print "AA file:", aafile
		Faa =  open(aafile, 'w')

		#all = Fasta.all(cds_contigfile)
		all = Fasta.all(cds_align_fasta)
		for s in all:
			aa = DNA.DNA(s.sequence.upper()).translate()
			print >>Faa, '>%s\n%s' % (s.name, aa)
		Faa.close()

		align.do_muscle(aafile, 'html')


	if do_mutreport:
		alignfile = ''
		if os.path.isfile(cds_align_fasta):
			alignfile = cds_align_fasta
		else:
			alignfile = align.do_muscle(nt_contigfile, 'fas')
		
		report_mutation(platename, alignfile, reffile)


	# cleanup
	if cleanup:
		os.system('rm -rf %s' % tmpdir)
	return

if __name__ == '__main__':
	do(sys.argv[1:])
