#!python
import os, sys, glob, re, getopt, pprint, time, copy
from qblib import Fasta, DNA 
import SampleScore

# 7/21/09
from qblib import process_dir, util


QUALITY_SCORE= 15
DNA_INT_LEN=400
AA_INT_LEN=130
DEBUG = 0
MUSCLE_EXE = "c:\\tools\\bin\\muscle.exe"

primer_d = {'5440S': 'L', '5442A': 'H', '544SX2':'L', '5455S':'L'}

H_StartTag = 'EVQL'; H_EndTag = 'TVSS'; H_MidTag = 'YYC'

MyD={
1: 'A11', 2: 'B11', 3: 'C11', 4: 'D11', 5: 'E11', 6: 'F11', 7: 'G11', 8: 'H11',
9: 'A12', 10: 'B12', 11: 'C12', 12: 'D12', 13: 'E12', 14: 'F12', 15: 'G12', 16: 'H12'
}
_5440S = 'AGCGGATAACAATTTCACACA'
_5442A = 'TGGGCTGCCTGGTCAAGGAC'

#LIGHT_STARTS = [('S[HY]SAQ', '((TC[TGCA])|(AG[TC]))[TC]A[CT]((TC[TGCA])|(AG[TC]))GC[GATC]', 0)]
LIGHT_STARTS = [ ('S[HY]SAQD', 'TCT[TC]A[CT]AGTGCACA[AG]GA[TC]', 15),  
		 ('S[HY]SAQ',  'TCT[TC]A[CT]AGTGCACA[AG]', 12),  
		('DRVT', 'GA[TC]((CG[CGAT])|(AG[AG]))', 0)]
'''
LIGHT_ENDS = [('QPKA', 'CA[AG]CC[GATC]AA[AG]GC[GATC]', 5), # lambda vector
		('RTVA', '((CG[GATC])|(AG[AG]))AC[AGTC]GT[GATC]GC[GATC]GT[GATC]GC[GATC]', 0)] # kappa vector
'''
LIGHT_ENDS = [('GQPK', 'GG[GATC]CA[AG]CC[GATC]AA[AG]', -12), #-12
('QPKA', 'CA[AG]CC[GATC]AA[AG]GC[GATC]', -15), # lambda vector
		('RTVA', 'CGAAC[AGTC]GT[GATC]GC[GATC]', 0), # kappa vector
		('TVA', 'AC[AGTC]GT[GATC]GC[GATC]', 0)] # kappa vector

ENZ_5440S = [('MfeI', 'CAATTG'), ('NheI', 'GCTAGC')]
ENZ_5442A = [('ApaL1', 'GTGCAC'), ('AscI', 'GGCGCGCC')]


def process_dir(dname, primer, pcrresult='', outdir='', alignment='', anqi=0, do_RD1=0):
	if primer in ['5442A', 'H']:
		chaintype = "H"
		#chaintype = "Heavy chain"
	else:
		chaintype = "L"
		#chaintype = "Light chain"

	if not outdir:
		print "DNAME:", dname, primer
		if primer:
			outdir = dname.replace(primer, 'analysis')	
		else:
			outdir = "%s_analysis" % dname

		print "  OUTDIR", outdir


	if not os.path.isdir(outdir):
		print ".. create directory", outdir
		os.mkdir(outdir)

	seqs = get_seqs(dname, primer)
	if not seqs:
		return ('', '', '')

	# requested by QA, translation of low quality bases within trimmed sequences changed to X
	regional_mix_seqs = []
	if regional_mix_seqs:
		fout = open(os.path.join(outdir, '%s_localmix_aa.fas' % os.path.basename(dname)), 'w')
		for s in regional_mix_seqs:
			print >>fout, '>%s' % s.basename
			print >>fout, s.aa
		fout.close()
		seqs.extend(regional_mix_seqs)

		fout = open(os.path.join(outdir, '%s_localmix_na.fas' % os.path.basename(dname)), 'w')
		for s in regional_mix_seqs:
			print >>fout, '>%s' % s.basename
			print >>fout, s.trimdna
		fout.close()

	if do_RD1:
		return report_RD1(dname, seqs, align=alignment)

	# try to figure out information for bad clone
	bad_d = {}
	badcnt = 0
	hc_90 = 'TGGCCCCAG'
	lc_90 = 'CTCACAGTGC'
	sc_90 = 'GCGCGCACAG'
	badnum = 0

def rename(change=0, format='name_well_primer', oldpat='', newpat='', primer=''):
	
	print "name format:", format, primer

	files = glob.glob('*.seq') + glob.glob('*.ab1') + glob.glob('*.phd.1')

	pd = {'5440S':'L', '5442A':'H', '5455S':'L', '544SX2':'L', 'H':'H', 'L':'L'}
			
	for f in files:
		me = re.search('(\.ab1$)|(\.seq)|(\.phd\.1)', f)
		if me:
			ext = f[me.start():]
			b = f.replace(ext, '')
		else:
			print "ERROR: error processing file name"
			sys.exit()

		if oldpat and newpat:
			f2 = f.replace(oldpat, newpat)
			print f, f2
			if change:
				os.rename(f,f2)
			continue

		cols = b.split('_')
		if len(cols) < 2:
			continue
		well = cols[-2]
		plate = cols[0]
		if not primer:
			primer= cols[1].upper()
		#primer = 'L'
		p2 = pd[primer]

		if format=='name_well_primer':
			name = '%s%s%s' % (plate, well, p2)
		elif format == 'name_primer':
			name = '%s%s' % (plate, p2)
		else:
			name = cols[0]
		f2 = "%s%s" % (name, ext)
		print f, f2
		if change:
			try:
				os.rename(f,f2)
			except:
				print "rename file error"

def _find_start(seq, stag):

	pat =re.compile(stag)

	m = re.search(pat, seq)
	if m:
		return m.start()
	return 0

def _find_end(seq, end_tags=[], start=0, offset=0, find_TVL=0):
	if type(end_tags) == type(''):
		end_tags = [end_tags]

	pats = [re.compile(s) for s in end_tags]

	for p in pats:
		seq = seq[start:]
		m = re.search(p, seq, start)
		if m:
			end = start+m.end()+offset
			if find_TVL:
				tag = re.compile('AC[GATC]GT[GATC]((CT[GATC])|(TT[AG]))' )
				m = re.search(tag, seq)
				if m:
					return start+m.end()
				else:
					return end
			return end

	return 0

def trimseq_5442A(name, seq):
	# use 3 different start tags
	stag = 'EVQL' ; natag = 'GA[GA]GT[GATC]CA[AG]((CT[GATC])|(TT[AG]))'
	start = _find_start(seq, natag)
	if not start:
		# VQLL
		natag = 'GT[GATC]CA[AG]((CT[GATC])|(TA[AG]))((CT[GATC])|(TA[AG]))'
		start = _find_start(seq, natag)
		if start:
			start -= 1*3
		else:
			# AMAE
			natag = 'GC[GATC]ATGGC[GATC]GA[GA]'
			start = _find_start(seq, natag)
			if start:
				start += 3*3

	endtags = [('TVSS', 'AC[GATC]GT[GATC]((TC[GATC])|(AG[TC]))((TC[GATC])|(AG[TC]))', 0),
			('ASTKG', 'GC[GATC]((TC[GATC])|(AG[TC]))AC[GATC]AA[AG]GG[GATC]', -15),
		('STKG', '((TC[GATC])|(AG[TC]))AC[GATC]AA[AG]GG[GATC]', -12),
		('FPLA', 'TT[TC]CC[GATC]((CT[GATC])|(TT[AG]))GC[GATC]', 8*3)]
	for etag, natag, offset in endtags:
		end = _find_end(seq, natag)
		if end:
			end += offset
			break

	if not start and not end:
		trimseq = ''
		stag = 'EVQL'; etag = 'ASTKG'
	elif not start and end:
		trimseq = ''
		stag = 'EVQL'

	elif start and not end:
		trimseq = seq[start:start+500]
		etag = 'ASTKG'
	else:
		trimseq = seq[start:end]
	

	return (start, stag, end, etag, trimseq)





def simple_grouping(notgrouped, funique, fgroup='', combine=0, anqi=0, seqs=[], bads=[], fails=[], align=''):
	sample2score = {}
	trimseq2score = {}
	for s in seqs+bads+fails:
		sample2score[s.basename] = s.sample_score
		trimseq2score[s.basename] = s.trimscore

	if fgroup:
		print >>fgroup, '\n---------------------------------------------------'

	groupd = {}
	well2group = {}
	noyyc = []
	name2well = {}

	name2s={}
	for s in seqs:
		name = s.basename
		name2s[name] = s
		groupd.setdefault(s.aa, []).append(name)
		name2well[name] = s.well
		
	group_by_count = [(len(v), k,v) for (k, v) in groupd.items()]
	group_by_count.sort()

	badnames = [x for (x,_) in notgrouped]
	group_by_count2 = []
	seqcount = 0
	for (l,seq,names) in group_by_count:
		gnames = []
		for n in names:
			if n not in badnames:
				gnames.append(n)
		if not gnames:
			continue
		seqcount += len(gnames) 
		gnames.sort()
		group_by_count2.append((l, seq, gnames))
	group_by_count = group_by_count2
	group_by_count.reverse()

	count_d = {}
	for (count, seq, names) in group_by_count:
		names.sort()
		#print "...count =%s, sample=%s" % (count, names )
		count_d.setdefault(count, []).append( (names, seq) )

	#print "-->group count", len(group_by_count)

	counts = count_d.keys()
	counts.sort()
	counts.reverse()

	# sort group for the same count
	counts.sort()
	counts.reverse()
	group_by_count3 = []
	for cnt in counts:
		grouplist =count_d[cnt]
		grouplist.sort()

		#print cnt,  len(grouplist)

		i = 1
		for (names, seq) in grouplist:
			names.sort()
			#print a
			group_by_count3.append((cnt, seq, names, "%s-%s" % (cnt, i)))
			i += 1

	group_by_count = group_by_count3
	print ".. Found %s groups from %s sequences:" % (len(group_by_count), seqcount)
	if fgroup:
		print >>fgroup, "Found %s groups from %s sequences:" % (len(group_by_count), seqcount)
		print >>fgroup, '--------------------------------------------------------'

	i = 1
	uniques = []
	
	unique_groupids=[]
	
	plate_d = {}
	group_sums= []
	
	for (l, seq, names, groupid) in group_by_count:

		#print "group %s has %s " % (i, l)
		if combine:
			first, names  = get_first(names)
		else:
			first = names[0]

		group_sums.append((first, names))

		if fgroup:
			names2 = [str(x) for x in names]
			names2[0] += "(%s)" % l
			print >>fgroup, "group %s has %s: %s " % (i, l, ",".join(names2))
			print >>fgroup, "seq: %s//" % seq
			#print "%s group, count:%s id:%s" % (i, l, groupid)

		#uniques.append((first, seq))
		uniques.append((",".join(names2), seq))
		unique_groupids.append((names, groupid))

		i += 1
		if combine:
			continue
		
		for n in names:
			#platename = n[:-4]
			p28 = 0
			if p28:
				platename, = re.findall('28P\d+', n)
			p31 = 1
			if p31:
				platename = re.findall('31P\d+', n)
				if not platename:
					platename = re.findall('31Y\d+', n)
				
				if platename:
					platename, = platename	
				else:
					platename = n[:5]
			else:
				platename = n[:5]
			plate_d[platename] = 1
			if fgroup: 
				try: 
					score = sample2score[n] 
				except:
					n2 = n.replace('(-)','')
					try:
						score = sample2score[n2]
					except:
						score = ''
						continue

				try:
					trimscore = trimseq2score[n]
				except:
					n2 = n.replace('(-)','')
					try:
						trimscore = trimseq2score[n2]
					except:
						trimscore = ''

				s = name2s[n]

				print >>fgroup, "name: %s, sample score = %s, average trimseq score = %s" % (n, s.sample_score, trimscore), s.tag

			well = name2well[n]
			well2group[well] = (i, score)

		if not fgroup:
			continue

		print >>fgroup, '++++++++++++++++++++\n'

	if funique:
		uniques.sort()
		for (name, seq) in uniques:
			print >>funique, '>%s' % name
			print >>funique, "%s//" % seq

	# write summary_table for An Qi
	if anqi:
		fsum = open('summary_table.txt', 'w')
		platenames = plate_d.keys()
		platenames.sort()
		print >>fsum, "\t".join(['']+platenames + ['Total', '1st_P39_P41'])

		for (first, names) in group_sums:
			gplate = []
			for x in names:
				p, = re.findall('28P\d+', x)
				gplate.append(p)
			total = len(names)
			tmp = ['%s(%s)' % (first, total)]
			for p in platenames:
				c = gplate.count(p)
				if c:
					tmp.append(c)
				else:
					tmp.append('')
			tmp.append(total)

			# special, find first aprerance in 39 or 41
			if 1:
				first39_41,_ = find_first39_41(names)
				tmp.append(first39_41)
			
			print >>fsum, '\t'.join([str(x) for x in tmp])
		fsum.close()

	print >>fgroup, '\n---------------------------------------------------'
	print >>fgroup, 'The following were not used in grouping'
	print >>fgroup, '---------------------------------------------------'
	for (name, msg) in notgrouped:
		print >>fgroup, '  %s: %s' % (name.replace('>', ''), msg) 
		
	for n in noyyc:
		print >>fgroup, "  %s: no YYC" % (n)


	for s in bads+fails:
		well2group[s.well] = ('-', s.sample_score)
		unique_groupids.append((s.basename,''))
	
	return (well2group, unique_groupids)

def get_first(names):
	names2 = []
	plate = ''
	for n in names:
		#print n
		plate, = re.findall('P\d+', n)
		if plate:
			try:
				well = n.replace(plate, '')
			except:
				print "find plate error for ", n, plate

			plate = eval(plate.replace('P', ''))
			names2.append([plate, well, n])

	if not plate:
		names.sort()
		return (names[0], names)

	names2.sort()

	first =	names2[0][-1]
	names = [x[-1] for x in names2]

	return (first, names)

def find_first39_41(names):
	names2 = [x for x in names if x.find('39') != -1 or x.find('41') != -1]
	if not names2:
		return ['', names]
	return get_first(names2)

def merge_group(g1, g2, outfile='groups.txt', outdir='', plate=''):
	p1, w2g1 = g1
	p2, w2g2 = g2

	data = []	
	wells = w2g1.keys()
	wells.sort()

	if plate:
		outfile = "%s_%s" % (plate, outfile)
    	fout2 = open(os.path.join(outdir, outfile), 'w')
	print >>fout2, '\t'.join(['well', '%s_Group'%p1, '%s_SampleScore' % p1, '%s_Group' % p2, '%s_SampleScore' % p2])
	data = []

	for r in ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H']:
		for c in range(1, 13):
			well = "%s%s" % (r, c)
			(g1, score1) = w2g1[well]

			well = well.replace('(-)','')

			if w2g2.has_key(well):
				(g2, score2) = w2g2[well]

				del w2g2[well]
			else:
				g2 = ''; score2 = ''
			d = (well, g1, score1, g2, score2)
			data.append(d)

	for k, v in w2g2.items():
		group, score = v
		data.append((k, '', '', group, score))

	for d in data:
		print>>fout2, '\t'.join([str(x) for x in d])
	fout2.close()

def clustal(trimseq_file):
	cmd = 'C:\\BioEdit\\apps\\clustalw.exe %s' % trimseq_file
	os.system(cmd)
	outfile = trimseq_file

def muscle(infile):
	infile = infile.replace('\\', '\\\\')
	outfile = infile.replace('.fas', '.aln')
	cmd = "%s -quiet -in %s -out %s -clwstrict" % (MUSCLE_EXE, infile, outfile)
	print cmd
	os.system(cmd)

def anqi_grouping(plates, plate2seqd, outdir):
	plate_hlseqd = {}
	for p, seqd in plate2seqd.items():
		print '... plate:', p, seqd.keys()
		plate_hlseqd[p] = {}
		sample2seq = {}

		print "====H===="
		primers = seqd.keys()

		hd = {}
		ld = {}
		for pr in primers:
			if re.search('5442', pr):
				hd = seqd[pr]
				pr = 'H'
			else: 
				ld = seqd[pr]
				pr = 'L'

		for s in hd:
			if not s.aa:
				continue

			sample = s.getsample(emd=1)
			if re.search('[\*\#]', s.aa):
				continue
			print "\tX=%s" % sample, s.aa
			sample2seq.setdefault(sample, s.aa)

		for s in ld:
			sample = s.getsample(emd=1)
			if sample2seq.has_key(sample):
				if not re.search('[\*\#]', s.aa):
					sample2seq[sample] += "++%s//" % s.aa
				else:
					del sample2seq[sample]

		# set up the plate dict
		for sample, seq in sample2seq.items():
			plate_hlseqd[p].setdefault(seq, []).append(sample)
		
	#plates = ['31P78', '31FR87', '31P102', '31FR103A', '3FR103B', '31FR108', '31FR109']
	fall = open(os.path.join(outdir, 'allseqs.fas'), 'w')
	print "\t ALL PLATES:", plate_hlseqd.keys()
	print "\t all unique sequences write to %s/allseqs.fas" % outdir

	all_data = []
	doned = {}

	#
	print "\n===> create allseqs.fas"
	for p in plates:
		seqd = plate_hlseqd[p]

		# sort the name of each unique sequence
		unique_seqs = []
		for seq, samples in seqd.items():
			samples.sort()
			unique_seqs.append([seq, samples])

		# sort all unique sequences for this plate by name
		unique_seqs.sort(lambda x,y: cmp(x[1], y[1]))
		print "\tunique seqs count:", len(unique_seqs)
	
		# for all unique sequences in this plate "p", find out how many are in other plates
		for (seq, samples) in unique_seqs:
			for name in samples:
				print "\t",name
				if doned.has_key(name):
					continue
				doned[name] = ''
				print >>fall, ">%s" % name
				print >>fall, seq

			sample1 = samples[0]
			platecnt = len(samples)
			data = ["%s(%s)" % (sample1, platecnt)]

			total = 0
			for xplate in plates:
				if not plate_hlseqd.has_key(xplate):
					data.append(0)
					continue
				seqd = plate_hlseqd[xplate]
				
				if not seqd.has_key(seq):
					data.append(0)
				else:
					data.append(len(seqd[seq]))
					total += len(seqd[seq])
					if xplate == p:
						continue

					for n in seqd[seq]:
						if doned.has_key(name):
							continue
							doned[name] = ''
						print >>fall, '>%s' % n
						print >>fall, seq

			data.append(total)
			all_data.append(data)

			data = [str(x) for x in data]
			#`print  "\t".join(data)
	fall.close()
	platecnt = len(plates)
	total_by_plate = ['Total']
	for i in range(1, platecnt+2):
		count = sum([x[i] for x in all_data])
		total_by_plate.append(count)
	all_data.append(total_by_plate)

	outfile = os.path.join(outdir, 'anqi_special_group.txt')
	print " !!!! ANQI group file:", outfile
	fout = open(outfile, 'w')
	if DEBUG:
		print "\t".join([""] + plates + ['Total'])
	print >>fout, "\t".join([""] + plates + ['Total'])

	for data in all_data:
		data2 = []
		for x in data:
			if type(x) == type(''):
				data2.append(x)
			elif x == 0:
				data2.append('')
			else:
				data2.append(str(x))
		if DEBUG:
			print  "\t".join(data2)
		print >>fout, "\t".join(data2)
	fout.close()
	print ".... done ...."

def anqi_sort(mylist):
	list2 = []

	for row in mylist:
		x = row[0]	
		p = re.search('P\d+', x)
		plate = well = ''
		if p:
			plate = x[p.start():p.end()]
			platenum, = re.findall('\d+', plate)
			platenum = int(platenum)

			well = x[p.end():]
		#
		print x, plate, well	
		list2.append((platenum, well, row))
	list2.sort()

	return [x for (_,_,x) in list2]


def get_summary(names):
	names2 = []

	plate2wells = {}

	wellpat = re.compile('[A-H]\d\d?\-?\d?$')
	for n in names:
		print 'NAME:', n
		well, = re.findall(wellpat, n)
		plate = re.sub('%s$' % well, '', n)	
		plate2wells.setdefault(plate, []).append(n)

	plates = plate2wells.keys()	
	plates.sort()
	samples = plate2wells[plates[0]]
	samples.sort()

	p2c = {}
	for p, w in plate2wells.items():
		p2c[p] = len(w)
	return (samples[0], p2c)


def align():
	files = glob.glob('*.seq')
	d = {}
	for f in files:
		b, e = f.split('.')
		name, primer = b.split('_')
		d.setdefault(name, []).append((primer, f))

	for name, seqs in d.items():
		if len(seqs)<2:
			continue
		fout = open('%s.fas' % name, 'w')
		for (primer, f) in seqs:
			b,e = f.split('.')
			seq = open(f).read().strip()
			seq = "".join(seq.split())
			if primer == '5440S':
				b = b+'(-)'
				seq = reverse_complement(seq)
			print >>fout, ">%s" % b
			print >>fout, seq
		fout.close()

		os.system('c:\\tools\\bin\\muscle.exe -quiet -in %s.fas -out %s.aln -clwstrict' % (name, name))

def locate_primer(dname):
	pat = '(5442A)|(5440S)|(5455S)|(544sx2)|(544SX2)'
	m = re.search(pat, dname, re.IGNORECASE)
	if m:
		primer = dname[m.start():m.end()]
		if primer in ['5442a', '5442A']:
			return "H"
		return "L"
	return ""

"""
this function combine data from different directories
"""
def combine_data(outdir, platename, dirs):
	# d is the directory path to subdir _5442* ...

	primer2seqs = {}
	datadirs = []
	for d in dirs:
		print '... checking' , d
		if not os.path.isdir(d):
			continue
		datadirs.append(d)

	if not os.path.isdir(outdir):
		os.mkdir(outdir)

	do_plate(platename, datadirs, outdir)

def do_qscore(optd, dirs):
	if not dirs:
		dname = os.path.abspath(os.path.curdir)
	else:
		dname, = dirs

	try:
		primer = optd['P']		
	except:
		primer = ''
	try:
		platename = optd['p']
	except:
		platename = os.path.basename(os.path.abspath(os.path.curdir))
	try:
		outdir = optd['o']
	except:
		outdir = os.path.curdir

	seqs = get_seqs2(dname)
	
	create_summary(plate=platename, seqd={primer:seqs}, outdir=outdir)

	if not optd.has_key('z'):
		return
	# create zip file for all phd.1 and .seq file
	zipfile = "%s_txt.zip" % platename
	scorefile = "%s_qscore.txt" % platename
	cmd = "zip -rq %s *.seq *.phd.1" % zipfile
	os.system(cmd)

	print """
	======================================
	qscore file: %s  
	phd.1 and seq zip file: %s
	======================================
	""" % (scorefile, zipfile)
	
 
	
def do(args):
	usage = """
	EMD_report.py [options] dir1 dir2 
		-m <d1, d2,,,> merge data from multiple directories
		-f failed PCR file
		-p platename
		-o output_directory
		-r 0|1 
		-U <analysis_directory> create unique sequence fasta (light, heavy, heavy++light)
		-G <unique1> < unique2> group my unique seqs with ref seqs
		-HL <heavylight.fas> do Heavy.Light grouping using 
		-O oldpattern to be replaced
		-N newpattern in the replacement
		-F file contains plates to be merged
		-P primer name to used
		-S: generate qscore.txt for all *ab1 within current directory
		-Q: create phd.1 files for all ab1 files withi current directory
		-C <dir1> <d2> <d3> ... combine data in multiple directories, don't use, retired
		-g <dname> do report on a directory (group) of data (both light and heavy chain) 
		-z: create zip file for phd.1 and seq
	e.g EMDreport.py -p 34AR50 34AR50_5440S 34AR50_5442A
	""" 
	optd, dirs = util.parse_args(args, 'o:r:f:p:mVGUSQ:n:CO:N:F:P:vg:z', usage)
	merge = outdir = gfile = change = ''
	pcrfailedfile= plate =''
	alignment = do_anqi = 0
	print "options:",optd
	print "args:", args

	global VERBOSE
	VERBOSE=0
	combine = 0
	do_RD1=0
	for o, v in optd.items():
		if o == 'S':
			return do_qscore(optd, dirs)

		if o == 'g':
			return do_group(v, dirs)

		if o == 'C':
			outdir = 'report_all'
			if optd.has_key('o'):
				outdir = optd['o']
			platename = 'combined'
			if optd.has_key('p'):
				platename = optd['p']
			return combine_data(outdir, platename, dirs)

		if o == 'U':
			return create_unique_lib(v) # pass in analysis directory, use _unique fasta file
		if o == 'G':
			return group_with_reference(dirs) # my unique_file, ref_file
		elif o == 'R':
			do_RD1=1
		elif o == 'C':
			combine = 1
		elif o == 'p':
			plate = v
		elif o == 'V':
			VERBOSE = 1
		elif o == 'm':
			try:
				platefile = optd['F']
				plates = [x.strip() for x in open(platefile).readlines()]
				plates = [x for x in plates if x]
				print "Plates: %s" % plates
			except:
				plates = dirs 
	
			try:
				outdir = optd['o']
			except:
				outdir = ''
			return merge_data(plates, moutdir=outdir)
	
		elif o == 'v':
			DEBUG=1

		elif o == 'f':
			pcrfailedfile = v 
			print v
		elif o == 'q':
			do_anqi = 1
		elif o == 'o':
			outdir = v
			print "GETO", v
		elif o == 'r':
			try:
				change = int(v)
			except:
				change = 0
		
			try:
				format = optd['n']
			except:
				format = 'name_well_primer'

			try:
				primer = optd['P']
			except:
				primer = ''

			old=new=''
			if optd.has_key('O'):
				old = optd['O']
			if optd.has_key('N'):
				new = optd['N']

			return rename(change=change, format=format, oldpat=old, newpat=new, primer=primer)

		elif o == 'M':
			alignment=1
		elif o == 'HL':
			heavy_ligth_group(v)	

	if not dirs:
		print usage
		sys.exit(1)

	print "output dir:%s" % outdir, "data dirs", dirs

	do_plate(plate, dirs, outdir, alignment, pcrfailedfile, anqi=do_anqi, combine=combine, do_RD1=do_RD1)

def do_plate_old(plate, dirs, outdir, alignment='', pcrfile='', anqi='', combine='', do_RD1=0):

	groups = []

	seqd = {}
	for d in dirs:
		primer = re.findall('5440S',d)
		if not primer:
			primer = re.findall('5442A', d)
		if not primer:
			primer = re.findall('5455S',d)
		if not primer:
			primer = re.findall('544SX2',d)
		if not primer:
			primer = re.findall('544sx2',d)
		try:
			primer, = primer
		except:
			primer = ''
		print '---=> processing data in %s, primer = %s' % (d, primer)
		aafile, well2group,seqs = process_dir(d, primer, pcrfile, outdir=outdir, alignment=alignment, anqi=anqi, do_RD1=do_RD1)
		seqd[primer] = seqs

		groups.append((primer, well2group))


	if do_RD1:
		return

	print "PRIMERS USED:", seqd.keys()

	create_summary(plate, seqd, outdir )

	print "OUTDIR", outdir
	create_unique_lib(outdir)

def merge_data(dirs, moutdir=''):
	if not moutdir:
		moutdir = 'mergeddata_analysis' 

	# dir is platename, e.g. 34FS34
	plates = dirs
	#if not dirs:
	#	dirs = [os.path.isdir(x) for x in glob.glob('*')]

	plate2seqd = {}
	for plate in plates:
		print "==>processing plate: %s" % plate
		subdirs = [x for x in glob.glob('*%s*' % plate) if os.path.isdir(x)]
		datadirs = []
		for d2 in subdirs:
			if not os.path.isdir(d2):
				continue
			if re.search('analysis$', d2):
				continue

			m = re.search('54\d\d?[SA]', d2)
			primer = ''
			if m:
				primer = d2[m.start():m.end()]
			else:
				m = re.search('544SX2', d2)
				if m:
					primer = d2[m.start():m.end()]
	
			if not m:
				continue
			else:
				datadirs.append(d2)
			continue

			#outdir = 'analysis_%s' % plate
			#print '\tDIR: %s,  primer:%s, outdir:%s' % ( d2, primer, outdir)

			seqs = []
			aafile, well2group,seqs = process_dir(d2, primer, outdir=moutdir)

			primer = primer.upper()

			seqd.setdefault(primer_d[primer], [])

			if not seqs:
				continue

			seqd[primer_d[primer]].extend(seqs)

		if DEBUG:
			print ".. set key PLATE:", plate
		if not datadirs:
			continue
		plate2seqd[plate] = do_plate(plate, datadirs, outdir="analysis_%s" % plate)

	print "ALLPLATES:", plate2seqd.keys()

	anqi_grouping(plates, plate2seqd, outdir=moutdir)

	# to remove later
	allseqd = {}
	for p, seqd in plate2seqd.items():
		for primer, seqs in seqd.items():
			allseqd.setdefault(primer, [])
			allseqd[primer].extend(seqs)

	create_summary('function-score', allseqd, outdir=moutdir) 
	#create_unique_lib(moutdir)
	# create uniq library
	allseqfile = os.path.join(moutdir, 'allseqs.fas')
	allseqs = Fasta.all(allseqfile)
	d = {}
	for s in allseqs:
		d.setdefault(s.sequence, []).append(s.name)

	groups = d.items()
	groups.sort(lambda x,y: cmp(len(y[1]), len(x[1])))

def create_summary(plate, seqd, outdir=''):

	if not os.path.isdir(outdir):
		os.mkdir(outdir)
	outf = os.path.join(outdir, '%s_qscore.txt' % plate)
	print "Create summary, outfile is:", outf
	fout = open(outf, 'w')
	
	summary_d = {}
	# key is well, value is result tag for both primers
	primers = []
	count_d = {}

	failedpcr = 0
	for primer, seqs in seqd.items():
		print primer, "seq count:", len(seqs)
		primers.append(primer)
		for s in seqs:
			#print s.sample_file_name, s.sample_score, s.trimscore, s.category, s.msg, s.cat
			s.category = s.cat
			# remove primer tag from sample name
			name = s.basename[:-1]
			summary_d.setdefault(name, {})

			if s.trimscore:
				score = s.trimscore
			else:
				score = s.sample_score
			summary_d[name][primer] = (s.category, score)

			count_d.setdefault(primer, {})
			category = s.category
			if re.match('SS=', category):
				category = 'SS'
					
			count_d[primer].setdefault(category,0)
			count_d[primer][category] += 1

	print "COUNT D:"
	#pprint.pprint(count_d)
	header = ['Well']
	for p in primers:
		header.append(p)
		header.append('%s Sample Score' % p)

		cats = count_d[p]
		print "--->", p, ": %s category" % len(cats) 

	print >>fout, "\t".join(header)	

	both_are_functional = [] 
	notmix_cnt = 0

	names = summary_d.keys()
	names.sort()

	for name in names:
		data = ["%s" % name]
		
		fcnt = 0
		ismix = 0
		for p in primers:
			try:
				a =summary_d[name][p]
			except:
				#pprint.pprint(summary_d[name])
				print "ERROR: failed to find summary data for %s primer %s" % (name, p)
				sys.exit()

			data.extend(a)
			if a[0] == 'F':
				fcnt += 1
			if a[0].find('mix') != -1:
				ismix = 1

		if not ismix:
			notmix_cnt += 1

		if fcnt == 2:
			both_are_functional.append(name) 

		print >>fout, "\t".join([str(x) for x in data])
			

	
	categories = []
	for p in primers:
		categories.extend(count_d[p].keys())

	cats = dict([(c,1) for c in categories]).keys()
	cats.sort()
	cats2 = [x for x in cats if x]
	if not cats2:
		return
	print "2:", cats
	termd ={'F': "Functional VH/VL",
		'NF': "Non-functional",
		'F-mix': "Functional,mix",
		'NF-mix': "Non-Functional,mix",
		'F-fs': "Functional,frameshift",
		'NF-fs': "Non-Functional,frameshift",
		'F-Tc': "Functional,truncated",
		'NF-Tc': "Non-Functional,truncated",
		'Tc': "Truncation",
		'fs': "Frameshift",
		'SS': "No sequence" }
	print >>fout, "\n"
	for k, v in termd.items():
		print >>fout, "%s\t%s" % (k, v)
	fout.close()

	# summary table 
	outf = os.path.join(outdir, '%s_summary.txt' % plate)
	fout = open(outf, 'w')

	headers = ['']
	totals = ['Total']
	symbols = ['']
		
	for p in primers:
		headers.append(p)
		headers.append('')
		symbols += ['#', '%', '#', '%']
		totals += [len(seqd[p]), '100%']

	print >>fout, "\t".join(headers)
	print >>fout, "\t".join(symbols) #['', '#', '%', '#', '%'])
	print >>fout, "\t".join([str(x) for x in totals]) #['Total', str(total), '100%', str(total), '100%'])

	totalmix_by_primer = {} 
	print "Category:", cats

	for c in cats:
		if termd.has_key(c):
			data = [ termd[c]]
		else:
			data = [c]

		ismix = 0
		for p in primers:
			total = len(seqd[primer])
			if count_d[p].has_key(c):
				x = count_d[p][c]
				data += [x, round(x*100.0/total, 2)] 

				if c.find('-mix') != -1:
					totalmix_by_primer.setdefault(p,0)
					totalmix_by_primer[p] += x
					ismix = 1
			else:
				data += ['','']

		#print '\t'.join([str(x) for x in data]) 
		print >>fout, '\t'.join([str(x) for x in data]) 

	#print totalmix_by_primer
	t = ['Total mix']
	for p in primers:
		try:
			t.append(totalmix_by_primer[p])
		except:
			continue

		t.append(round(totalmix_by_primer[p]*100.0/total, 2) )
	print >>fout, "\t".join([str(x) for x in t])

	print >>fout, '\n'

	fcnt = len(both_are_functional)
	t = ["Functional Fab/Not-mix:", "%s/%s" % (fcnt, notmix_cnt), round(fcnt*100.0/notmix_cnt, 2)]
	print >>fout, '\t'.join([str(x) for x in t])
	t = ["Functional Fab/All:", "%s/%s" % (fcnt, total), round(fcnt*100.0/total, 2), ",".join(both_are_functional) ]
	print >>fout, '\t'.join([str(x) for x in t])

	# seqd={primer:seqs}

def srename(plate, primer="H", change=0):
	
    for ext in [ ".ab1", '.seq', '.phd.1']:
	files = glob.glob("*%s" % ext)
	for f in files:
		b = f.replace(ext, '')
		n = "%s%s%s%s" % (plate, b, primer, ext)
		print f,n 
		if change:
			os.rename(f, n)

def rename2(mapfile, change=0):
	lines = open(mapfile).readlines()
	d = {}
	for l in lines:
		cols = [x.strip() for x in l.split('\t')]
		well, name = cols[:2]
		d[well] = name

	files = glob.glob("*.seq") + glob.glob("*.ab1")
	for f in files:
		ext = f.split('.')[-1]
		well = f.split('_')[-2]
		name = d[well]
		f2 = "%s.%s" % (name, ext)
		print f, f2

		if change:
			os.rename(f, f2)

def doall():
	for plate in ['Fab410P1', 'Fab410P2', 'Fab410P3', 'Fab410P4']:
		d1 = '%s_5440S' % plate; d2='%s_5442A' % plate
		do_plate(plate, [d1,d2], alignment=1, outdir='')

def report_RD1(dname, seqs,align):
	tag1 = [('ATGGC[GATC]GA[GATC]', 12), ('GGCGGATAT', 10 ),('AA[CT]GT[GATC]CA[AG]', 0)] # MAD, NVQ
	tag2   = [('TA[TC]GC[GATC]TA[TC]',0),
		('GA[AG]((TC[TAGC])|(AG[TC]))GA[AG]',0),
		('AA[AG]GA[TC]TA[TC]', 0)] # YAY, ESE, KDY
	tag3   =[
		('GA[TC]((CT[GATC])|(TT[AG]))GG[GATC]', 0), # DLG
		('AT[ATCG]GC[GATC]((CG[GATC])|(AG[AG]))',0),# TAR
		('AA[AG]GA[AG]GC[GATC]GA[GA]', 0) # KEAE
		]
	tag4   =[
		('GG[GATC]GA[TC]AC[GATC]GT[GATC]', 0) #GDTV
		,('GA[GA]GG[GATC]CA[AG]((CT[AGTC])|(TT[GA]))', -12) # EGQL
		,('TGCGGCCGC', -47) # NotI
		]
	'''
		('AT[ATC]AA[TC]GT[TCGA]', 9), # TWT
		('GT[GATC]GA[AG]GG[GATC]', -30)] # TWT, GDT , TVT, VEG
	'''

	rdir = "report_%s" % dname
	if not os.path.isdir(rdir):
		os.mkdir(rdir)

	aafile = os.path.join(rdir, 'aa_%s.fas' % dname)
	nafile = os.path.join(rdir, 'nt_%s.fas' % dname)
	logfile = os.path.join(rdir, 'log_%s.txt' % dname)

	faa = open(aafile , 'w')
	fna = open(nafile , 'w')
	flog = open(logfile , 'w')

	print >>flog, "Report created at", time.ctime()
	print >>flog, '------------------------------------------=='

	loopseq = fscnt = 0
	for s in seqs:
		print s.basename, s.sample_score, s.ntinfo
		dna = "".join(s.bases)
		indexes = []

		for loop_tags in [tag1, tag2, tag3, tag4]:
			x = 0
			for (ls, offset) in loop_tags:
				m=re.search(ls, dna)
				if m:
					x = m.start() + offset
					break

			indexes.append(x)
		miss = indexes.count(0)
		print indexes

		if miss > 0:
			print '... miss tag', indexes
			print >>flog, '%s: score=%s, missing tag' % (s.basename, s.sample_score)
		else:	
			loopseq +=1

			aaseq=''
			ntseq = ''
			last = 0
			mes = ''
			for i in range(0, len(indexes)-1):
				cs=indexes[i]; ce = indexes[i+1]
				d1 = dna[cs:ce]; 
				aa1= DNA.DNA(d1).translate(); 
				print cs, ce, aa1
				if (ce-cs+1) % 3 == 0:
					mes += ', frameshift in loop %s' % (i+1)
					aa1 += '#'
				elif aa1.find('*') != -1:
					mes += ', frameshift in loop %s' % (i+1)

				#print 'Loop: %s..%s' % (cs, ce), aa2, aa1
				aaseq += aa1
				ntseq += d1

			if mes:
				fscnt += 1

			print >>flog, "%s: score=%s, start=%s, end=%s%s" % (s.basename, s.sample_score, indexes[0],indexes[-1], mes)
			print >>faa, '>%s' % s.basename
			print >>faa, '%s//' % aaseq
			if s.aax:
				print >>faa, '>%s_x' % s.basename
				print >>faa, s.aax

			print >>fna, '>%s' % s.basename
			print >>fna, '%s//' % ntseq
	
	print >>flog, '------------------------------------------=='
	print >>flog, 'processed %s sequences: %s have 3 loops, among them, %s have frameshift' % (len(seqs), loopseq, fscnt)

	print >>flog, '------------------------------------------=='
	faa.close()
	fna.close()

	if align:
		muscle(aafile)
		muscle(nafile)

	return ('','','')


def create_unique_lib(d=''):
	myd = {}
	if not d:
		d = glob.glob('*_analysis')
	files = glob.glob(os.path.join(d, 'aa_trim_*.fas')) 
	print "Fasta File counts:", len(files)

	for f in files:
		print '... processing %s' % f
		seqs = Fasta.all(f)

		for x in seqs:
			n = x.name; s = x.sequence
			myd.setdefault(n,s)

	seqd = {}
	for n, s in myd.items():
		seqd.setdefault(s, []).append(n)

	mynames = myd.keys()

	hd = {}
	ld = {}
	plate = os.path.basename(d).replace('-analysis', '').replace('_analysis', '')
	print "Plate:", plate

	for s, names in seqd.items():
		names2 = []
		for n in names:
			names2.extend(n.split(','))
		#print "NAMES:", names
		#print "NAMES2:", names2
		names = [re.sub('\(\d+\)$', '', x) for x in names2]
		named = dict([(x,1) for x in names])			
		names = named.keys()
		for n in names:
			if n.endswith('H'):
				hd[n[:-1]] = s
			if n.endswith('L'):
				ld[n[:-1]] = s

	heavy_light_group(d, plate, hd, ld)

def heavy_light_group(dname, platename, hd={}, ld={}):
	print "\tdname:", dname
	print "\tplatename:", platename

	myd = {}
	heavy_cnt_d = {}

	print "heavy chain count:", len(hd.keys())
	print "light chain count:", len(ld.keys())

	for name, hseq in hd.items():
		if not ld.has_key(name):
			continue

		heavy = hseq;
		light = ld[name]

		# do not group non-functional group
		skip=0
		for x in [heavy, light]:
			if re.search('[\*\#]', x) or x == '//' or  len(x) < 90:
				skip=1
				break;
		if skip:
			continue

		myd.setdefault(heavy, {})
		myd[heavy].setdefault(light, []).append(name)
		heavy_cnt_d.setdefault(heavy, 0)
		heavy_cnt_d[heavy] += 1

	# write grouping and sequence 
	fname = os.path.join(dname,'%s_HL_groups.txt' % platename)
	print"OUTFILE:", fname

	fout = open(fname , 'w')
	print >>fout, "Report created at", time.ctime()
	print >>fout, "---------------------------------------"
	faa = open(os.path.join(dname, '%s_HL_uniqueseq.fas' % platename), 'w')

	heavy_groups  = [(v,k) for (k,v) in heavy_cnt_d.items()]
	heavy_groups.sort()
	heavy_groups.reverse()

	i = 1
	for (hcnt, hseq) in heavy_groups:
		ld = myd[hseq]
		hseq = re.sub('\/\/$', '', hseq)

		if not hseq:
			continue

		allnames = []
		for lseq, names in ld.items():
			allnames.extend(names)

		allnames.sort()
		allnames[0] = "%s(%s)" % (allnames[0], len(allnames))
		tmp = []
		for x in (0, len(allnames), 10):
			tmp.append(",".join(allnames[x:x+10]))

		print >>fout, "\ngroup H%s has %s" % (i, hcnt)
		print >>fout, "------------------"

		lseqs = ld.items()
		lseqs.sort(lambda x,y: cmp(len(y[1]), len(x[1])))

		j = 1
		for lseq, anames in lseqs:
			if lseq == "//":
				continue
			anames.sort()
			# special case, for YD1 plate, put YD1 at the begining
			n1 = []; n2=[]
			for n in anames:
				if re.match('YD1', n):
					n1.append(n)
				else:
					n2.append(n)
			anames = n1+n2

			print >>fout, "  group H%s.L%s has %s: %s" % (i, j, len(anames), ",".join(anames))
			print >>fout, "    seq:", "%s++%s" % (hseq, lseq)
			x=anames
			x[0] = x[0] + "(%s)" % len(x)
			print >>faa, '>%s' % ",".join(x)
			print >>faa, '%s++%s' % (hseq, lseq)
			j += 1
		i += 1
	fout.close()

def group_with_reference(files):
	myseqs = Fasta.all(files[0])

	refseqs = []
	for f in files[1:]:
		refseqs.extend(Fasta.all(f) )
	print "grouping", files

	# get reference sequences
	rseqd = {}
	refseq_names = []
	for (n,s) in refseqs:
		names = [re.sub('\(\d+\)$', '', x) for x in n.split(',')]
		rseqd.setdefault(s,[])
		rseqd[s].extend(names)
		refseq_names += names

	# extract plate name from seq name
	ref2plate = {}
	plates = []
	for name in refseq_names:
		if re.search('[HL]$', name):
			platename = re.sub('[A-H]\d\d?[HL]$', '', name)
			if platename == name:
				platename = re.sub('[A-H]\d\d?\-\d[HL]$', '', name)
		else:
			platename = re.sub('[A-H]\d\d?\-?\d*', '', name)

		ref2plate[name] = platename
		if platename not in plates:
			plates.append(platename)
		#print "samplename=%s, plate=%s" % (name, platename)

	print "REF plate:", plates
	print "REF seqs:" , refseq_names

	# get my sequences
	mseqd = {}
	for (n,s) in myseqs:
		names = [re.sub('\(\d+\)$', '', x) for x in n.split(',')]
		mseqd.setdefault(s,[])
		mseqd[s].extend(names)

	b1 = os.path.basename(files[0]).split('.')[0].replace('_unique','')
	b1 = re.sub('aa_trim[\-_]', '', b1)
	b2 = os.path.basename(files[1]).split('.')[0].replace('_unique','')
	b2 = re.sub('aa_trim[\-_]', '', b2)

	# build table like the following
	'''
	REF-P1	REF-P2	REF-P3
myseq1	A1
myseq2	C3
myseq3
	'''

	faa = open('group_%s_with_%s.fas' % (b1, b2), 'w')
	fout = open('group_%s_with_%s.txt' % (b1, b2), 'w')

	# sort my sequences by names
	myseqs = []
	for seq, names in mseqd.items():
		names.sort()
		myseqs.append([seq, names])
	myseqs.sort(lambda x,y: cmp(x[1], y[1]))

	headers = ['']+ plates
	print >>fout, "\t".join(headers)

	for (myseq, mynames) in myseqs:
		#mynames[0] = mynames[0] + '(%s)' % len(mynames)
		myname = mynames[0]
		try:
			refnames = rseqd[myseq]
		except:
			refnames = []

		my_refplates = ['' for x in plates]
		my_refwells=['' for x in plates]

		refnames.sort()
		for rname in refnames:
			plate = ref2plate[rname]
			plateidx = plates.index(plate)
			my_refplates[plateidx] = rname
			my_refwells[plateidx] = re.sub('[HL$]', '', rname.replace(plate, ''))

		if refnames:
			defline = my_refplates + mynames
		else:
			defline = mynames
		defline[0] = defline[0] + "(%s)" % len(defline)
		print "DEF:", defline
		print >>faa, ">%s" % ",".join(defline)
		print >>faa, myseq

		# convert ref sample name to well
		row = ["%s(%s)" % (myname, len(mynames))] + my_refwells
		print "\t".join(row)
		print >>fout, "\t".join(row)
			
	faa.close()
	fout.close()

#----------------------------------------------------------------------------------------------
# New implementation
#----------------------------------------------------------------------------------------------
def do_annotation(primer2seqs):
	for primer, seqs in primer2seqs.items():
		print "   A: group data for primer %s" % primer

		# assign data category
		seqs2 = []
		for s in seqs:
			#print "-->", s.sample_file_name, type(s.sample_score), s.sample_score
			if s.fail():
				s.msg.append("SS=%s" % s.sample_score)
				s.cat = "failed"
				seqs2.append(s)
				continue
			if not s.aa:
				s.cat = "NF"
				if not s.trimstart:
					s.msg.append('No START-TAG')
				if not s.trimend:
					s.msg.append('No END-TAG')
				
				seqs2.append(s)
				continue
			
			if re.search('[\*]', s.aa):
				if len(s.aa) < 90:
					s.msg.append( 'deletion')
					s.cat = 'NF-Tc'
				else:
					s.cat = 'NF-fs'
					s.msg.append('has internal stop codon')
				seqs2.append(s)
				continue
			elif re.search('[\#]', s.aa):
				s.cat = 'NF-fs'
				s.msg.append('frameshift')
				seqs2.append(s)
				continue
			else:
				if len(s.aa) < 90:
					s.msg.append( 'deletion')
					s.cat = "F-Tc"
				else:
					s.cat = 'F'
				if s.sample_score < 30:
					s.msg.append( "low quality sequence")

			if s.trimscore > 30 and s.cat == 'F':
				seqs2.append(s)
				continue

			# is this a mixed clone?
			# search for region with low score
			hc_90 = 'TGGCCCCAG'
			lc_90 = 'CTCACAGTGC'
			sc_90 = 'GCGCGCACAG'
			if re.match('5442a', primer, re.IGNORECASE):
				tag90 = hc_90
			else:
				tag90 = lc_90

			dna = "".join(s.bases).upper()

			s1 = dna.find(tag90)
			taglen = len(tag90)
			
			if s1 != -1:
				ascore1 = sum(s.scores[s1:s1+taglen])/taglen
				s1 = s1+50
				ascore2 = sum(s.scores[s1:s1+taglen])/taglen
				s1 = s1+50
				ascore3 = sum(s.scores[s1:s1+taglen])/taglen
				ascore2 = (ascore2+ascore3)/2
			else:
				ascore1 = sum(s.scores[90:100])/10
				ascore2 = sum(s.scores[190:200])/10

			if ascore2 <= ascore1 -20:
				s.cat = s.cat + '-mix'

			#print "XXX:", s.sample_file_name, s.sample_score, s.cat, ascore1, ascore2
			seqs2.append(s)
			primer2seqs[primer] = seqs2
	return primer2seqs


def _translate(dna):
	# try to guess translation, there may be frameshift 
	if not dna:
		return ""

	aa = DNA.DNA(dna).translate()
	yycpat = 'YYC'
	yyc = re.search(yycpat, aa)
	if not yyc:
		for i in [ 1, 2]:
			seq = dna[i:]
			aa2 = DNA.DNA(seq).translate()
			# look for required aa YYC 
			yyc = re.search(yycpat, aa2)
			if yyc:
				s = yyc.start()
				# introduce frameshift notation
				aa = aa[:s] + "#" + aa2[s:]
				break
	return (yyc, aa)

def _trim(start_tags, end_tags, seq):
	start = end = 0
	for saatag, snatag, offset in start_tags:
		start = _find_start(seq, snatag)
		if start:
			start += offset
			break
	for eaatag, enttag, offset in end_tags:
		end = _find_end(seq, enttag)
		#print eaatag, enttag , end
		if end:
			end += offset
			break
	return (start, end)
	

def do_trimseq(primer, seqs):
	rc = 0

	if re.match('QB1564', primer, re.IGNORECASE):
		start_tags = [('EVQL', 'GA[GA]GT[GATC]CA[AG]((CT[GATC])|(TT[AG]))', 0 ), 
		('VQLL', 'GT[GATC]CA[AG]((CT[GATC])|(TA[AG]))((CT[GATC])|(TA[AG]))', -3 ),
		('AMAE', 'GC[GATC]ATGGC[GATC]GA[GA]', 9)]

		end_tags = [ ('TVSS', 'AC[GATC]GT[GATC]((TC[GATC])|(AG[TC]))((TC[GATC])|(AG[TC]))', 0),
		('ASTKG', 'GC[GATC]((TC[GATC])|(AG[TC]))AC[GATC]AA[AG]GG[GATC]', -15),
		('STKG', '((TC[GATC])|(AG[TC]))AC[GATC]AA[AG]GG[GATC]', -12),
		('FPLA', 'TT[TC]CC[GATC]((CT[GATC])|(TT[AG]))GC[GATC]', 8*3),
		]
		rc = 0
	if re.match('5442A', primer, re.IGNORECASE):
		start_tags = [('EVQL', 'GA[GA]GT[GATC]CA[AG]((CT[GATC])|(TT[AG]))', 0 ), 
		('VQLL', 'GT[GATC]CA[AG]((CT[GATC])|(TA[AG]))((CT[GATC])|(TA[AG]))', -3 ),
		('AMAE', 'GC[GATC]ATGGC[GATC]GA[GA]', 9)]

		end_tags = [ ('TVSS', 'AC[GATC]GT[GATC]((TC[GATC])|(AG[TC]))((TC[GATC])|(AG[TC]))', 0),
		('ASTKG', 'GC[GATC]((TC[GATC])|(AG[TC]))AC[GATC]AA[AG]GG[GATC]', -15),
		('STKG', '((TC[GATC])|(AG[TC]))AC[GATC]AA[AG]GG[GATC]', -12),
		('FPLA', 'TT[TC]CC[GATC]((CT[GATC])|(TT[AG]))GC[GATC]', 8*3),
		]

		rc = 1
	elif re.match('5440S', primer, re.IGNORECASE) or re.match('544SX2', primer, re.IGNORECASE):
		start_tags = [ ('S[HY]SAQD', 'TCT[TC]A[CT]AGTGCACA[AG]GA[TC]', 15),  
		 ('S[HY]SAQ',  'TCT[TC]A[CT]AGTGCACA[AG]', 12),  
		('DRVT', 'GA[TC]((CG[CGAT])|(AG[AG]))', 0)
		]
		end_tags = [('GQPK', 'GG[GATC]CA[AG]CC[GATC]AA[AG]', -12), #-12
		('QPKA', 'CA[AG]CC[GATC]AA[AG]GC[GATC]', -15), # lambda vector
		('RTVA', 'CGAAC[AGTC]GT[GATC]GC[GATC]', 0), # kappa vector
		('TVA', 'AC[AGTC]GT[GATC]GC[GATC]', 0)] # kappa vector 

	elif re.match('5455S', primer, re.IGNORECASE):
		start_tags = [	('HSAQD', 'CA[TC]((TC[GATC])|(AG[TC]))GC[GATC]CA[AG]GA[TC]', 12),
	 	('[HY]SAQ',  'TCT[TC]A[CT]AGTGCACA[AG]', 0),
	 	('AHSA',  'GC[GATC]CA[TC]((TC[GATC])|(AG[TC]))GC[GATC]', 12)
	 	]
		end_tags = [('GQPK', 'GG[GATC]CA[AG]CC[GATC]AA[AG]', -12), #-12
		('QPKA', 'CA[AG]CC[GATC]AA[AG]GC[GATC]', -15), # lambda vector
		('RTVA', 'CGAAC[AGTC]GT[GATC]GC[GATC]', 0), # kappa vector
		('TVA', 'AC[AGTC]GT[GATC]GC[GATC]', 0)] # kappa vector 

	#print "XXX:", end_tags
	for s in seqs:
		s.rc = rc
		myseq = "".join(s.bases)
		#print "BEFORE TRIM: len:", len(s.bases)
		#print myseq
		myscores = []
		for i in range(0, len(s.scores)):
			myscores.append(s.scores[i])
		if rc:
			myseq = DNA.DNA(myseq).reverse_complement()
			myscores.reverse()
		start, end = _trim(start_tags, end_tags, myseq)

		if start and end:
			s.trimstart = start
			s.trimsend = end

		start, end = _trim(start_tags, end_tags, myseq)

		s.trimstart = start
		s.trimend = end

		#print s.basename, "start=%s, end=%s, score=%s, seqlen=%s" % (start, end, s.sample_score, len(s.bases))
		if start and end and start<end:
			s.trimseq = myseq[start:end]
			s.trimscore = sum(myscores[start:end])/(end-start+1)
			yyc, s.aa = _translate(s.trimseq)
			#print ' ', s.basename, len(s.trimseq), yyc
			if not yyc:
				s.msg = ["No YYC"]
			
		#print s.basename,  s.trimstart, s.trimend, len(s.trimseq), s.sample_score, s.trimscore, len(s.bases)
		#print s.aa
	return seqs

			
# create report, write all files
def do_report(plate, primer2seqs):
	# create output directory
	outdir = "%s_analysis" % (plate)
	if not os.path.isdir(outdir):
		os.mkdir(outdir)

	sep1 = '++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++'
	sep2 = '------------------------------------------------------------------'

	hc_groups = []
	lc_groups = []
	hc_primer = lc_primer = ''

	# write details grouping log file
	for primer, seqs in primer2seqs.items():
		if primer in ['H', '5442A', '5442a']:
			hc_primer = primer
			chaintype = 'H'
		else:
			lc_primer = primer
			chaintype = 'L'

		goods = []
		fails = []
		fss = []
		mixs = []

		s = seqs[0]
		aafile = os.path.join(outdir, "aa_trim_%s.fas" % primer)
		faa = open(aafile, 'w')
	
		ntfile = os.path.join(outdir, "nt_trim_%s.fas" % primer)
		fnt = open(ntfile, 'w')

		name2seq = {}
		seq2names = {}
		for s in seqs:
			name2seq[s.basename] = s
			if s.cat == 'failed':
				fails.append(s.basename)

			elif s.cat == 'F':
				goods.append(s.basename)
				seq2names.setdefault(s.aa, []).append(s.basename)
				print >>faa, '>%s' % s.basename
				print >>faa, s.aa

				print >>fnt, '>%s' % s.basename
				print >>fnt, s.trimseq


			elif re.search('mix$', s.cat):
				mixs.append(s.basename)
			else:
				fss.append(s.basename)
		faa.close()
		fnt.close()

		# write unique sequence file
		uaafile = aafile.replace('.fas', '_unique.fas')
		fuaa = open(uaafile, 'w')
		untfile = ntfile.replace('.fas', '_unique.fas')	
		#funt = open(untfile, 'w')

		log = os.path.join(outdir, "grouping_%s_%s.txt" % (plate, primer))
		flog = open(log, 'w')

		print >>flog, "Report create at: %s" % time.ctime()
		print >>flog, sep1
		print >>flog, 'Sequenced %s samples' % len(seqs)
		print >>flog, sep1, '\n'

		print >>flog, '%s contain functional %s chain (%.01f%%)' % (len(goods), chaintype, len(goods)*100.0/len(seqs))
		print >>flog, sep2

		# what are the grouping information for this primer?
		groupcnt = len(seq2names.keys())
		seqcnt = len(seqs)

		print "   R: Found %s groups from %s sequences" % (groupcnt, len(goods) )
		print >>flog, "Found %s groups from %s sequences" % (groupcnt, len(goods)), '\n', sep2
		
		mygroups_by_cnt = {}
		for seq, names in seq2names.items():
			#print "1:", names, seq
			mygroups_by_cnt.setdefault(len(names), []).append(seq)

		# update log with grouping information for this primer
		# sort group by count, the most popular ones at top
		mygroup_cnts = mygroups_by_cnt.keys()
		mygroup_cnts.sort(lambda x,y: cmp(y,x))
	
		g = 1
		for groupcnt in mygroup_cnts:
			seqs_with_samecnt__names = []

			for seq in mygroups_by_cnt[groupcnt]:
				names = seq2names[seq]
				names.sort()
				seqs_with_samecnt__names.append([names, seq])
	
			seqs_with_samecnt__names.sort()

			for (names, seq) in seqs_with_samecnt__names:
				#print "XXX:", names, '--%s--' % seq

				if chaintype == 'H':
					hc_groups.append(names)
				elif chaintype == 'L':
					lc_groups.append(names)

				myseqcnt = len(names)

				names2 =[x for x in  names]
				names2[0] += "(%s)" % myseqcnt
				print >>flog, "group %s has %s: %s" % (g, myseqcnt, ",".join(names2))
				print >>flog, "seq: %s//" % seq
				for name in names:
					s1 = name2seq[name]
					print >>flog, "name: %s, sample score = %s, average trimseq score = %s " % (name, s1.sample_score, s1.trimscore)
				print >>flog, sep2, '\n'
				g += 1

				#print >>funt, '>%s' % ",".join(names2)
				#print >>funt, seq

				print >>fuaa, '>%s' % ",".join(names2)
				print >>fuaa, seq
		#funt.close()
		fuaa.close()
	
		# bad sequences
		print >>flog, '\n', sep1
		print >>flog, "%s samples were used used in the grouping " % (seqcnt-len(goods))
		print >>flog, sep1

		print >>flog,  sep2	
		print >>flog, "%s have frameshift or deletion" % len(fss)
		print >>flog,  sep2	

		# group frameshift samples
		fsd = {}
		for name in fss:
			ss = name2seq[name]
			fsd.setdefault(ss.aa, []).append(name)
		fs_groups = [(names, seq) for (seq, names) in fsd.items()]
		fs_groups.sort(lambda x,y: cmp(len(y[0]), len(x[0])))

		for (names, seq) in fs_groups:
			print >>flog, 'Seq:', seq
			for name in names:
				ss = name2seq[name]
				tmp = ["%s: sample_score=%s" % (name, ss.sample_score)]
				if ss.trimscore:
					tmp.append("trimscore=%s" % ss.trimscore)

				tmp.extend(ss.msg)
				tmp.extend(['trimstart=%s' % ss.trimstart, 'trimend=%s' % ss.trimend])
				print >>flog, '  name:', ", ".join(tmp)
			print >>flog, '\n'
		print >>flog, '\n'

		print >>flog,  sep2	
		print >>flog, "%s are mixed " % len(mixs)
		print >>flog,  sep2	
		for name in mixs:
			ss = name2seq[name]
			print >>flog, "%s: sample_score=%s, trim_score=%s, %s, %s" %(name, ss.sample_score, ss.trimscore, s.cat, ",".join(s.msg))
		print >>flog, '\n'


		# write information about failed samples
		nopcr = []
		# write failed samples
		print >>flog, sep2
		print >>flog, "%s with poor data (%.01f%%): no PCR product(%d), failed sequencing (%d)" % (len(fails), len(fails)*100.0/seqcnt, len(nopcr), len(fails)-len(nopcr) )
		print >>flog, sep2
		print >>flog, "With PCR product but no sequence: %d" % (len(fails)-len(nopcr))
		for name in fails:
			s = name2seq[name]
			print >>flog, '  %s: seqlen=%s, score=%s ' % ( name, len(s.bases), s.sample_score)

		# check restrictionsite
		print " writing restriction enzyme information"
		print >>flog, '\n', sep1
		print >>flog, "Restriction Enzyme Cutting Sites"
		print >>flog, sep1

		if chaintype == 'H':
			sites = ENZ_5442A
		elif chaintype == 'L':
			sites = ENZ_5440S
		for s in seqs:
			dna = "".join(s.bases).upper()
			for (enzyme, cutsite) in sites:
				cutsite = cutsite.upper()

				start = 0; pos = []
				#print  enzyme, cutsite
				while 1:
					idx = dna.find(cutsite, start)
					if idx == -1:
						break
					start = idx + len(cutsite)

					x = s.strimstart
					y = s.trimend
					if not s.strimstart:
						x = 0
					if not s.trimend:
						y = len(s.bases)
					if s.trimstart and s.trimend:	
						x = min(s.trimstart, s.trimend)
						y = max(s.trimstart, s.trimend)

					if x <= start and start <= y:
						pos.append(idx+1)
				if pos:
					print >>flog, " %s: %s (%s) at %s" % (s.basename, enzyme, cutsite, ",".join([str(x) for x in pos]) )
		print >>flog, sep1
		flog.close()

	# combine heavy chain and light chain result, and do summary report
	hlgroup = os.path.join(outdir, '%s_HL_groups.txt' % plate)
	fhl = open(hlgroup, 'w')
	print >>fhl, "Report created at", time.ctime()
	print >>fhl, sep1

	hlseq = os.path.join(outdir, "%s_HL_uniqueseq.fas" % plate)
	fhlseq = open(hlseq, 'w')

	# get heavy chain sequence first
	hc_names = []
	for names in hc_groups:
		hc_names.extend(names)
	hc_d = {}
	print "HC_PRIMERS:", hc_primer
	if hc_primer and primer2seqs.has_key(hc_primer):
	    for s in primer2seqs[hc_primer]:
		if s.basename in hc_names:
			hc_d.setdefault(s.basename, s)

	lc_d = {}
	lc_names = []
	print "hc_groups", len(hc_groups)
	print "lc_groups", len(lc_groups)

	for names in lc_groups:
		lc_names.extend( names)
	if lc_primer:
	    for s in primer2seqs[lc_primer]:
		if s.basename in lc_names:
			lc_d.setdefault(s.basename, s)

	'''
	print "lc_names:" 
	print "\n".join(lc_names)
	print "hc_names:", "\n".join(hc_names)
	'''
	
	hc_groupcnt = 0
	for names in hc_groups:
		try:	
			hc_s = hc_d[names[0]]
		except:
			continue
		hc_seq = hc_s.aa
		if not hc_seq:
			continue

		hc_lc_d = {}
		for name in names:
			lcname = re.sub('H$', 'L', name)
			if not lc_d.has_key(lcname):
				continue

			lc_s = lc_d[lcname]
			if not lc_s.aa:
				continue

			hc_lc_d.setdefault('%s++%s' %(hc_seq, lc_s.aa), []).append(name)
	
		hc_lc_groups = [(names, seq) for (seq,names) in hc_lc_d.items()]
		hc_lc_groups.sort(lambda x,y: cmp(len(y[0]), len(x[0]) ) )
		lc_counts = sum([len(x[0]) for x in hc_lc_groups])

		if not lc_counts:
			continue

		hc_groupcnt += 1

		lc_groupcnt = 0
		print >>fhl, "Group H%s has %s" % (hc_groupcnt, lc_counts)
		print >>fhl, sep2

		for names, seq in hc_lc_groups:
			lc_groupcnt +=1
			names2 = [re.sub('[HL]$', '', x) for x in names]

			names2[0] += '(%s)' % len(names2)
			print >>fhl, '  group H%s.L%s has %s:' % (hc_groupcnt, lc_groupcnt, len(names)), ",".join(names2)
			print >>fhl, '    seq: %s//' % seq

			print >>fhlseq, '>%s' % ",".join(names2)
			print >>fhlseq, seq

	fhl.close()
	fhlseq.close()
	return outdir

def do_plate(plate, dirs, outdir='', alignment='', pcrfile='', anqi='', combine='', do_RD1=''):
	primer2seqs = {}

	for d in dirs:
		print "DATADIR:", d
		# get primer from directory name
		m = re.search('54\d\d[aAsS]', d)
		if not m:
			m = re.search('544SX2', d, re.IGNORECASE)
			if not m:
				m = re.search('QB1564', d, re.IGNORECASE)
				if not m:
					print "ERROR getting primer from directory name", d
					sys.exit()
		primer = d[m.start():m.end()]
		print "\tPrimer: ", primer
		
		seqs = get_seqs(primer, d)

		print "\tSeq count:", len(seqs)
		primer = primer.upper()
		primer2seqs.setdefault(primer.upper(), [])
		primer2seqs[primer] += seqs

	print "... done getting sequence"

	primer2seqs = do_annotation(primer2seqs)
	print "... done annotation", primer2seqs.keys()

	outdir = do_report(plate, primer2seqs)
	print "... done writing report"	

	# do summary
	#anqi_grouping(plate, primer2seqd, outdir)
	create_summary(plate, primer2seqs, outdir)
	return primer2seqs

# for 071809 data (5 plates: 33PR27-31)
def do_group(groupname, datadirs):
	primer2seqs = {}

	seqs = []
	for d in datadirs:
		#seqs += get_seqs1(d)
		seqs += get_seqs2(d)

	for s in seqs:
		# get primer from sample_file_name or basename
		name = s.sample_file_name
		if s.basename:
			name = s.basename

		if name.endswith('H'):
			primer = '5442A'
		else:
			primer = '5440S'
		primer2seqs.setdefault(primer, []).append(s)

	for primer, seqs in primer2seqs.items():
		primer2seqs[primer] = do_trimseq(primer, seqs)

	primer2seqs = do_annotation(primer2seqs)
	outdir = do_report(groupname,primer2seqs)
	create_summary(groupname, primer2seqs ,outdir)


def get_seqs(primer, dname):
	print "dname: %s, primer: %s "  % (dname, primer)
	#seqs = get_seqs1(dname)
	seqs = get_seqs2(dname)
	return do_trimseq(primer, seqs)

def get_seqs2(dname):
	phdfiles = glob.glob(os.path.join(dname, '*.phd.1'))	
	print "PHD file count:", len(phdfiles)
	seqs = []
	for phdfile in phdfiles:
		basename = os.path.basename(phdfile).replace('.phd.1', '' )
		ss = SampleScore.SampleScore(dict(sample_file_name=basename, sample_score=0, lor=0 ))
		lines = open(phdfile).readlines()
		
		# get name
		name = lines[0].split()[1]
		if name != basename:
			print "ERROR: sample name in phd.1 file (%s) differs from file name (%s), please redo basecall" % (name, basename)
			sys.exit()
		ss.msg = []
		ss.bases = []
		ss.scores = []
		ss.dname = dname
		cnt = 0
		for l in lines[18:]:
			if not re.match('[GATC]', l):
				continue
			if cnt >= 700 or re.match('END_DNA', l):
				break
			cnt += 1

			cols = l.split()
			base, score = cols[:2]
			score = eval(score)
			ss.bases.append(base); ss.scores.append(score)
		ss.lor = len(ss.scores)
		if ss.lor:
			ss.sample_score = int(sum(ss.scores)/ss.lor)
		seqs.append(ss)
	print "get_seqs2: count=%s" % len(seqs)

	return seqs

def get_seqs1(dname):
	# get summary information using ABI report file
	seqs = SampleScore.get_seqsamples(os.path.join(dname, 'qscore.txt'))
	print "SEQSAMPLE count:", len(seqs)

	# get sequence data
	for ss in seqs:
		phdfile = os.path.join(dname, '%s.phd.1' % ss.basename)
		#print phdfile

		lines = open(phdfile).readlines()[18:]

		ss.msg = []
		ss.bases = []
		ss.scores = []
		ss.dname = dname
		cnt = 0
		for l in lines:
			if not re.match('[GATC]', l):
				continue
			if cnt >= 700 or re.match('END_DNA', l):
				break
			cnt += 1

			cols = l.split()
			base, score = cols[:2]
			#print cols, "BASE:", base, score, cnt
			score = eval(score)
			ss.bases.append(base); ss.scores.append(score)
		#print "xxx:", ss.basename, cnt
		#sys.exit()
	return seqs

def get_score(datadirs):
	d = {}
	for  d in datadirs:
		primer = ''
		f = os.path.join(d, 'qscore.txt')
		if os.path.isfile(f):
			well2score = _get_score(f)
			d[primer] = well2score

def _get_score(score):
	lines = open(score).readlines()
	g = 0
	for l in lines:
		if re.match('Sample_File_Name', l):
			g = 1
			continue

# 11-24-2010
def do_all():
	plates = ['34FS109', '34FS110', '34FS111', '34FS112', '34FS113','34FS114', '34FS115', '34FS116', '34FS117', '34FS118']
	plates = ['34FS35', '34FS89', '34FS93', '34FS103', '34FS109', '34FS110', '34FS111', '34FS112', '34FS113','34FS114', '34FS115', '34FS116', '34FS117', '34FS118']
	#plates = ['34FS35', '34FS118']

	dirs = []
	#'2010-05-17_34FS89_544SX2', '2010-05-17_34FS89_5442A', '2010-02-12_34FS35_544SX2', '2010-02-12_34FS35_5442A']
	for p in plates:
		ds = [x for x in glob.glob('*%s*544*'% p) if os.path.isdir(x)]
		for d in ds:
			if re.search('analysis', d) or re.search('combine', d):
				continue
			dirs.append(d)
		
	plate = 'combined_%splates' % len(plates)
	print "Plate count: %s, datadirs count:%s" % (len(plates), len(dirs))
	do_plate(plate, dirs, outdir='', alignment='', pcrfile='', anqi='', combine='', do_RD1='')
	return

	ng_files = ['nogrouth_%s.txt' % p for p in plates]

	outdir = 'combined_%s_plates' % len(plates)
	if not os.path.isdir(outdir):
		os.mkdir(outdir)

	plated = {}
	for plate in plates:
		datadirs = [x for x in glob.glob('2010*%s*' % plate) if os.path.isdir(x)]
		print datadirs
		ngfile = 'nogrowth_%s.txt' % plate
		if not os.path.isfile(ngfile):
			ngfile = ''

		primerd = do_plate(plate, datadirs, outdir='analysis_%s' % plate, pcrfile=ngfile)
		plated.setdefault(plate, {})
		for p in primerd.keys():
			print "PRIMER: %s" % p 
			if re.match('5442', p, re.IGNORECASE):
				p2 = 'H'
			else:
				p2 = 'L'
			plated[plate].update({p2:primerd[p]} )

	# merge plates
	print "==> do anqi grouping"
	anqi_grouping(plates, plated, outdir=outdir)

	allseqd = {}
	print "==> group sequence"
	for p, seqd in plated.items():
		for primer, seqs in seqd.items():
			allseqd.setdefault(primer, [])
			allseqd[primer].extend(seqs)

	for p, seqs in allseqd.items():
		print "... ", p, len(seqs)
	create_summary('function-score', allseqd, outdir=outdir) 
	print "==> done creating summary\n"

	print "==> create uniq library"
	create_unique_lib(outdir)

if __name__ == '__main__':
	#do_all()
	do(sys.argv[1:])

