#!python

import os, sys, re, glob, pprint, getopt, shutil

from qblib.Plate import * 
from qblib.config import *
from qblib import process_dir, Fasta, DNA, align, BlastParser
import CloneAssembler, score_abi, rename
# -----------------------------------------------------------------------------------
def get_mutation_codon(file, LOG=None):
	print "  FILE:", file
	all = Fasta.all(file)

	ref = all[0]
	refname = ref.name
	refseq  = ref.sequence.upper()
	if not re.match('ref', refname):
		print "ERROR: first sequence is not reference"
		return
	mutations = []
	mindx = {}

	welld = {}
	for a in all[1:]:
		name = a.name
		cols = name.split(':')[0]
		m = re.search('[A-H]\d+$', cols[0])
		if not m:
			print "ERROR: sequence name does not contain well info:", name
			sys.exit()
		myseq = a.sequence
		mut = []	
		cnt = 0
		for i in range(0, len(refseq), 3):
			rcodon = refseq[i:i+3]
			mcodon = myseq[i:i+3]

			if rcodon != mcodon:
				j = i/3 + 1
				raa = DNA.DNA(rcodon).translate()
				maa = DNA.DNA(mcodon).translate()

				mut.append( [raa, j, maa , rcodon, mcodon] )
				mindx[i] = 1
		mutations.append([a.name, mut])			

	# write outfile, [TAG, mutation, ref codon, mut condon ...
	new_data = []
	for m in mutations:
		[name, mut] = m
		# set
		data = [name]
		mcnt = len(mut)
		tag = '' 
		if mcnt >= 3:
			tag = 'multiple mutations'
		elif mcnt == 0:
			tag = 'WT'
		else:
			wt=0
			for x in mut:
				raa, j, maa, rc, mc = x
				data += ['%s%03d%s' % (raa,j, maa),  rc, mc]
				raa,i, maa, rc, mc = x
				if raa == maa:
					wt += 1
			if wt == len(mut):
				tag = 'WT'
		data.insert(0, tag)
		new_data.append( "\t".join(data))

	return new_data

def get_wildtype_codon(outfile):
	plates = [x for x in glob.glob('*') if os.path.isdir(x) and not re.match('_', x) and not re.match('tmp_', x)]
	print plates
	
	F = open(outfile, 'w')	
	platecount = 0
	LOG = open('excluded_%s' % outfile, 'w') 
	for plate in plates:
		for root, dirs, files in os.walk(plate):
			dirs = [d for d in dirs if re.search('report_', d)]
			if not dirs:
				continue
			plate = os.path.basename(root)
			print "PLATE:", plate, dirs

			print >>LOG, "PLATE:", plate
			platecount += 1
			dirs.sort(lambda x,y: cmp(os.path.getmtime(os.path.join(root, x)), os.path.getmtime(os.path.join(root, y))))
			rdir = os.path.join(root, dirs[0])
			cds_file = os.path.join(rdir, '%s_cds.fas' % plate)
			if os.path.isfile(cds_file):
				new_data = get_mutation_codon(cds_file, LOG)
				
				print >>F, "\n".join(new_data)
				F.flush()
				LOG.flush()
	F.close()
	print "plate processed:", platecount
	"""

			fount = 0
			for f in files:
				if f == '%s_mutation_summary.txt' % plate:
					fount = 1
					print root, f
					break
			if fount:
				fullpath = os.path.join(root, f)
				lines = open(fullpath).readlines()
				wt_wells = []
				for l in lines:
					cols = l.split('\t')
					if cols[2] in ['WT', 'wt']:
						print "\t", l.strip()
						wt_wells.append(cols[0])

				if wt_wells:
					# get DNA sequence
					cdsfile = os.path.join(root, '%s_cds.fas' % plate)
					all = Fasta.all(cdsfile)
					r = all[0]
					for a in all[1:]:
						name = a.name.split(':')[0]
						if name in wt_wells:
							diffs = []
							for i in range(0, len(r.sequence), 3):
								rc = r.sequence[i:i+3].upper()
								mc = a.sequence[i:i+3].upper()
								if rc != mc:
									diffs.append([ rc,i,mc])
							if diffs:
								print "\t\t", a.name, diffs
				break
			"""


def unzip_files():
	cdir = os.path.abspath(os.path.curdir)
	for root, dirs, files in os.walk(os.path.abspath(os.path.curdir)):
		for f in files:
			if re.match('V36', f) and f.endswith('zip'):
				plate = f.replace('.zip', '')
				os.chdir(root)
				cmd = 'unzip -qfo %s -d %s' % (f, plate)
				print cmd
				os.system(cmd)
				os.chdir(cdir)


def list_plate(platename):
	for d in ['Q', 'P', 'M', 'T', 'V']:
		cmd = "ls -ld %s:\*%s*" % (d, platename)
		print "\n",d
		os.system(cmd)

def _update_with_to_repeat(platename, dirlist):
	# get to_repeat_ directory
	rdir = 'to_repeat_%s' % platename
	#412files = glob.glob(os.path.join(rdir, "*412.*"))
	#411files = glob.glob(os.path.join(rdir, "*411.*"))
	# to be implemented
	for d in dirlist:
		print "DIRNAME:", d
		if d.find('411') != -1:
			print ('mv -f %s %s' % (os.path.join(rdir, '*411.*'), d) )
			os.system('mv -f %s %s' % (os.path.join(rdir, '*411.*'), d) )
		elif d.find('412') != -1:
			print ('mv -f %s %s' % (os.path.join(rdir, '*412.*'), d) )
			os.system('mv -f %s %s' % (os.path.join(rdir, '*412.*'), d) )

def plate_report(platename, nt_refile, nt_name, cds_name, dirlist, optd):
	_update_with_to_repeat(platename, dirlist)

	# get assembly program
	assembly_exe = 'cap3'
	if optd.has_key('S'):
		assembly_exe = optd['S']
	tags = []
	if optd.has_key('T'):
		tags = optd['T'].split(',')
		
	print "REF:", nt_refile, nt_name, cds_name
	if not os.path.isfile(nt_refile):
		print "ERROR: invalid reference file '%s' " % nt_refile
		sys.exit()

	fprimer = '0411'; rprimer = '0414'; 
	min_distance=500; max_distance = 1500

	clean_start = 0
	if optd.has_key('-C'):
		clean_start = 1

	# get plate notes
	platenotes = {}
	if optd.has_key('n'):
		if os.path.isfile(optd['n']):
			N = open(optd['n'])
			lines = N.readlines()
			for l in lines:
				cols = l.strip().split('\t')
				p = cols[0]
				platenotes.setdefault(p, [])
				for c in cols[1:]:
					c2 = util.format_well(c.strip())
					platenotes[p].append(c2)
					#print p, c, c2

			N.close()

	myplate = FNAPlate(platename, fprimer, rprimer, min_distance, max_distance, nt_refile, clean_start=clean_start, col_order=1)
		
	print "\n===> NOW processing data as a plate:", myplate.name
	
	cds_align = aa_align = ''
	myplate.set_seqdata('_', dirlist)

	myplate.do_assembly(trimlength=0, prog=assembly_exe, trimtags = tags)
	
	if optd.has_key('X'):
		print "align to:", optd['X']
		ntaligns = myplate.do_align2(refile=nt_refile, refnames=optd['X'].split(','))
		for nt in ntaligns:
			if re.search('Promoter', nt):
				mout = os.path.join(myplate.report_dir, '%s-Promoter_nt_mutation_summary.txt' % myplate.name)
				CloneAssembler.report_mutation(mout, nt)
				
	else:
		# trim to reference sequence and do multiple sequence alignment
		cds_align, aa_align = myplate.do_align(refile=nt_refile, nt=nt_name, cds=cds_name, debug=1)
		print "cds_align:", cds_align
		print "aa_align", aa_align

	if not optd.has_key('M'):
		return None 
	
	# new implementation, used for V36
	print ">>FNAReport: do mutation report"
	notes = []
	if platenotes.has_key(platename):
		notes = platenotes[platename]
	print platename, notes
	(well_mutation_d, stat_d, repeat_file) = myplate.do_mutation_summary(well_without_growth=notes)

  	# copy all report files to a new directory that will be zipped 			
	x = time.localtime()
	(y, m, d) = x[:3]
	today = "%s-%s-%s" % (y, m, d)
	if optd.has_key('z'):
		if re.match('\d{4}\-\d\d', platename):
			n = re.sub('\d{4}\-\d\d\-\d\d', '', platename)
		else:
			n = platename
		rdir = "report_%s_%s" % (today, n)
		if not os.path.isdir(rdir):
			os.mkdir(rdir)

		# copy report files:
		for f in ['%s_aa.fas' % platename, 
				'%s_aa_align.html' % platename,
				'%s_cds.fas' % platename,
				'%s_cds_align.html' % platename,
				'%s_mutation_summary.txt' % platename]:
			f2 = os.path.join(myplate.report_dir, f)
			shutil.copy(f2, rdir)

		# create zip file 
		cmd = "zip.exe -rq %s.zip %s %s" % (platename, " ".join(dirlist), rdir)
		print cmd
		try:
			os.system('rm %s.zip' % platename)
			os.system(cmd)
		except:
			pass

	return (well_mutation_d, stat_d, repeat_file)

def combine_reports():
	# get all report files from the subdirectories of current directory

	resd = {}
	for root, dirs, files in os.walk(os.path.curdir):
		d = os.path.basename(root)
		if re.match('report', d):
			# get platename 
			cols = d.split('__')
			if len(cols) ==2:
				platename = cols[-1]
				resd.setdefault(platename, []).append(root)
			next
	#pprint.pprint(resd)
	# get the latest one
	f = open('V36-CONSD7-18_summary.txt' , 'w')
	
	for i in range(7, 19):
		platename = "V36-CONSD#%s" % i
		results = resd[platename]
		results.sort(lambda x,y: cmp(os.path.getmtime(y), os.path.getmtime(x)))
		print "\n"
		print results
		print platename, results[0]
		rfile, = glob.glob(os.path.join(results[0], '*_summary.txt'))

		lines = open(rfile).readlines()
		if i == 7:
			line0 = lines[0]
			print >>f, line0.strip()
		for l in lines[1:]:
			print >>f, l.strip()
	f.close()

def write_repeat(repeat_files):
	f = "repeat_%s.txt" % util.today()
	F = open(f, 'w')
	repeat_files.sort()
	for f in repeat_files:
		print >>F, open(f).read().strip()
	F.close()

def write_stat_summary(plate_stat_d):
	file = "V36_plate_stat.txt"
	F = open(file, 'w')

	tagd = {}
	for pid, stat_d in plate_stat_d.items():
		for tag, count in stat_d.items():
			tagd[tag] = 1
	tags = tagd.keys()
	tags.sort()
	tmp = ['Plate'] + tags
	print >> F, "\t".join(tmp)
	for pid, stat_d in plate_stat_d.items():
		data = [pid]
		for tag in tags:
			if stat_d.has_key(tag):
				data.append(stat_d[tag])
			else:
				data.append("")
		print >>F, "\t".join([str(x) for x in data])
	F.close()


def write_mutation_summary(plate_mutation_d, refile):
	print plate_mutation_d.keys()
	
	# read information from template
	summary_textfile = 'v36_master_mutation_summary.txt'
	summary_template = 'template_v36_master_mutation_summary.txt'
	if not os.path.isfile(summary_template):
		summary_template = '../template_v36_master_mutation_summary.txt'
		if not os.path.isfile(summary_template):
			return

	basename = os.path.basename(os.path.abspath(os.path.curdir))
	
	plates = plate_mutation_d.keys()
	mn = min(plates)
	mx = max(plates)
	name = "%sto%s" % (mn,mx)
	new_file = 'mutation_summary_V36-%s.txt' % name
	repeat_file = 'repeat_V36-%s.txt' %  name

	if os.path.isfile(summary_textfile):
		Fin = open(summary_textfile)
	elif os.path.isfile(summary_template):
		Fin = open(summary_template)
	else:
		print "ERROR: no template file in cwd"
		sys.exit()

	Fout = open(new_file, 'w')
	Frpt = open(repeat_file, 'w')

	while 1:
		line = Fin.readline()
	
		if not line:
			break
		if line.find('Target') != -1:
			print >> Fout, line.rstrip()
			continue

		elif line.find('Plate') != -1:
			print >>Fout, line.rstrip()
			continue
		 
		# get plate, well, and target aa pos
		cols = [x.strip() for x in line.split('\t')]
		col_len = len(cols)
		if col_len < 7:
			print >>Fout, line.rstrip()
			continue
		plate, well, row, col, pos, refaa, variant = cols[:7]
		try:
			plate = int(plate); 
		except:
			pass
		try:
			pos = int(pos)
		except:
			pass
		if not plate_mutation_d.has_key(plate):
			#print >> Fout, line.rstrip()
			continue

		#print plate, well, pos, refaa, variant
		other_mutations	= []
		target=other_mutation=tag = msg=""

		# update with new mutation information
		cols = cols[:7]
		well_mutationd = plate_mutation_d[plate]
		if well_mutationd.has_key(well):
			mutations = well_mutationd[well]['mutations']
			tag = well_mutationd[well]['tag']
			msg = well_mutationd[well]['msg']
			#print "XXX:", well, tag, msg
			if msg or tag == 'low quality':
				print >>Frpt, 'plate %s' % plate, well, tag, msg

			for mutation in mutations:
				f, p, m = mutation
				if p == pos:
					target = m
				else:
					other_mutations.append('%s%s%s' % (f, p, m))

			if mutations and not target:
				target = refaa

			# wt or no data?
			if re.match('WT',tag, re.IGNORECASE):
				target = refaa
		cols += [target, ",".join(other_mutations), tag, msg]
		print >>Fout, "\t".join(cols)
	Fout.close()
	Fin.close()
	Frpt.close()

def do_AP_summary():
	"""
	for AP25-AP28 plates
	expect mutation for each well:
	S24R
T33S
S76R
P84W
G116R
Y203A
K229F
W235R
N237F
	each well you might have 9 columns, one column per mutation, where it is notet as 1 if it is present or 0 if it is not


	"""
	expected_mutations = ['S024R', 'T033S', 'S076R', 'P084W', 'G116R', 'Y203A', 'K229F', 'W235R', 'N237F']
	LEN = len(expected_mutations)
	#mutation_positions = [24, 33, 76, 84, 116, 203, 229, 235, 237]
	mutation_positions = []
	for x in expected_mutations:
		p = re.findall('\d+', x)[0]
		mutation_positions.append(int(p))
	print mutation_positions

	summary= 'all_AP25-28_mutation_table.txt'
	F = open(summary, 'w')
	print >> F, "\t".join([ '']+expected_mutations+ ['Other'])

	# combine all mutations from all plates, generate special table
	for root, dirs, files in os.walk(os.path.curdir):
		basename = os.path.basename(root)
		
		if re.match('report_', basename):
			# check text file only
			for f in files:
				if re.search('_mutation_summary.txt', f):
					print f
					lines = open(os.path.join(root, f)).readlines()
					# sample_file, well, tag, info, mutations

					last_index = LEN
					for l in lines[1:]:
						well_data = ['0' for i in range(0, LEN)]
						cols = [x.strip() for x in l.split('\t')]

						filename, well, tag, info = cols[:4]

						for m in cols[4:]:
							i = re.findall('\d+', m)
							try:
								i, = i
								i = int(i)							
								index = mutation_positions.index(i)
								if m == expected_mutations[index]:
									well_data[index] = '1'
								else:
									well_data[index] = m
							except:
								well_data.append(m)
						print >>F, "\t".join([filename] + well_data)
	F.close()
								


def do_Janice_report():
	mutation_positions = [1, 25, 56, 218, 344, 356, 382]
	LEN = len(mutation_positions)
	summary= '091110_mutation_table.txt'
	F = open(summary, 'w')
	print >> F, "\t".join([ '']+ [str(x) for x in mutation_positions] + ['Other'])

	# combine all mutations from all plates, generate special table
	for root, dirs, files in os.walk(os.path.curdir):
		basename = os.path.basename(root)
		
		if re.match('report_', basename):
			# check text file only
			for f in files:
				if re.search('_mutation_summary.txt', f):
					print f
					lines = open(os.path.join(root, f)).readlines()
					# sample_file, well, tag, info, mutations

					last_index = LEN
					for l in lines[1:]:
						well_data = ['' for i in range(0, LEN)]
						cols = [x.strip() for x in l.split('\t')]

						filename, well, tag, info = cols[:4]

						for m in cols[4:]:
							i = re.findall('\d+', m)
							if not i:
								continue
							i, = i
							i = int(i)							
							if i in mutation_positions:
								index = mutation_positions.index(i)
								well_data[index] = m
							else:
								well_data.append(m)
						well_data = [filename] + well_data
						print >>F, "\t".join(well_data)
	F.close()

def do_QC(plates):
	"""
	new as 3/16/2011
	"""
	# get data directories
	dirs = []
	for p in plates:
		print p
		dirs += [d for d in glob.glob('2011*_%s_*' % p) if os.path.isdir(d)]
		
	print dirs
	for d in dirs:
		cols = d.split('_')
		try:
			date, plate, primer, machine = cols
		except:
			try:
				date, plate, primer = cols
			except:	
				print "\tskipping %s" % d
				continue
		#if not re.match('NCE', plate):
		#	continue
		print d, plate, primer	
		
		os.chdir(d)
		files = glob.glob("*")
		# rename file
		for f in files:
			if not re.search('2011', f):
				pass
			else:
				_rename_file(plate, primer, f)

		rename.rename_seq()
		# get repeats: any thing that is score < 40  and Q40 <400, seqlen < 700
		os.chdir('..')
		#rpts = score_abi.process_plate(d, {'s':40, 'l':700, 'q':400})
		#print rpts

def _rename_file(plate, primer, f):
			cols = f.split('.')
			ext = cols[-1]
			tmp = cols[0].split('_')
			well = tmp[-1]
			if not re.match('[A-H]\d\d', well):
				well = tmp[-2]
			if not re.match('[A-H]\d\d', well):
				print "ERROR:", well, ext, f	
				return

			f2 = "%s-%s_%s.%s" % (plate, well, primer, ext)
			#print f, f2
			os.rename(f, f2)

def build_download(optd, plates ):
	destd = 'download'
	if optd.has_key('O'):
		destd = optd['O']
	# create download files in destd
	if not os.path.isdir(destd):
		os.mkdir(destd)
	for plate in plates:
		plate_dir = os.path.join(destd, plate)
		if not os.path.isdir(plate_dir):
			os.mkdir(plate_dir)
		# copy data to plate_dir
		for primer in ['QB0411', 'QB0412']:
			name_pat = '2011*_%s_%s*' % (plate, primer)
			datadirs = glob.glob(name_pat)
			datadir, = datadirs
			print plate, datadir
		
			dest_datadir = re.sub('_[MJBOPuVT]+$', '', datadir)
			primer_dir = os.path.join(plate_dir, dest_datadir)
			if optd.has_key('z'):
				print "  zipping ... %s" % datadir
			else:
				if not os.path.isdir(primer_dir):
					os.mkdir(primer_dir)
				print "  copying to ... %s " % primer_dir
				if datadir == primer_dir:
					continue
				# get src files
				for root, dirs, files in os.walk(datadir):
					if len(files) != 192:
						print "Warning: file count %s" % len(files)
					for f in files:
						shutil.copy(os.path.join(root,f), os.path.join(primer_dir,f) )

						#print f, primer_dir
					break	

		# copy report to plate_dir
		report_dirs = glob.glob('report_%s_*' % plate)
		report_dirs.sort(lambda x,y: cmp(os.path.getctime(x), os.path.getctime(y)))
		report_dir = report_dirs[-1]
		for r, d, files in os.walk(report_dir):
			d_report_dir = os.path.join(plate_dir, 'report_%s' % plate)
			if not os.path.isdir(d_report_dir):
				os.mkdir(d_report_dir)
			for f in files:
				# copy cds.fas
				infile = os.path.join(report_dir, f)
				outfile = os.path.join(d_report_dir, f)	
				if f == '%s_mutation_summary.txt' % plate  :
					shutil.copy(infile, outfile)
				elif f in ['%s_aa.fas' % plate, '%s_cds.fas' % plate ]:
					F = open(outfile, 'w')
					all = Fasta.all(infile)
					ref = all[0]
					all.sort(lambda x, y: cmp(x.name, y.name))

					print >>F, '>%s\n%s' % (ref.name, ref.sequence)
					for a in all:
						if a.name == ref.name:
							continue
						print >>F, '>%s\n%s' % (a.name, a.sequence)
					F.close()
	
					cmd = "c:\\tools\\qbpy\\qblib\\align.py %s html" % outfile
					os.system(cmd)	
def create_zipfile(plates):
	print plates
	curd = os.path.abspath(os.path.curdir)

	zipfiles = []
	for plate in plates:
		print plate
		for root, dirs, files in os.walk(plate):
			os.chdir(root)
			_zipfiles = []
			for d in dirs:
				cmd = "zip -rq %s.zip %s" % (d,d)
				print cmd
				os.system(cmd)
				_zipfiles.append('%s.zip' % d)
			zipfiles += _zipfiles
			os.chdir(curd)
	for row in zipfiles:
		for f in row:
			tmp = '<a href="./%s">%s</a> &nbsp' % (f, f.replace('.zip', ''))
			print tmp

def do(args):
	usage = """
	Usage: FNAreport.py [option] <data directory>1 .. 2 .. 3
		options : 
	required: -p <plate name>

	optional:
	-A align
	-C cleanup
	-D build for data delivery
	-M identify mutation
	-O <output_dir> 
	-l <plate>: list plate in all drives
	-s start_pos
	-e end_pos, both -s and -e are required if -M 
	-u : unzip all zil files

	if none of the above is defined, do all three

	-r reffile, default is ref/ref.fas for FNA lib
	-N reference NT, name only, default is 'FNA:Plat-Tlat'
	-G reference CDS, name used in the reffile, default is 'FNA'
	-p myplate -r ref.fas dir1 dir2
	-A -p myplate -r ref.fas all_contig_seqfile
	-M -p myplate cds_alignment aa_alignment
	-S combine report files
	-x skip building report if there is a report_* directory
	-n <notefile, fullpath> tab delimited file, one line per plate, list all wells without bacteria
	-V V36 plates
	-X <refname1,refname2,...>: list of refname to be aligned to # new as 12/8/10
	-Q : rename file, and create repeat list
	-S <assembly program, cap or cap3, default is cap3>
	-T <starttag,endtag>
	-R <libary> <startplate> <endplate>: build summary report for Roopi
	-Z <plates>

	example

	1. to generate report for plate JK55-1
	cwd file/directory structure: directory name format <date>_<platename>_<primer>
		062211_JK55-1_QB0411 
		2011-06-22_JK55-1_QB0412

	FNAreport.py -M -a -r ref_PHLT-GG36.fas -G ref-GG36 JK55-1

	or 

	FNAreport.py -M -p JK55-1 -r ref_PHLT-GG36.fas -G ref-GG36 062211_JK55-1_QB041 2011-06-22_JK55-1_QB0412

	"""

	#opts, ars = getopt.getopt(args, 'aczACMRTup:r:N:G:s:e:l:')
	optd, ars = util.parse_args(args, 'acACDMQRSTuVp:Zr:N:n:G:s:e:l:xmX:S:T:O:', usage)

	if optd.has_key('Z'):
		return create_zipfile(ars)

	if optd.has_key('R'):
		return create_RA_summary_030911(ars)

	if optd.has_key('Q'):
		return do_QC(ars)

	#optd['T'] = 'ggagatatacat,ggatccgaattcga'
	if optd.has_key('D'):
		return build_download(optd, ars)
		
	if optd.has_key('m') and len(ars) == 2:
		return combine_reports()

	if optd.has_key('S'):
		return combine_reports()

	if optd.has_key('l'):
		return list_plate(optd['l'])

	if optd.has_key('u'):
		# unzip files
		return unzip_files()

	if optd.has_key('a'):
		print "ARS: ", ars
		return do_all(optd, ars)

	if optd.has_key('C'):
		return do_cleanup(1)

	if not optd.has_key('p'):
		print usage
		sys.exit()

	if not ars:
		if optd.has_key('M'):
			pass
		else:
			usage()

	platename = optd['p']

	try:
		nt_reffile = optd['r']
	except:
		nt_reffile = "c:\\tools\\ref\\V36_ref.fas"

	try:
		nt_name = optd['N']
	except:
		nt_name = ''
		#nt_name = 'ref-nt'

	try:
		cds_name = optd['G']
	except:
		cds_name = ''

	return plate_report(platename, nt_reffile, nt_name, cds_name, ars, optd)

def do_cleanup(tmp=0):
	cdir = os.path.abspath(os.path.curdir)
	for root, dirs, files in os.walk(cdir):
		basename = os.path.basename(root)
		if re.match('V36', basename):
			print basename
			for d in dirs:
				if re.match('report_', d):
					os.system('rm -rf %s' % os.path.join(root, d))
					print "rm %s" % d
				if tmp:
					if re.match('tmp_', d):
						os.system('rm -rf %s' % os.path.join(root, d))
						print "rm %s" % d

			for f in files:
				if f == '%s.zip' % basename:
					os.system('rm -rf %s' % (os.path.join(root, f)) )
					print "rm %s" % f
def do_all(optd, plates):
	print "PLATES:", plates

	if optd.has_key('z'):
		# do report for all and create download files
		return do_all_download(optd, plates)
	else:
		return do_all_check(optd, plates)

def do_all_check(optd, plates):
	rd = os.path.dirname(os.path.dirname(__file__))
	nt_name = ''#optd['N']
	cds_name = optd['G']
	
	#plates2 = [d for d in plates if os.path.isdir(d)]
	plates2 = plates
	if not plates2:
		plates2 = ["NCE2-%s" % i for i in range(1, 11)]
	plates = plates2
	for plate in plates:
		datadirs = glob.glob('201*_%s_*'% plate)
		if not datadirs:
			print "GLOB", '*_%s_*'% plate
			datadirs = glob.glob('*_%s_*'% plate)
			datadirs = [x for x in datadirs if re.match('\d+_', x)]
			
		print "do_all_check:plate=%s" % plate, datadirs
		if not datadirs:
			print "ERROR: cannot find data directory for plate %s" % plate
			sys.exit()

		plate_report(plate, optd['r'], nt_name, cds_name, datadirs, optd)

	#write_stat_summary(plate_stat_d)
	#write_repeat(repeat_files)
def xx(d):
	for root, dirs, files in os.walk(d):
		basename = os.path.basename(root)
		if re.match('(done)|(tmp)|(report)', basename):
			dirs = [];
			continue;
		if re.match('(NHJ)|(NCE)|(JS)|(RA\d\d)|(V36)|(AP\d+)', basename) or basename.find('KARA') != -1 or basename.find('V36-') !=-1 or \
			basename.find('AP') !=-1:
			platename = basename
			print root, platename
			datadirs = []
			skip = 0
			for d in dirs:
				if re.match('201', d):
					datadirs.append(d)

				if re.match('report_', d):
					skip = 1
			if optd.has_key('x') and skip :
				print '... skip, report directory found'
				continue

			if len(datadirs) < 2:
				print "Number of data directories != 2"
				print "\t", "  ".join(datadirs)
			else:
				os.chdir(root)
				platenumber = re.findall('\d+$', platename)
				try:
					platenumber, = platenumber
					x = int(platenumber)
				except:
					x = platenumber
				platenumber = x

				print "building report for plate %s" % platename
				x = plate_report(platename, refile, nt_name, cds_name, datadirs, optd=optd)
				if not x:
					continue
				(mutd, statd, repeat_file) = x
				plate_mutation_d[platenumber] = mutd
				plate_stat_d[platenumber] = statd
				repeat_files.append(repeat_file)

				os.chdir(cdir)


def add_codon_1plate(mutation_summary, rdsfile):
	cds = Fasta.all(cdsfile)

	seqs = []
	refseq = ''
	for a in cds:
		if re.match('ref', a.name):
			refname = a.name
			refseq  = a.sequence
		else:
			name = a.name.split(':')[0]
			seq  = a.sequence
			seqs.append((name, seq))

	if not refseq:
		print "ERROR: unknow reference sequence"
		sys.exit()

	ntdiffs = {}
	for a in seqs:
		name, seq = a
		ntdiffs.setdefault(name, {})
		for i in range(0, len(refseq), 3):
			_rc = refseq[i:i+3].upper()
			_ra = DNA.DNA(_rc).translate().upper()
			_mc = seq[i:i+3].upper()
			_ma = DNA.DNA(_mc).translate().upper()
			if _rc != _mc:
				ntdiffs[name].setdefault('%s%03d%s' % (_ra, i/3 +1 , _ma), _mc)

		#print name, ntdiffs[name]
		
	#_locd, _mutd = _read_mutation(mutation_summary)
	# rewrite mutation_summary
	f1 = mutation_summary.replace('.txt', '_1.txt')
	F = open(f1, 'w')
	lines = open(mutation_summary).readlines()
	print >> F, lines[0].rstrip()
	for l in lines[1:]:
		cols = l.split('\t')
		file, well, tag, info = cols[:4]	
		if ntdiffs.has_key(file):
			ntd = ntdiffs[file]
			muts =[]
			for mut in cols[4:] :
				mut = mut.strip()
				if ntd.has_key(mut):
					muts.append('%s, %s' % (mut,ntd[mut]))
					del ntd[mut]
				else:
					muts.append(mut)
			cols[4:] = muts
			for k, v in ntd.items():
				cols.append('%s, %s' % (k,v))
		print >>F, "\t".join(cols).rstrip()
	F.close()

def _read_mutation(mutation_summary, filter=1):
	locd = {}
	mutd = {}
	lines = open(mutation_summary).readlines()
	headers = lines[0].split('\t')

	for l in lines[1:]:
		cols = [x.strip() for x in l.split('\t')]
		file, well, tag, info = cols[:4]
		if re.search('framesfit',tag):
			continue
		if re.search('insertion', tag):
			continue
		if filter and tag != '':
			#print well, tag, 'skip'
			continue

		mutations = cols[4:] 
		mutd.setdefault(well, { 'tag': tag, 'info': info, 'mutations':{} } )

		for mut in mutations:
			if not mut:
				continue
			try:
				i, = re.findall('\d+', mut)
			except:
				print "Exception:", mut
				sys.exit()
			locd.setdefault(i, 0)
			locd[i] += 1
			w, m = re.split(i, mut)
			mutd[well]['mutations'].setdefault(i, (w,m))
	return (locd, mutd)

"""
NOTES: 2/22/2011, for Alex P, M1-M5 plates	:	
	- this set of plates have large chunk of deletions
	- first round: 
		-sequences with only outer primers
		-assembly reads to see whether they form contig
		- trim contig sequence using sequence tag at 5' and 3' of gene
			for pET24a vector: _5tag = 'gaaggagatatacat'
					3_tag = 'ggatccgaattcga'
		- filter out done clones, and do mutation for these clones
		- for notdone clones, figure out whether we need to repeat or add internal primers
"""
def create_RA_summary_030911(ars):
	print ars
	#f1 = 'NCE3_1-8_summary.txt'
	#for (lib, start, end) in [("NCE2", 1, 18), ('NCE1', 1, 10), ('NHJ18', 1, 10), ('NHJ19', 1, 7)]:
	#for (lib, start, end) in [("WCE25", 1, 10)]:
	(lib, start, end) = ars
	start = int(start)
	end = int(end)
	f1 = '%s_%s-%s_summary.txt' % (lib,start, end)
	t1 = "Library %s Plates %s to %s" % (lib, start, end)
	plates = ["%s%s" % (lib, i) for i in range(start,end+1)]
	_create_RA_summary(f1, t1, plates)

    	return

def _create_RA_summary(outfile, t, plates):
	F = open(outfile, 'w')

	plated = {}
	locd = {}	

	total_cnt = 0
	for plate in plates:
		print "PLATE:", plate
		for root, dirs, files in os.walk(plate):
			dirs = [d for d in dirs if re.search('report_', d)]

			fount = 0
			for f in files:
				if f == '%s_mutation_summary.txt' % plate:
					fount = 1
					print root, f
					break
			if fount:
				fullpath = os.path.join(root, f)
				_locd, mutd = _read_mutation(fullpath, filter=0)
				#pprint.pprint(_locd)
				#pprint.pprint(mutd.keys())
				total_cnt += len(mutd.keys())

				plated.setdefault(plate, mutd)
				locd.update(_locd)

	# now write to file
	# 10/7/2011, change to only list major mutations
	# define major mutations as 
	majorlocd = locd
	"""
	for loc, cnt in locd.items():
		if cnt >= 10 or cnt >= 0.01*total_cnt:
			majorlocd[loc] = cnt
	pprint.pprint(majorlocd)
	"""
	all_locs = locd.keys()
	all_locs.sort()

	mutation_locations = majorlocd.keys()
	mutation_locations.sort()
	print "Mutation LOCS:", len(mutation_locations), mutation_locations
	print >>F, "\t".join([t, "", "", "", "", "", "Mutations"] ) 
	#print >>F, "\t".join(["Sample Name", "Well", "Row", "Column", "Tag", "Info"] + mutation_locations + ['Other Mutations'])
	print >>F, "\t".join(["Sample Name", "Well", "Row", "Column", "Tag", "Info"] + mutation_locations )
	for plate in plates:
		mutd = plated[plate]
		for col in range(1, 13):
			for row in ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H']:	
				others = []
				well = '%s%02d' % (row, col)
				if not mutd.has_key(well):
					continue

				else:
					data = [plate, well, row, str(col)]
					tag  = mutd[well]['tag']
					info = mutd[well]['info']
					md   = mutd[well]['mutations']
					data.append(tag)
					data.append(info)
					#for loc in mutation_locations:
					for loc in all_locs:
						#print loc, type(loc), md.keys()
						if md.has_key(loc):
							r, m = md[loc]
							mut = '%s%s%s' % (r, loc, m)
						else:
							mut = ''

						if majorlocd.has_key(loc):
							data.append(mut)
						elif mut:
							#print "Other Mut:", others
							others.append(mut)
					data.append(",".join(others))
					print >>F, "\t".join(data)

	F.close()

	# write a sorted version
	lines = open(outfile).readlines()
	lines2 = []
	for line in lines[2:]:
		cols = line.strip().split('\t')
		lines2.append(cols)

	lines2.sort(lambda x,y: cmp(y[6:], x[6:]))
	F = open('sort_%s' % outfile, 'w')
	for l in lines[:2]:
		print >>F, l.rstrip()

	for l in lines2:
		print >>F, "\t".join(l)		
	F.close()

def do_AP_022211():
	nt_refile = "O:\\tools\\ref\\APisarchik\\ref_020311.fas"
	cds_name = 'ref'
	nt_name = ''

	optd= {'T':'gaaggagatatacat,ggatccgaattcga'}
	plates = ['M1', 'M2', 'M3', 'M4', 'M5']
	plates = ['M1']
	for plate in plates:
		dirlist = glob.glob(os.path.join(plate, '2011*'))
		plate_report(plate, nt_refile, nt_name, cds_name, dirlist, optd)
 
	# find unfinished clones
	for plate in plates:
		#reportdir = glob.glob(os.path.join(plate, 'report_%s*' % plate))
		reportdir = glob.glob('report_%s*' % plate)
		if not reportdir:
			continue
		reportdir.sort(lambda x,y: cmp(os.path.getmtime(x), os.path.getmtime(y)))
		contigfile = os.path.join(reportdir[-1], '%s_contig.fas' % plate)
		if not os.path.isfile(contigfile):
			continue

		# get contig clone name
		all = Fasta.all(contigfile)
		donedir = os.path.join(plate, 'done')
		if not os.path.isdir(donedir):
			os.mkdir(donedir)

		datadirs = glob.glob(os.path.join(plate, '2011*'))
		print datadirs
		for a in all:
			clone = a.name.split(':')[0]
			print clone
			for d in datadirs:
				f = os.path.join(d, '%s*' % clone)
				cmd = 'mv %s %s' % (f, donedir)
				os.system(cmd)
				print cmd
if __name__ == '__main__':
	#mutation_summary = 'O:\\Genecor_Project\\Roopi\\download\\NCE1-1\\report_NCE1-1\\NCE1-1_mutation_summary.txt'
	#cdsfile = 'O:\\Genecor_Project\\Roopi\\download\\NCE1-1\\report_NCE1-1\\NCE1-1_cds.fas'
	#add_codon_1plate(mutation_summary, cdsfile)
	#create_RA_summary_030911()
	#do_Janice_report()
	#do_AP_022211()
	#sys.exit()
	args = sys.argv[1:]
	do(args)
	"""
	get_wildtype_codon(sys.argv[1])
	"""
