#!python

import os, sys, re, types, shutil, glob, pprint, cPickle
import findPrimer
import pick_best

from qblib import util, process_dir, Plate, Fasta, BlastParser
from qblib.config import *

FINDPRIMER = os.path.join(os.path.dirname(__file__), 'findPrimer.py')
print FINDPRIMER

TOOLDIR = os.path.dirname(__file__)
REFDIR = TOOLDIR.replace('qbpy', 'ref')

PRIMERFILE = os.path.join(REFDIR, "FanYan_oligo.fas")
ORFFILE = os.path.join(REFDIR, "FanYan_ORF.fas")
ORF2PRIMER_FILE = os.path.join(REFDIR, "FanYan_ORF_OLIGO_mapping.txt")

def locate_primers(datadirs, primer_fasta=PRIMERFILE):
	print """Locate primers for reads in a list of directory, if no directory is passed in
		use current directory"""

	"""
	first batch is run with M13F and M13R primers, the sequences were then comapre to reference sequence 
	to find out the internal primers to used
	"""
	cloned = process_dir.get_allclones('_', datadirs)
	print "==> GOT %s unique clones" % len(cloned.keys())

	# do assembly, if M13F and M13R sequences formed a contig, no internal primer is needed
	all_read_fasta = os.path.join(REPORT_DIR, 'all_reads.fas' )
	Fas = open(all_read_fasta, 'w')

	wkdir = os.path.join(TMP_DIR, 'tmp_%s' % os.path.basename(os.getcwd()))
	if not os.path.isdir(wkdir):
		os.mkdir(wkdir)

	print "REPORTDIR:", REPORT_DIR
	print "TMPDIR:", wkdir
	
	for clone in cloned.values():
		clone.set_dir(REPORT_DIR, wkdir)
		clone.do_assembly(fprimer='M13F', rprimer='M13R')
		contig_seqs = clone.get_contig_seqs()
		for s in contig_seqs:
			print >>Fas, '>%s:%s\n%s' % (clone.name, s.name, s.sequence) 
		clone.contigs = contig_seqs
		for read in clone.reads:
			#print "READ:", read.sample, len(read.bases)
			s = read.trimseq()
			if not s:
				s = "".join(read.bases)
			print >>Fas, '>%s\n%s' % (read.sample, s)
	Fas.close()

	print "==> DO blast to find what it match to"
	# do blast against ORF sequence, determine the clone it matches
	cmd = "%s -p blastn -i %s -d %s -o %s.out -m 8 -e 1e-5" % (BLASTALL, all_read_fasta, ORF_FASTA, all_read_fasta)
	print cmd
	os.system(cmd)

	blastres = BlastParser.parse('%s.out' % all_read_fasta)
	query2hits = {}
	print "Blast Result count:", len(blastres)

	for r in blastres:
		best_hits = r.get_best_hits()
		print r.query, 'hit count:', len(best_hits)
		
		query2hits[r.query] = best_hits	

	# get gene2oligo mapping
	OLIGO_PICKLE = ORF2PRIMER_FILE.replace('.txt', '.cpickle')
	gene2primer = {}
	try:
		gene2primer = cPickle.load(open(OLIGO_PICKLE))
	except:
		lines = open(ORF2PRIMER_FILE)
		for l in lines:
			l = l.strip()
			if not l:
				continue
			cols = l.split('\t')
			primerid, primername, gene = cols[:3]
			gene2primer.setdefault(gene, []).append('%s:%s' % (primerid, primername))	
			picklefile = open(OLIGO_PICKLE, 'w')	
			cPickle.dump(gene2primer, picklefile)
			picklefile.close()
	# check that each read, and the contig sequence for each clone match to the same thing
	# use the oligo.txt file to find out what primer to use

	print "... do blast to find primer"
	cmd = "%s -p blastn -i %s -d %s -o match2primer.out -m 8 -e 12-5" % (BLASTALL, all_read_fasta, primer_fasta)
	os.system(cmd)

	# get oligo sequence
	oligo_name2seq = {}
	oligos = Fasta.all(primer_fasta)
	for o in oligos:
		oligo_name2seq[o.name] = o.sequence

	blast_results = BlastParser.parse_m8and9('match2primer.out', identity_cutoff=100)
	clone2primers = {}
	for bres in blast_results:
		for hit in bres.hits:
			oligo_seq = oligo_name2seq[hit.subj]
			for hsp in hit.hsps:
				if hsp.length != len(oligo_seq):
					continue
				print bres.query, hit.subj, hsp.ident, hsp.length, len(oligo_seq)
				clone2primers.setdefault(bres.query, []).append(hit.subj)

	clonenames = cloned.keys()
	clonenames.sort()

	fout = open('quality_report.txt', 'w')
	print >>fout, "\t".join(['Clone', 'Match To', 'Form Contig', 'Internal Primer', 'To Repeat', 'Read Quality'])
	for clonename in clonenames:
		clone = cloned[clonename]

		has_match=0
		hits = []
		for r in clone.reads:
			if query2hits.has_key(r.sample):
				r.hits = query2hits[r.sample]
				has_match = 1
				hits.extend(r.hits)

		hitnames_d  = dict([(h.name,1) for h in hits])
		hitnames = hitnames_d.keys()

		# use name to locate primer
		primers = []
		for hitname in hitnames:
			if gene2oligos.has_key(hitname):
				primers.extend( gene2oligos[hitname] )

		results = [clonename]

		if not hitnames:
			results.append( "do not match to any reference")
		else:
			results.append( "%s" % ( ",".join (hitnames) ) ) 

		# form a contig?
		if len(clone.contigs):
			a = clone.contigs[0]
			results.append( "form contig:%s bp" % len(a.sequence))

		else:
			results.append( "" )
			
		# need new primers?
		if primers:
			primer = []
			for p in primers:
				id, name = p
				primer.append('%s:%s' % (id, name))

			results.append(", ".join(primer))
		elif clone.contigs:
			results.append('done')
		else:
			results.append('')

		# get read quality, any one to be repeated?
		torepeat = []
		read_quality = []
		for r in clone.reads:
			r.trim_by_quality()
			if not r.trimscore or r.trimscore < 40 and r.trimend-r.trimstart < 600:
				torepeat.append(r.sample)
			if r.trimscore:
				read_quality.append('%s:read_len=%s,score=%s' % (r.sample, r.trimend-r.trimstart, r.trimscore))
		if torepeat:
			results.append('to repeat: %s' % ", ".join(torepeat))
		else:
			results.append('')
		if read_quality:
			results.append("; ".join(read_quality) )
		else:
			results.append('')


		print >>fout, "\t".join(results)

	for clone, primers in clone2primers.items():
		print >>fout, '%s\t%s' % (clone, primers)
	fout.close()

usage = """
	%s [option] datadirs
	options:
	-R: build report
	-P: find primer
	-r <refile>
	-N <refnt>
	-G <refcds>

"""
def do(args):
	rootpath = os.path.dirname(os.path.dirname(__file__))
	orf_file = os.path.join(rootpath, 'ref', 'ORF.fas')
	oligo_file = os.path.join(rootpath, 'ref', 'oligo_seqs.fas')
	print orf_file, oligo_file
	optd, ars = util.parse_args(args, "RPr:N:G:", usage)

	if optd.has_key('P'):
		return locate_primers(ars)

	platename = os.path.basename(os.path.abspath(os.path.curdir))

	plate = Plate.FanYanPlate(name=platename, ref_fasta=orf_file)
	plate.set_seqdata('_', ars)
	plate.do_assembly()
	plate.align2ref()
	plate.build_html_report()


if __name__ == '__main__':
	do(sys.argv[1:])
