#!python

#import parse_arg
import os, sys, glob, re, pprint

from qblib import DNA, Fasta, align, util, process_dir

class Stat:
	def __init__(self, trimscore=None, length=None, startloc=None, endloc=None):
		self.trimscore = trimscore
		self.length = length
		self.startloc = startloc
		self.endloc = endloc

def do_trim(reffasta):
	print "do trim"
	# get all sequences
	dname = os.path.basename(os.path.abspath(os.curdir))

	fas = "%s.fas" % dname
	fout = open(fas, 'w')

	print "fasta file:", fas
	files = glob.glob("*.seq")
	for f in files:
		fin = open(f)
		while 1:
			line = fin.readline()
			if not line:
				break
			if re.match('>', line):
				print >>fout, line.strip()
				print >>fout, fin.read()
				break
			else:
				print >>fout, '>%s' % re.sub('\.seq$', '', os.path.basename(f))
				print >>fout, line.strip()
				print >>fout, fin.read()
				break
	fin.close()
	fout.close()

	# do blast to figure out alignment coordinates
	blastout = '%s.out' % fas
	cmd = "blastall -p blastn -i %s -d %s -e 1e-5 -o %s -m 8" % (fas, reffasta, blastout)
	# build index file if there is none
	files = glob.glob('%s.nin' % reffasta)
	if not files:
		os.system('formatdb -i %s -p F' % (reffasta))

	os.system(cmd)

	# process blast result
	lines = open(blastout).readlines()
	hitd = {}
	for l in lines:
		l = l.strip()
		if not l:
			continue
		cols = l.split('\t')
		query, subj, ident, alen, mismatch, gap, qstart, qend, sstart, send, evalue, score = cols

		ident = eval(ident); alen = eval(alen)
		sstart = eval(sstart); send = eval(send)
		qstart = eval(qstart); qend = eval(qend)

		print query, subj, ident, alen, qstart, qend, sstart, send

		hitd.setdefault(query, [])

def trim_by_restriction_enzymes(sequence, startsite, endsite, ignore3=0):
	d = DNA.DNA(sequence)

	stag, sidx = startsite
	etag, eidx = endsite
	#print "----"
	start_sites, start_positions = d.find_restriction_sites(startsite)
	#print "FORWARD:", start_sites, start_positions
	if not start_positions:
		sequence = d.reverse_complement()
		d2 = DNA.DNA(sequence)
		start_sites, start_positions = d2.find_restriction_sites(startsite)
		end_sites, end_positions = d2.find_restriction_sites(endsite)
		#print "REVERSE:", start_sites, start_positions, end_sites
	else:	
		end_sites, end_positions = d.find_restriction_sites(endsite)
	"""
	if endsite == startsite:
		end_sites = start_sites; end_positions = start_positions
	else:
		end_sites, end_positions = d.find_restriction_sites(endsite)
	"""

	trimseqs = []

	s=e = None
	if start_positions:
		s = start_positions[0]
	if end_positions:
		#e = end_positions[-1]
		e = end_positions[0]
	elif ignore3:
		e = len(sequence)
	else:
		e = None

	if s:
		if e: 
			if s < e:
				s = s + sidx
				e = e + eidx
				return [(s, e, sequence[s:e] )]
		elif ignore3:
			s = s+sidx
			e = len(sequence)
			return [(s,e, sequence[s:])]
	else:
		return [(s,e,'')]


def test(seqfile, startsite=['GGCCNNNNNGGCC', 8], endsite=['GGCCNNNNNGGCC', 8]):
	allseqs = Fasta.all(seqfile)

	fout = open('trimseqs.fas', 'w')
	for a in allseqs:
		print '==> trim %s' % a.name
		seqs = trim_by_restriction_enzymes(a.sequence, startsite, endsite)
		if not seqs:
			continue

		i = 1
		for st, end, s in seqs:
			print >>fout, '>%s' % a.name
			print >>fout, s

	fout.close()
	print "Trimmed sequences in trimseqs.fas"

def do_GangChen(startsite='NdeI', endsite='EcoRI', platename='', outdir='', datadir=[], score=20, length=300, optd={}):
	# get all sequences from current working directory
	#startsite =['TATTCTCACT',1]
	#endsite = ['GAAGAAGGTAAA',12]
	if not datadir:
		datadir = [os.getcwd()]
	print "datadir: ", datadir

	allseqs = process_dir.get_allseqs(datadir, score=score, length=length, trim=1)
	print "seq count:", len(allseqs)

	if platename:
		dname = platename
	else:
		cwd = os.path.abspath(os.path.curdir)
		dname = cwd.split(os.sep)[-1]

	if not outdir:
		outdir = os.path.abspath(os.path.curdir)
	if not os.path.isdir(outdir):
		os.mkdir(outdir)
	ntfile = os.path.join(outdir, 'trimseq_%s_nt.fas' % dname)
	aafile = os.path.join(outdir, 'trimseq_%s_aa.fas' % dname)

	F = open(ntfile, 'w')
	A = open(aafile, 'w')

	# do sequencing quality stat
	statd={}
	INFOTAGS = ['total', 'lowscore', 'starttag', 'endtag', 'bothtags', 'notag']
	for x in INFOTAGS: 
		statd[x] = 0

	for abi in allseqs:
		statd['total'] += 1
		if abi.score() <= score:
			statd['lowscore'] += 1

			print "... skipping low quality sequence %s" % abi.sample, abi.score()
			continue
		
		f = abi.fasta()
		ignore3=0
		if optd.has_key('z'):
			ignore3=1

		print abi.sample,
		seqs = trim_by_restriction_enzymes(f.sequence, startsite, endsite, ignore3=ignore3)
		(s,e,seq) = seqs[0]
		print  s,e
		if s and e:
			statd['bothtags'] += 1
		elif s and not e:
			statd['starttag'] += 1
		elif not s and e:
			statd['endtag'] += 1
		else:
			statd['notag'] += 1
		if not seqs:
			continue

		for s in seqs:
			start, end, seq = s

			if seq.strip() == '':
				continue
				
			print >>F, '>%s\n%s' % (f.name, seq)
			aa=DNA.DNA(seq).translate(endatstop=1)
			 
			if aa.endswith('X'):
				aa = re.sub('X$', '', aa)
			print >>A, '>%s\n%s' % (f.name, aa)

	F.close()
	A.close()

	print "trimseq file: %s" % ntfile
	if not os.path.isfile(ntfile) or os.path.getsize(ntfile) == 0:
		print "INFO: no trimseq file, exit"
		return

	# align sequence
	align.do_muscle(aafile,  'html')	

	# write quality stat
	pprint.pprint(statd)
	F = open(os.path.join(outdir, 'stat_%s.txt' % dname), 'w')
	start_tag = startsite[0]
	end_tag = endsite[0]

	headers = []
	tmp = []
	for s in INFOTAGS:
		if s in ['total', 'notag', 'lowscore', 'notag', 'bothtags']:
			headers.append(s.title())
		elif s == 'starttag':
			headers.append('%s: %s only' % (s.title(), start_tag))
		elif s == 'endtag':
			headers.append('%s: %s only' % (s.title(), end_tag))
		tmp.append(statd[s])
	print >>F, "\t".join(headers)
	print >>F, "\t".join([str(x) for x in tmp])
	F.close()

	#return group_result(dname, ntfile, outdir=outdir)
	return group_result(dname, aafile, outdir=outdir)

def group_result(dname, aafile, outdir=''):
	# build grouping report
	all_aas = Fasta.all(aafile)
	d = {}
	name2aa = {}
	for a in all_aas:
		if a.sequence.strip() == '':
			continue

		d.setdefault(a.sequence, []).append(a.name)
		name2aa[a.name] = a.sequence

	print "UNIQUE seq count:", len(d.keys())
	
	# sort item by count
	items = d.items()
	items.sort(lambda x,y: cmp(len(y[1]), len(x[1])))
	group = 1

	print 'GROUP count:', len(items)
	name2group = {}
	for item in items:
		sequence, names = item
		for name in names:
			name2group[name] = group
			#print name, group
		group += 1

	# create unique sequence file
	file = os.path.join(outdir, 'unique_trimseq_%s.fas' % dname)
	# create unique sequence with count info
	f2 = os.path.join(outdir, 'unique_trimseq_%s_summary.txt' % dname)
	F2 = open(f2,'w')
	F = open(file , 'w')

	#seqs = d.keys()
	#seqs.sort()
	# sort by count
	all_seqs = d.items() # key is sequence, value is name list
	all_seqs.sort(lambda x, y: cmp(len(y[1]), len(x[1])))
	
	for (sequence, names) in all_seqs:
		#names = d[sequence]
		print >>F, '>%s\n%s' % (",".join(names), sequence)
		print >>F2, '%s\t%s\t%s' % (len(names), ",".join(names), sequence)
	F.close()
	F2.close()

	align.do_muscle(file, 'html')

	# count amino acid frequency
	do_special_report(file, dname, outdir)

def trim_common(datadirs, start=('CGACGGGGCC',10), end=('GGGGCCGC',0), outdir=''):
	"""
	trim off common sequences at both ends for all sequences in data directory: datadirs
	"""
	if not outdir:
		outdir = os.getcwd()
	cloned = process_dir.get_clones(datadirs, outdir)
	clonenames = cloned.keys()
	clonenames.sort()

	F = open('trimseq_%s.fas' %outdir , 'w')
	for clonename in clonenames:
		clone = cloned[clonename]
		for read in clone.reads:
			seq = read.seq()	
			trimseq = trim_by_restriction_enzymes(seq.sequence, start, end)
			if not trimseq:
				continue

			print clonename, trimseq
			i = 1
			for [s,e,seq] in trimseq:
				print >>F, '>%s:%s-%s\n%s' % (clonename,s,e,seq)
				i += 1
	F.close()
def do_special_report(fasta, platename, outdir=''):
	"""
	AA-POS	A	
	1	count	%		
	2
	3
	"""
	dname = platename
	print "FILE:", fasta
	all = Fasta.all(fasta)
	maxlen = 0
	pos2aad = {}
	n=0
	names = []
	aad = {}
	for a in all:
		name = a.name
		names.append(name)
		seq = a.sequence
		seq = re.sub('^DGA', '', seq)
		seq = re.sub('[Gx]+$', '', seq)
		seqlist = list(seq)
		for i in range(0, len(seq)):
			aa = seqlist[i]
			aad[aa] = 1
			pos2aad.setdefault(i+1, {})
			pos2aad[i+1].setdefault(aa, 0)
			pos2aad[i+1][aa] += 1
		n += 1
	# now create the table
	# header is aas
	headers = aad.keys()
	headers.sort()
	F = open(os.path.join(outdir, 'aa_usage_summary_table_%s.txt'%dname), 'w')
	print len(headers)
	HEADERS = ['POS', ''] + headers
	pos= pos2aad.keys()
	pos.sort()
	print >>F, "\t".join(HEADERS)
	for p in pos:
		data1 = []
		_aad = pos2aad[p]

		for a in headers:
			try:
				cnt = _aad[a]
			except:
				cnt = 0

			data1.append(cnt)
		total = sum(data1)
		data2 = [x*100.0/total for x in data1]
		x = [p, 'count'] + data1
		print >>F, "\t".join([str(xx) for xx in x])
		x = [p, '%'] + data2
		print >>F, "\t".join([str(xx) for xx in x])
	F.close()

def combine_results(optd, datalist):
	# data list should either be a list of directory or a list for fasta file
	f_data = []
	d_data = []
	for d in datalist:
		if os.path.isfile(d):
			f_data.append(d)
		elif os.path.isdir(d):
			d_data.append(d)
	if d_data:
		print "ERROR: unimplement, cannot process a list of directories"
		sys.exit()

	if optd.has_key('p'):
		pname = optd['p']
	else:
		print "ERROR: please provide platename using -p <platename>"
		sys.exit()
	platename = optd['p']

	fastad = {}
	for f in f_data:
		all = Fasta.all(f)
		for a in all:
			fastad.setdefault(a.sequence, []).append(a.name)
	file = "%s.fas" % platename
	F = open(file, 'w')
	for seq, names in fastad.items():
		print >>F,">%s\n%s" % ( ",".join(names), seq)
	F.close()
	group_result(platename, file)

def do_all(optd, ars):
	if not ars:
		ars = [os.path.curdir]
		
	for d in ars:
		for root, dirs, files in os.walk(d):
			dname = os.path.basename(root)
			pname = re.sub('_[BOPUTVMJ]+$','', re.sub('20\d\d\-\d{2}\-\d{2}_', '', dname))
			files = [f for f in files if f.endswith('ab1')]
			optd['p'] = pname
			if re.match('20\d\d\-\d\d', dname ) and files:
				print root, dname, pname
				do1(optd, [root])
				

def process(args):
	usage = """
	%s [option] (required one of A B D E F tags, other are optional)
	-A: process phage library, using phage tag, sequenced with primer S1 (QB1515)
	-B: process pET libaray, sequenced with T7F primer 
	-H: process Heavy chain library for Qi AN
	-L: process Heavy chain library for Qi AN
	-X: process pET libaray, sequenced with T7F primer, special case, move 5' tag 12 bases upstream
	-Y: process pET libaray, sequenced with T7F primer, special case, move 3' tag 12 bases downstream
	-D: process CAO's special tags
	-E: process CAO's seq tag with Top-F primer
	-F: process MAL3 primer tag
	-G: process pGADT7 vector
	-V: process v124ER4 plate, for samples sequenced with Bot-1 primer
	-S <summary_file>
	-c <ntfasta_filelist>: combine results from a list of nt fasta files
	-p <platename>
	-o <output name>
	-l <length cutoff>
	-s <score cutoff>
	-t 1|0: 1 to include tag in the alignment, 0 to remove tag from alignment, default is 0
	-o <output dir>
	-z: if specified, ignore 3 end match 

	e.g GangChenRepeat.py -A -t 1 -p 042011JZ_ABC 
		for phage primer S1 (QB1515) sequences
	""" % __file__
	optstr  = "A|B|C|D|E|F|G|H|L|S|V|X|Y|a|s:|p:|o:|l:|t:|o:|x|y|z"
	optd, ars = util.parse_args(args, optstr, usage)
	if not optd:
		print usage
		return

	print optd
	if optd.has_key('a'):
		return do_all(optd, ars)

	else:
		return do1(optd, ars)

def do1(optd, ars):
	try:
		outdir = optd['o']
	except:
		outdir = ''

	datadirs = ars

	include_tag = 0
	if optd.has_key('t'):
		try:
			include_tag = int(optd['t'])
		except:
			pass
	if optd.has_key("A"):
		# DG ...... G for translation
		_5 ='GACGGGGCC'
		_3 = 'GCCGCTGGG'
		#startsite =['CGACGGGGCC',10]
		endsite = ['GCCGCTGGGG',0]
	elif optd.has_key("B"):
		_5 = 'GCCGACGGGGCC'
		_3 = 'GGGGCCGCTGGG'
		#startsite = ['GCCGACGGGGCC', 0 ]
		#endsite   = ['GGGGCCGCTGGG', 12]
	elif optd.has_key('C'):
		_5 = 'TCTATTCCGACACCCTCC'
		_3 = 'GAAACACAACAGTGT'
	elif optd.has_key('D'):
		_5 = 'TCACTCGGCCGACGGGGCC'
		_3 = 'GGGGCCGCTGGGAATTC'
	elif optd.has_key('E'):
		# process Top-F sequencing
		_5 = 'GGCCAGGCGGCC' #GQAA
		_3 = 'TAGGGCCAGGCC' #*GQA
	elif optd.has_key('F'):
		# process MAL3 tag
		#_5 = 'GAGTCTATTCCGACACCCTCC' #SIPTPS
		#_3 = 'GAAACACAACAGTGTTCCAACCTGG' #ETQQCSNLE
		_5 = 'ATTCCGACACCCTCC' #PTPS
		_3 = 'GAAACACAACAG' #ETQQC
	elif optd.has_key('V'):
		# process V128ext sequencing
		_5 = 'ACCGTGGCCCAGGCGGCC' #GQAA
		_3 = 'GGCCAGGCCGGCCAG' #*GQA
	elif optd.has_key('G'):
		_5 = 'TGGCCATTACGG'
		_3 = 'GGCCGCCTCGGC'
	elif optd.has_key("X"): # T7F primer, but move 5' tag 12 bases upstream
		_5 = 'TCTCACTCGGCC'
		_3 = 'GGGGCCGCTGGG'
	elif optd.has_key("Y"):
		# DG ...... G for translation
		_5 ='GACGGGGCC'
		_3 = 'GAAGAAGGTAAA'
	elif optd.has_key("H"):
		#VH-CH1:  TVSS---ASTKGPSVFP
		#VK-CK:  LEIK---RTVAAPSVFI
		#VL-CL: LTVL---GQPKAAPSVT 
		# used for processing Qi AN sequences
		_5 = 'CTGCTGCTGTGGGTG'
		_3 = 'CATCATCATCACCATCAC'
	elif optd.has_key("L"):
		# used for processing Qi AN sequences
		_5 = 'CTGCTGCTGTGGGTG'
		_3 = 'AACAGGGGAGAGTGT'
	elif optd.has_key("x"):
		_5 = "GAATTCGCCCTTC"
		_3 = "AAGGGCGAATTC"

	# for topo cloning
	# for smita
	elif optd.has_key("y"):
		_5 = "AGCAGTGGTATCAACGCAGAGTAC"
		_3 = "gtcagtcttcccactggccccctg".upper() # H chain
		_3 = "TGAAGGGCGAAT".upper() # H chain
		#_3 = "actgaaaacctcgaccgtggttccac" # light chain

	elif optd.has_key('c'):
		return combine_results(optd, datadirs)
	else:
		print "ERROR: trim option not provided"
		sys.exit()

	if include_tag:
		# include tag in the alignment
		startsite = [_5, 0]	
		endsite = [_3, len(_3)]
	else:
		startsite = [_5, len(_5)]	
		endsite = [_3, 0]


	try:
		score_  = int(optd['s'])
	except:
		score_ = 20
	try:
		length_ = int(optd['l'])
	except:
		length_ = 300

	platename = ''
	if optd.has_key('p'):
		platename = optd['p']
	print "Plate name:",  platename
	do_GangChen(startsite, endsite, platename, outdir=outdir, datadir=datadirs, score=score_, length=length_, optd=optd)



if __name__ == '__main__':
	#seqfile = sys.argv[1]
	#test(seqfile, startsite=['CTGCTGCTGTGGGTG', 0])
	#file = sys.argv[1]
	#dname = sys.argv[2]
	#outdir = '.'
	#do_special_report(file, dname, outdir)
	#sys.exit()
	#util.log_cmd(sys.argv)
	process(sys.argv[1:])
	sys.exit()


