#!python

import os, glob, sys, re, pprint, getopt, time, string
from qblib import DNA, Fasta, ABISeq, Clone, process_dir, align, BlastParser, util
import CloneAssembler

ROWS = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H']
COLS = ['01', '02', '03', '04', '05', '06', '07', '08', '09', '10', '11', '12']

CAP = "d:\\tools\\BioEdit\\apps\\cap.EXE"
MUSCLE = "c:\\tools\\bin\muscle.exe"

_20AA=['I', 'L', 'V', 'F', 'M', 'C', 'A', 'G', 'P', 'T', 'S', 'Y', 'W', 'Q', 'N', 'H', 'E', 'D', 'K', 'R', '*']
_20AA.sort()

DEFAULT_NTFILE = 'C:\\quintara\\cdata\\Genecor\\Kudzu-pET24d_nt.txt'
DEFAULT_AAFILE = 'C:\\quintara\\cdata\\Genecor\\Kudzu-pET24d_aa.txt'

class Sample:
	def __init__(self, **kw):
		for k, v in kw.items():

	def __setattr__(self, k,v):
		self.__dict__[k] = v	

	def __getattr__(self, k):
		try:
			return self.__dict__[k]
		except:
			return ""
	def __str__(self):
		tmp = ["%s=%s" % (k,v) for (k,v) in self.__dict__.items()]
		return ", ".join(tmp)

def get_score(fdir):
	print "... processing %s" %fdir
	sample2score = {}
	return sample2score



def guess_mutation_position(fdir, rdir):
	wd[well] = (seq, aa)
	return get_mutation_pos(wd)


def get_contig_new(tmpd, well, seqs):
	all = Fasta.all(contig)
	alls = [a.sequence for a in all]
	return alls
	
	
def group_mutation_052208(tmpd, libname, mut_pos, well2samples):
	seqd = {}
	wells = well2samples.keys()
	wells.sort()
	for well in wells:
		seqs = well2samples[well]
		dnaseq, aaseq, coding_seq = get_contig(tmpd, libname, well, seqs) 
		print "WELL:%s, DNALEN:%s, AALEN: %s, CDSLEN:%s" % (well, len(dnaseq), len(aaseq), len(coding_seq)), aaseq[:10]
		
		seqd[well] = (dnaseq, aaseq, coding_seq)
	if 1:
		mut_aad=group_mutation_Eraser32(mut_pos, well2samples)
	else:
		mut_aad = group_mutation2(mut_pos, seqd)

	return (seqd, mut_aad)



def merge_sequence(libname, mut_wells, fdir, rdir, mdir={}, overide=0, fdname='', rdname='', mdname=''):
	
		(seq, aa) = get_contig(tmpd, libname, well, fseq, rseq, mseq, fdir=fdname, rdir=rdname, mdir=mdname)
		if not seq:
			bads.append(well)

		seqs.append( (well, seq, aa, fscore, rscore, mscore))

		return seqs


def group_mutation(tmpd, libname, mut_pos, well_d):
	seqd = {}
	for well, dir_d in well_d.items():
		dnaseq, aaseq = get_contig(tmpd, libname, well, dir_d.items()) 
		seqd[well] = (dnaseq, aaseq)

	mut_aad = group_mutation2(mut_pos, seqd)
	return (seqd, mut_aad)

def group_mutation2(mut_pos, well2seq, rc=0):
	
	aa_d = {}
	# group all reverse sequence reads
	print "XXX:mutation position: %s, rc=%s" % ( mut_pos, rc)

	# get 15 bases downstream of the mutation point
	# use it as tag sequence to locate the potential mutation in other sequences
	offset = 5

	mutidx = mut_pos-1
	aastart = mutidx-offset
	aaend = mutidx
	nastart = aastart *3
	naend = aaend *3

	#na_tag = refnt[nastart:naend].upper()
	na_tag = CDS[nastart:naend].upper()
	aa_tag = refaa[aastart:aaend]

	#print "TAG seq:", na_tag, aa_tag 
	print "XXXXwild type:", refaa[mutidx]

	sd = well2seq

	for well, seqinfo in sd.items():
		ntseq, aaseq, seq = seqinfo
		if not ntseq:
			continue
		# length of refnt and mynt shuld be the samem otherwise, possible mutation
		#print well, len(refnt), len(ntseq)
		if len(ntseq) < len(refnt):
			aa_d.setdefault('', []).append(well)
			continue


		if aaseq:
			aa = aaseq[mutidx]
			aa_d.setdefault(aa, []).append(well)
			continue

		#print "well %s, no aa, with ntseq" % well, ntseq[:40]
		# find tag seqeunce in ntseq if there is no corresponding aaseq
		tidx = ntseq.find(na_tag)
		if tidx == -1:
			mytag = na_tag[:-3]
			tidx = seq.find(mytag)
			if tidx == -1:
				print "No tag:", well, tidx, "seq len", len(seq), na_tag, mytag
				aa_d.setdefault('', []).append(well)
				
				continue
			else:
				tidx += 3
		else:
			mytag = na_tag

		# include mutation aa and downstream tag
	
		if tidx == -1:	
			aa_d.setdefault('', []).append(well)
		else:
			tidx = tidx + len(mytag)
			ntseq = seq[mutidx*3:(mutidx+1)*3]
			ntseq2 = seq[tidx:tidx+3]
			print "--", mutidx, ntseq, tidx, ntseq2
			aaseq = DNA.DNA(ntseq).translate()
			aa = aaseq[0]
			aa_d.setdefault(aa, []).append(well)

		#print well, ntseq, tidx, aa
	return aa_d

def group_mutation_031809(mut_pos, aa_fasta, cloned) :
	aaseqs = Fasta.all(aa_fasta)
	mutaa_d = {}
	done_clone = []
	other_mut_d = {}

	refseq = aaseqs[0]
	ref_aaseq = refseq.sequence
	for aaseq in aaseqs[1:]:
		i = aaseq.name.find(':Contig')
		name = aaseq.name[:i]
		if name == refseq.name:
			continue
		#print "%s: ref aa len:%s, myaa len: %s" % (name, len(ref_aaseq), len(aaseq.sequence))
		for i in range(0, len(ref_aaseq)):
			if len(aaseq.sequence) < len(ref_aaseq):
				continue
			aa = aaseq.sequence[i]
			if i == mut_pos-1:
				mutaa_d.setdefault(aa, []).append(name)
			elif aa != ref_aaseq[i]:
				other_mut_d.setdefault(name, []).append(i+1)
			done_clone.append(name)

	failed_clones = []
	for clonename, clone in cloned.items():
		if clonename not in done_clone:
			print "NEED to figure out mutant info from read: %s" % clonename
			failed_clones.append(clonename)
	return (mutaa_d, other_mut_d, failed_clones)

def find_mutation_from_read(tmpdir, mut_pos, cloned, failed_clones):
	aad = {}
	failed = []
	welld = {}
	if not failed_clones:		
		return (aad, failed, well2mut)

	# get temp reference sequence
	p = mut_pos*3
	s = p - 30
	e = p + 30
	if s < 0:
		s = 0
	if e > len(CDS):
		e = len(CDS)
	tmp_refseq = CDS[s:e] 

	print "XXX:", tmp_refseq

	# create tmp reference file
	tmpfile = os.path.join(tmpdir, 'tmp_reference.fas')
	F = open(tmpfile, 'w')
	print >>F, '>%s\n%s' % ('tmp', tmp_refseq)
	F.close()

	aad = {}
	failed = []
	welld = {}
	for clonename in failed_clones:
		clone = cloned[clonename]

		read_fasta = clone.read_fasta()
		if not read_fasta:
			continue

		blastout = os.path.join(tmpdir, '%s_read.bls' % clone.name)
		align.blast_ref2db(tmpfile, read_fasta, blastout)
		hitseqs = BlastParser.get_hit_seqs(blastout, tmpfile, read_fasta)
		print "tmp blastresult file:%s, hitcount:%s" % ( blastout, len(hitseqs))
		try:
			hitseq, = hitseqs
			readname = hitseq.name
			readseq = hitseq.sequence
			mut_nt = readseq[(mut_pos-1)*3:mut_pos*3]
			mut_aa = DNA.DNA(mut_nt).translate()
			aad.setdefault(mut_aa, []).append(clone.name)
			welld[clonename] = (mut_nt, mut_aa)
			print clone.name, mut_nt, mut_aa
		except:
			failed.append(clonename)
			continue
	# get failed 
	pprint.pprint(welld)
	return (aad, failed, welld)


#========================================================================================================
# get reference
#========================================================================================================
def get_ref(optd):
	refd = {}
	if not optd.has_key('r'):
		return refd
	allseqs = Fasta.all(optd['r'])
	ntname=aaname=''
	if optd.has_key('N'):
		ntname = optd['N']
	if optd.has_key('G'):
		ntname = optd['G']
	if not ntname and not aaname:
		return refd

	for a in allseqs:
		if a.name == ntname:
			refd[a.name] = a.sequence
			continue
		if a.name == 'aaname':
			refd[a.name] = a.sequence
	return refd

def report_103111(contig_file, optd, refd, outdir, platename):

	sampled = get_sampled(contig_file, optd, refd, outdir, platename)

	loc = optd['l']	
	loci = int(loc)-1

	aad = {}
	for s in sampled.items():
		s.translate()
		aa=s.get_mutation(loci)
		aad.setdefault(aa, []).append(s.name)

	for aa, clones in aad.items():
		set_toppick(clones)


	for aa in _20AAS:
		if aad.has_key(aa):
			clones = aad[aa]
			for c lin clones:
				str = c.format_result()
		else:
				str = aa

			
def get_sampled(contig_file, optd, refd, outdir, platename):
	sampled = {}

	# trim according to reference
	if optd.has_key('N'):
		ntname = optd['N']
		refnt = ''
		if refd.has_key[ntname]:
			refnt = refd[ntname]
		if refnt:
			outfile = os.path.join(outdir, "%s_nt.fas" % platename)
			trim2ref(ntname, refnt, contig_file, outfile, outdir)
		for a in Fasta.all(outfile):
			s = Sample(name=a.name, ntseq = a.sequence)
			sampled[s.name] = s


	# trim according to reference
	if not optd.has_key('G'):
		return samples

	ntname = optd['G']
	refnt = ''
	print "XXX: 1"
	if not refd.has_key(ntname):
		return samples

	refnt = refd[ntname]
	outfile = os.path.join(outdir, "%s_cds.fas" % platename)
	trim2ref(ntname, refnt, contig_file, outfile, outdir)
	for a in Fasta.all(outfile):
		if not sampled.has_key(s.name):
			s = Sample(name=a.name, cdsseq = a.sequence)
			sampled[s.name] = s
		else:
			sampled[s.name].cdsseq = a.sequence

	return sampled
	"""
	# get translation
	aafile = "%s_aa.fas" % platename
	print aafile
	outfile = os.path.join(outdir, aafile)
	aas = Fasta.all(contig_file)
	F = open(outfile, 'w')

	for a in aas:
		aaseq = DNA.DNA(a.sequence).translate()

		# do we have mutation location
		if loc and len(aaseq) >= loc:
			aa = aaseq[loc]
			mut_aad.setdefault(aa, []).append(a.name)

		i = aaseq.find('*')
		if i == -1:
			i = len(aaseq)
		else:
			if i < len(aaseq):
				i += 1
		aaseq = aaseq[:i]
		print >>F, '>%s\n%s' % (a.name, aaseq)
	F.close()


	# get mutations
	#pprint.pprint(mut_aad)
	print "Variant count:", len(mut_aad.keys())
	create_report(loc+1, wt_nt, wt_aa, mut_aad, platename, outdir, {})
	"""

def create_report(mut_pos, wt_nt, wt_aa, aa_d, platename, outdir, cloned):
	wts = aa_d[wt_aa]
	print "WT %s: %s: %s" % (wt_aa, len(aa_d[wt_aa]), aa_d[wt_aa])


	for aa in _20AA:
		if aa == wt_aa:
			continue

		clones = []
		if aa_d.has_key(aa):
			clones = aa_d[aa]
		print "Variant %s: %s: %s" % (aa, len(clones), clones)

def trim2ref(refname, refseq, contig_fasta, outfile, tmpdir):
	ntfile = os.path.join(tmpdir, 'ref_%s.fas' % refname)
	NT = open(ntfile, 'w')
	print >>NT, '>%s' % refname
	print >>NT, refseq
	NT.close()
	align.trim2ref(ntfile, contig_fasta, outfile)	


def create_report_old(mut_pos, wt_nt, wt_aa, aa_d, platename, outdir, cloned):
	# write all nt sequence for checking 
	logf =	'%s_mut%s_notes.txt' % (platename, mut_pos)
	print "\n\nCREATE LOG FILE", logf 
	flog = open(os.path.join(outdir, logf), 'w')
	print >>flog, time.ctime()
	print >>flog, "Plate:", platename

	print >>flog, "================================================="
	print >>flog, " Amino acid permutation position:", mut_pos
	print >>flog, "================================================="

	# wild types

	wt_clones = []
	wtcount = 0

	variants = len(aa_d.keys())
	if aa_d.has_key(wt_aa):
		wt_clones = aa_d[wt_aa]
		wt_clones.sort()
		wtcount = len(wt_clones)
		variants -= 1
	
	print "WT:", wt_clones
	return
	print >>flog, "------------------------------------------------"
	print >>flog, " Found %s variants" % variants
	print " Found %s variants" % variants, len(aa_d.keys())
	print >>flog, ""
	print >>flog, "  wild type: amino acid=%s, base=%s, count: %s" % (wt_aa, wt_nt, wtcount) 

	# write wild type to log file, sort by row order
	tmp = {} 
	print "Wild type wells:", wt_clones
	for w in wt_clones:
		clone = cloned[w]
		well = clone.well()
		r = re.sub('\d+', '', well)
		tmp.setdefault(r, []).append(well)
	xs = tmp.keys()
	xs.sort()
	for x in xs:
		print >>flog, "\t", ", ".join(tmp[x])

	mut_wells = []
	mtcount = 0
	mt_aas = []

	for aa, clones in aa_d.items():
		print aa, clones
		wells = [cloned[name].well() for name in clones]
		mtcount = len(wells)

		wells.sort()
		for well in wells:
			well2mut.setdefault(well, {})
			well2mut[well]['mutaa'] = aa

		if aa == wt_aa:
			continue
		mt_aas.append(aa)
	
		print "  mutation: %s, count:%s, %s" % (aa, len(wells), wells )
		print >>flog, "  mutation: %s, count:%s" % (aa, len(wells)), wells

	missing = [x for x in _20AA if x not in aa_d.keys()]

	print "MISSING: %s" % missing

	keys = cloned.keys()
	total_seq_count = len(keys)
	failed =[]
	good_seq_count = total_seq_count - len(failed)
	
	print >>flog, " Missing ", ", ".join(missing)
	print >>flog, "================================================="
	print >>flog, "\nSequencing Stat:"
	print >>flog, notes
	print >>flog, "Used %s sequences to categorize mutation." % good_seq_count
	print >>flog, "Cannot identify mutation in:", failed
	i = 0

	failed_clones=[]
	print >>flog, "These clones did not form contig:"
	for well in failed_clones:
		i += 1
		clone = cloned[well]
		status = []
		for r in clone.reads:
			if not r.status:
				status.append(r.status)
			
		print >>flog, "==>%s: %s, " % (i, well) 
	
		for s in clone.reads:
			print >>flog, "  seqfile: %s, score: %s, read_length: %s"  % (s.sample, s.score(), s.trimlength()) 
	print >>flog, "================================================="


	top_aa = os.path.join(reportdir, '%s_mut%s_toppick_aa.fas'% (libname, mut_pos))
	ftop_aa = open(top_aa, 'w')
	top_na = os.path.join(reportdir, '%s_mut%s_toppick_na.fas'% (libname, mut_pos))
	ftop_na = open(top_na, 'w')

	"""
	print >>ftop_na, ">refcds" 
	print >>ftop_na, CDS
	print >>ftop_aa, '>refaa'
	print >>ftop_aa, refaa
	"""

	n = 1
	# write details information

	summary_d = {}
	for aa, clonenames in aa_d.items():
		# check other mutations
		tops1 = []; tops2 = []

		wells = [x.split('-')[-1] for x in clonenames]
		print >>flog, "%s: %s %s" % (n, aa, wells)
		print "%s: %s %s" % (n, aa, clonenames)

		s = (mut_pos-1)*3
		e = s + 3
		for name in clonenames:
			clone = cloned[name]
			summary_d.setdefault(name , {'mutaa':aa, 'mutnt':'', 'other_mutation':'', 'top_pick':''} )

			if aa == 'X' or aa == 'x':
				summary_d[name]['other_mutation'] = 'Yes'
			elif other_mut_d.has_key(name):
				tops2.append(clone)
				summary_d[name]['other_mutation'] = 'Yes'
			else:
				tops1.append(clone)
				summary_d[name]['other_mutation'] = 'No'
				#summary_d[name]['top_pick'] = 'Top Pick'
				#print >>ftop_na, '>%s\n%s' % (clone.name, clone.cds.sequence)
				#print >>ftop_aa, '>%s\n%s' % (clone.name, clone.cds.translate())

			if clone.cds:
				mut_nt = clone.cds.sequence[s:e]
			elif well2mut.has_key(clone.name):
				mut_nt, mut_aa = well2mut[clone.name]
			else:
				mut_nt = ''
			print name, mut_nt, aa
			summary_d[name]['mutnt'] = mut_nt 
					
		top_picks = []
		if tops1:
			# order by sum of score
			tops1.sort(lambda x,y: cmp(y.score(), x.score()) )
			top_picks += tops1[:3]	
		else:
			tops2.sort(lambda x,y: cmp(y.score(), x.score()) )
			top_picks += tops2[:3]

		for clone in top_picks:
			if other_mut_d.has_key(clone.name) and other_mut_d[clone.name] in ['No Contig', 'FrameShift']:
				continue
			tmp = [clone.well()]
			for r in clone.reads:
				template, primer = r.get_template_primer()
				tmp.append('%s_score=%s' % (primer, r.score()))
			
			tmp.append('base_changedto=%s' % mut_nt)
			if other_mut_d.has_key(clone.name):
				tmp.append('other_mutation=Yes' )
			else:
				tmp.append('other_mutation=No' )
				summary_d[clone.name]['top_pick'] = 'Top Pick'
				print >>ftop_na, '>%s\n%s' % (clone.name, clone.cds.sequence)
				print >>ftop_aa, '>%s\n%s' % (clone.name, clone.cds.translate())

			print >>flog, '  top pick: %s' % ", ".join(tmp)		

		n += 1
	ftop_na.close()
	ftop_aa.close()
	flog.close()
	 
	print "\n.... write summary file summary_%s.txt" % libname

	fout = open(os.path.join(reportdir, 'summary_%s.txt' % libname), 'w')
	headers1 = ['Well']
	# get primer
	clonename = aa_d.values()[0][0]
	clone = cloned[clonename]
	for r in clone.reads:
		_,primer = r.get_template_primer()
		headers1.append(primer)

	headers2 = ['mutaa', 'mutnt', 'other_mutation', 'top_pick']
	SUMMARY_HEADERS = headers1 + headers2
	print "HEADERS:", SUMMARY_HEADERS
	print >>fout, '\t'.join(SUMMARY_HEADERS)

	clonenames = cloned.keys()
	clonenames.sort()
	for clonename in clonenames:
		clone = cloned[clonename]
		if summary_d.has_key(clonename):
			summaryd = summary_d[clonename]
			tmp = [clone.well()]
			for r in clone.reads:
				tmp.append(r.score())
			for h in headers2:
				tmp.append(summaryd[h])
			print tmp
			tmp = [str(x) for x in tmp]
			print >>fout, "\t".join(tmp)
		else:
			print "ERROR: clonename:%s not in summary_d" % clonename
	fout.close()

	# create alignment file for top picks
	align.do_muscle(top_aa, 'html')
	align.do_muscle(top_na, 'html')

def create_summary(notefile, seqfile):
	print "notefile:", notefile
	lines = open(notefile).readlines()
	mut_pos, = re.findall('\d+', lines[0])

	for i in range(0, len(lines[1:])): 
		l = lines[i].strip()
		if re.match('Missing', l):
			break
	i += 1
	lines = lines[i:]

	well_d = {}

	for l in lines:
		l = l.strip()
		if re.match('top pick', l):
			l = l.replace('top pick:', '')
			cols = [x.strip() for x in l.split(',')]
			well, fwd_score, rev_score = cols[:3]
			fwd_score = eval(fwd_score.replace('fwd_score=', ''))
			rev_score = eval(rev_score.replace('rev_score=', ''))
			well_d[well] += [fwd_score, rev_score]
			continue

		cols = l.split(':')
		#print cols
		if len(cols) != 2:
			continue
		tmp = cols[1]
		# get wells
		idx = tmp.find('[')
		if idx == -1:
			continue

		aminoacid = tmp[:idx].strip()

		wells = eval(tmp[idx:].strip())
		for well in wells:
			well_d[well] = [aminoacid] 

	# get amino acid sequence
	text = open(seqfile).read()
	blocks = text.split('>')
	for b in blocks[1:]:
		#print b
		lines = b.split('\n')
		name = lines[0]
		seq = "".join(lines[1:])
		if not well_d.has_key(name):
			continue
		well_d[name].append(seq)

	wells = well_d.items()
	wells.sort()

	f = seqfile.replace('_aa.fas', '_summary.txt')
	fout = open(f, 'w')
	#print f

	print >>fout, "\t".join(['Sample Well', 'Amino Acid', 'Forward Score', 'Reverse Score', 'Sequence'])
	for (w, values) in wells:
		if len(values) < 4:
			continue
		print >>fout, "\t".join([str(x) for x in [w]+values])
	
def get_contig(tmpd, libname, well, rawseq_list, overide=0):
	'''	
	if not os.path.isdir(tmpd):
		os.mkdir(tmpd)

	inf = os.path.join(tmpd, '%s_%s.fas' % (libname, well) )
	capfile = '%s.cap' % inf
	try:
		capfile_mtime = os.path.getmtime(capfile)
	except:
		capfile_mtime=0

	seqs = []
	#startseq = refnt[:20].upper()
	startseq = STAG	
	#for (dname, sample) in rawseq_list:
	for sample in rawseq_list:
		pat = os.path.join(sample.dir, '%s.seq' % sample.sample_file_name)
		seqfile = glob.glob(pat)
		seqfile, = seqfile

		seq = open(seqfile).read().upper()
		seq = re.sub('\s+', '', seq)
		seqname = os.path.basename(seqfile) 

		if seq.find(startseq) != -1:
			seqs.insert(0, (seqname, seq))
		else:
			seqs.append((seqname, seq))
		
		sf_mtime = os.path.getmtime(seqfile)

		if sf_mtime > capfile_mtime:
			overide = 1

	#print "OVERIDE=%s" % overide, len(seqs)
	# last assembled file
	if overide:
		print "Overide %s:" % well, overide
		fin = open(inf, 'w')
		#print "fasta file:", inf
		for (seqname, seq) in seqs:
			print seqname, seq[:30] 
			print >>fin, '>%s' % seqname
			print >>fin, seq 

		fin.close()
		os.system('%s %s %s.cap 25 80' % (CAP, inf, inf))

	txt = open('%s.cap' % inf).read()
	seqs = txt.split('>')
	contigs = []
	for s in seqs:
		if re.match('Contig', s):
			contig_seq = "".join(s.split('\n')[1:])
			contigs.append(contig_seq) '''	
	contigs = get_contig_new(tmpd, well, rawseq_list)

	coding_seq = ''
	if len(contigs) == 1:
		seq0, = contigs
		seq = seq0.upper()
		s = seq.find(STAG); e = seq.find(ETAG)
		if s == -1 or e == -1:
			print "%s: TAG in contig seq:" % well, s, e
		if s != -1:
			if e != -1:
				seq = seq[s:e+len(ETAG)]
			else:
				seq = seq[s:]


			# get translated region
			if not CDS_START or not CDS_END:
				aa = DNA.DNA(seq).translate()
				coding_seq= seq
			else:
				#print seq[:30], seq[-30:]
				#print "S:%s, E:%s"% (CDS_START, CDS_END)
				x = seq.find(CDS_START)
				y = seq.find(CDS_END) 
				if y != -1:
					y += len(CDS_END)
				coding_seq = seq[x:y]
				aa = DNA.DNA(coding_seq).translate()
				#print aa
		else:
			aa = ''
	else:
		seq0 = aa = ''

	return (seq0, aa, coding_seq)

def get_top_hits(seqd, mut_pos, seqs):
	refaa_list = list(refaa)

	refaa_list[mut_pos-1] = '.'
	top3 = []

	top_1s = []; top_2s=[]; not_sogood = []

	msg = []
	for (well, dnaseq, aaseq) in seqs:
		myseqs = seqd[well]
		brk = 0
		for myseq in myseqs:
			if myseq.sample_score < 40:
				#print "\t\tNOT SO GOOD:", well
				not_sogood.append(myseq)
				brk = 1
				break;	
		if brk:
			continue
		print well, dnaseq[:30], aaseq[:10]
		aa_list = list(aaseq)
		#if len(aa_list) < mut_pos or len(aa_list) < 480: # the length cutoff
		if len(aa_list) < mut_pos :
			print "  %s: ERROR, mut_pos=%s, aalen=%s" % (well, mut_pos, len(aa_list) )
			msg.append('deletion')
			continue
		if abs(len(dnaseq) - len(CDS)) > 30:
			msg.append('insertion')
			print "... len of DNA is too long/short: dif=%s" % (len(dnaseq)-len(refnt))
			continue

		aa_list[mut_pos-1] = '.'

		# all have E30G mutation
		aa_list[29] = '.'
		refaa_list[29] = '.'
		#print 'ref:', refaa
		#print '%s :' % well, aaseq
		if aa_list == refaa_list:
			top_1s.append((well, dnaseq, aaseq, 1))
		else:
			top_2s.append((well, dnaseq, aaseq, 0))
	print "\tTOP", [x[0] for x in top_1s]
	return (msg, top_1s, top_2s+not_sogood)


def translate(file):
	fin = open(file)
	names = []
	n2s={}
	while 1:
		l = fin.readline()
		if not l:
			break

		if re.match('>', l):
			name = l.strip().replace('>', '')
			names.append(name)
			n2s.setdefault(name, '')
		else:
			n2s[name] += l.strip()

	out =file.replace('na', 'aa')
	faa = open(out , 'w')
	for n in names:
		s = n2s[n].upper()
		m = re.search('[^GATC]', s)
		if m:
			print n, m.start(), s[m.start():m.end()]

		s = re.sub('[^GATC]', '',s)
		aa = DNA.DNA(s).translate()
		i = aa.find('*')
		if i != -1:
			print n, i*3, s[i*3:(i+1)*3]
		print >>faa, '>%s' % n
		print >>faa, aa
	faa.close()

	notes = out.replace('toppick_aa.fas', 'notes.txt')


def top_hits(dname):
	files = glob.glob(os.path.join(dname, "*.seq"))

	lib_d = {}
	for f in files:
		basename = os.path.basename(f)
		sample = basename.split('_')[0]
		lib, well = sample.split('-')

		#print basename, sample, lib, well
		seq = open(f).read()
		seq = re.sub('\s+', '', seq)

		lib_d.setdefault(lib, []).append((basename, seq))

	for lib, seqs in lib_d.items():
		# find sequences from other primer run
		dirs = glob.glob('*%s*' % lib)
		#print lib, len(seqs), dirs
		for d in dirs:
			if not os.path.isdir(d):
				continue


def rename(change=0):
	files = []
	for ext in ['seq', 'ab1', 'phd.1']:
		files = glob.glob("*.%s" % ext)
		for f in files:
			'''	
			well, = re.findall('[A-H]\d\d_', f)
	
			name = '%s-%s%s.%s' % (libname, well, primer, ext)
			print f, name
			if change:
				os.rename(f, name)
			'''	
			cols = f.split('_')
			if len(cols) < 5:
				continue
			plate, primer, date, well, cap = cols

			if cols[0].find(well) == -1:
				cols[0] = cols[0]+'-%s' % well

			name = "_".join(cols[:2])+'.%s' % ext

			print f, name
			if change:
				try:
					os.rename(f,name)
				except:
					continue



def get_goodseqs(platename, fdir, rdir, assemble=0, tmpd=''):
	print "... get good sequences" 
	check_seqs = []
	goodseqs = []

	fwd_d = get_seq(fdir)
	rev_d = get_seq(rdir)

	if not tmpd:
		tmpd = "tmp_%s" % platename

	if not os.path.isdir(tmpd):
		os.mkdir(tmpd)

	assemble_sequence(tmpd, platename, fdir, rdir, fwd_d, rev_d)

	files = glob.glob("%s/*.cap" % tmpd)
	seqd = {}

	for f in files:
		contigs = [] 
		fname = rname = ''
		lines = open(f).readlines()
		for  i in range(0, len(lines)):
			l = lines[i]
			if re.match('>Contig',l):
				name = os.path.basename(f.replace('.cap', ''))
				i += 1
				seq = "".join(lines[i:])
				seq = re.sub('\s+', '', seq)
			
				if seq.find('-') != -1:
					check_seqs.append(name)
					break
				else:
					goodseqs.append((name, seq, fwd_d[name][0], rev_d[name][0]))
	if check_seqs:
		print 'require manual check ===>>>>>' , len(check_seqs)
		print ", ".join(check_seqs)
	return goodseqs

def report_E4_12V(platename, fdir, rdir, mut_pos=''):

	reportdir = 'report'
	if not os.path.isdir(reportdir):
		os.mkdir(reportdir)

	fnt = open(os.path.join(reportdir, '%s_nt.fas' % platename), 'w')
	faa = open(os.path.join(reportdir, '%s_aa.fas' % platename), 'w')

	refnt = open('E4_nt.fas').read()
	refaa = open('E4_aa.fas').read()

	print >>fnt, refnt
	print >>faa, refaa

	goodseqs = get_goodseqs(platename, fdir, rdir, 1)

	pos = [26, 26, 91,91,91,91, 259,298,333, 449, 479, 479]
	aaseqs = []
	myd = {}
	for (name, seq) in goodseqs:
		l = len(seq)

		noaa=''
		if l < 1458:
			print "%s: %s bp insert deletion" % (name, 1458-l)
			noaa ='%sbp-deletion' % (1458-1); aa =''
		elif l > 1458:
			print "%s: %s bp insertion" % (name, l-1458)
			noaa = '%sbp-insertion' % (l-1458); aa =''

		aa = DNA.DNA(seq).translate()
		print name, "aa sequence length %s " % len(aa)

		print >>fnt, '>%s' % name
		print >>fnt, seq
		if aa:
			print >>faa, '>%s' % name
			print >>faa, aa

		c, = re.findall('\d+', name)
		r = name.replace(c,'')
		cn = int(c)
	
		# get mutation position
		p = pos[cn-1]

		if noaa:
			a = noaa
		else:
			a = aa[p-1]

		myd.setdefault(r, {})
		myd[r].setdefault(c, a)

	fnt.close()
	faa.close()

	variant = 0
	if variant:
		report_E4Variants(platename, reportdir, myd)


def report_E4Variants(platename, reportdir, myd):
	# mutation position
	pos = [26, 26, 91,91,91,91, 259,298,333, 449, 479, 479]

	fout = open(os.path.join(reportdir,'%s_summary.txt' % platename), 'w')
	print >>fout, "\t".join([""]+COLS)
	for r in ROWS:
		rows = [r]
		for c in COLS:
			try:
				v = myd[r][c]
			except:
				v = ''
			rows.append(v)
		print >>fout, "\t".join(rows)
	fout.close()
	
	os.system('%s -in %s_nt.fas -out %s_nt.aln -clwstrict' % (MUSCLE, platename, platename))
	os.system('%s -in %s_aa.fas -out %s_aa.aln -clwstrict' % (MUSCLE, platename, platename))

def assemble_sequence(tmpd, platename, fdir, rdir, fwd_d, rev_d):
	stag = 'gccgcaccgttt'.upper()
	etag = 'cctagaaaaacgacc'.upper()

	refseq = refnt

	print ".... processing %s" % fdir
	#fwd_d = get_seq(fdir)
	fout = open(os.path.join(tmpd, '%s.fas' % fdir), 'w')
	fseq_d = {}

	fail_d = {}
	for k, v in fwd_d.items():
		(score, seq) = v
			
		start = seq.find(stag)
		if start == -1 or len(seq) < 750:
			fail_d.setdefault(k, []).append('F: score=%s, seqlen=%s' % (score, len(seq)) )
			continue

		seq = seq[start:start+800]
		print >>fout, '>%s' % k
		print >>fout, seq
		fseq_d[k] = seq
	fout.close()

	print "\n.... processing %s" % rdir
	#rev_d = get_seq(rdir)
	rseq_d = {}
	fout = open(os.path.join(tmpd, '%s.fas' % rdir), 'w')
	for k, v in rev_d.items():
		(score, seq) = v
		seq = DNA.DNA(seq).reverse_complement()
		end = seq.find(etag)
		if end == -1 or len(seq)<750:
			print "%s: cannot find end sequence tag" % k
			fail_d.setdefault(k, []).append('R: score=%s, seqlen=%s' % (score, len(seq)) )
			continue
		#print k, score, end 

		e = end + len(etag)
		s = e-800
		seq = seq[s:e]
		print >>fout, '>%s' % k
		print >>fout, seq
		rseq_d[k] = seq
	fout.close()

	if not os.path.isdir(tmpd):
		os.mkdir(tmpd)

	# write fails
	fout = open(os.path.join(tmpd, 'failed_%s.txt' % platename), 'w')

	print >>fout, platename
	keys = fail_d.keys()
	keys.sort()
	for k in keys:
		v = fail_d[k]
		print >>fout, "%s: %s" % (k, ", ".join(v))
	fout.close()

	files = glob.glob(os.path.join(tmpd, "*.cap"))
	if not files:
		# assemble sequence
		for k, fseq in fseq_d.items():
			fout = open('%s/%s.fas' % (tmpd, k), 'w')
			#print >>fout, refseq
			print >>fout, '>%s_seqF1' % k
			print >>fout, fseq
			if rseq_d.has_key(k):
				rseq = rseq_d[k]
				print >>fout, '>%s_seqR1' % k
				print >>fout, rseq

			fout.close()
			os.system('%s %s/%s.fas %s/%s.cap 25 80' % (CAP,tmpd, k, tmpd, k) )

	return files


def report_E4(platename, fdir, rdir):
	#fwd_d = get_seq(fdir)
	#rev_d = get_seq(rdir)
	goodseqs = get_goodseqs(platename, fdir, rdir)

	aa_d = {}
	fnt = open('%s_nt.fas' % platename, 'w')
	faa = open('%s_aa.fas' % platename, 'w')

	print >>fnt, '>refnt'
	print >>fnt, refnt

	print >>faa, '>refaa'
	print >>faa, refaa
	bads = []
	for (name, seq, fscore, rscore) in goodseqs:
		l = len(seq)
		bad = 0

		if l < 1458:
			print "%s: %s bp deletion" % (name, 1458-l)
			bad = 1
			bads.append((name, seq, fscore, rscore))
		elif l > 1458:
			print "%s: %s bp insertion " % (name, l-1458)
			bad = 1
			bads.append((name, seq, fscore, rscore))

		
		aa = DNA.DNA(seq).translate()

		print >>fnt, '>%s' % name
		print >>fnt, seq
		print >>faa, '>%s' % name
		print >>faa, aa

def check_duplicate():
	files = glob.glob("*.ab1")
	well_d = {}
	for f in files:
		cols = f.split('_')
		well = cols[-2]
		well_d.setdefault(well, []).append(f)

	wells = well_d.keys()
	wells.sort()

	if not os.path.isdir('DUP'):
		os.mkdir('DUP')

	for well in wells:
		files = well_d[well]
		if len(files) <=1:
			continue
		for f in files:
			print f
			os.rename(f, 'DUP/%s' % f)


def do(args):
	usage = """
	Genecor.py [option] <data dir list> 
		-a: process all subdirectories
		-c: clean start (remove tmp* report*)
		-O: overide previous assembly
		-s: create summary
		-l: mutated aa position
		-p: platename (libray  name)
		-r <full path to ref fasta>
		-N <nt refname> 
		-G <cds refname>
		-f <note file> [well=notes per line]
		-F <forward primer>
		-R <reverse primer>

		-v: verbose
		-D: create download file for all plates
	"""
	optd, ars = util.parse_args(args, 'A:aCcDf:F:r:R:Hp:l:Ot:s:N:v:G:', usage)
	if not ars:
		print usage
		sys.exit()

	# get reference file
	refd = get_ref(optd)
	platename = os.path.basename(os.path.dirname(os.path.abspath(os.path.curdir)))
	if optd.has_key('p'):
		platename = optd['p']

	# do assembly
	dirlist = ars
	outdir = 'report_%s' % platename	
	#contig_file = CloneAssembler.do_assembly(outdir, dirlist, outdir=outdir, platename=platename)
	contig_file = os.path.join('report_GG72-prosite6', 'GG72-prosite6_cds.fas')
	report_052208(contig_file, optd, refd, outdir, platename)

if __name__ == '__main__':
	do(sys.argv[1:])

