#!python

import os, sys, re, pprint, glob
from qblib import Fasta, stats, DNA, align
import CloneAssembler 

"""
generate specifif stats for Sherry
"""

class Mutation:
	def __init__(self, name, align_start, align_end, seq_type='nt'):
		self.name = name
		self.align_start = align_start
		self.align_end = align_end
		self.align_len = align_end-align_start+1
		self.seq_type = seq_type
		self.tag = ''
		self.mutations = []

	def header(self):
		return ['name', 'align_start', 'align_end', 'align_len', 'tag', 'mutation_count', 'mutation_rate/1000%s' % self.seq_type, 'mutations']

	def data(self):
		mutcnt = len(self.mutations)
		alignlen = self.align_len
		tmp = [self.name, self.align_start, self.align_end, self.align_len, self.tag, mutcnt, 1000.0*mutcnt/alignlen] + self.mutations
		return [str(x) for x in tmp]

def dostats(ntalign, outfile_sufix, aalign='', ntfasta=''):
	start = 61; end = 850
	if not os.path.isfile(ntalign) and os.path.isfile(aalign):
		return _do_aa(aalign)
	allnts = Fasta.all(ntalign)
	print "--> Sequence count:", len(allnts)

	nt0 = allnts[0]
	refname = nt0.name
	refseq = nt0.sequence.upper()

	icnt0 = refseq[:start].count('-')
	icnt1 = refseq[:end].count('-')

	res = []

	name2tag = {}
	for nt in allnts[1:]:
		name = nt.name; seq = nt.sequence.upper()
		print name, refname, len(seq)

		# find alignment start and end
		m = re.search('[GATC]+', seq, re.IGNORECASE)
		if m:
			start = m.start()+1
		seq2 = re.sub('^[\~\-]', '', seq)
		seq2 = re.sub('[\~\-]+$', '', seq2)
	
		i = len(seq2)-1
		while i >= 0 :
			if re.match('[AGTC]', seq2[i], re.IGNORECASE ):
				end = i+1
				break
			i -= 1

		checkend = min([end, len(refseq)])
		ntmut = Mutation(nt.name, start, checkend)
		tag = ''
		details = []
		dcnt = 0
		print name, "Alignment start=%s, end=%s" % (start, checkend)
		for i in  range(start, checkend):
			icnt = refseq[:i].count('-')
			if refseq[i] != seq[i]:
				x =  "%s%s%s" % (refseq[i], i-icnt+1, seq[i])
				details.append(x)
				dcnt += 1
				#print "R:", refseq[(i-10):(i+10)], refseq[i:(i+30)]
				if seq[i] in ['n', 'N']:
					tag = 'mixed bases'

				#print 'M:', seq[(i-10):(i+10)], seq[i:(i+30)] 
				#print i, refseq[i], seq[i]
				#sys.exit()

			if i == checkend -1:
				my_dashcnt = seq[:i].count('-')
				dash_diff = my_dashcnt - icnt
				if dash_diff < 0:
					tag = 'insertion'
				elif dash_diff > 0:
					tag = 'deletion'
				'''
				# this is overstate frameshift
				xx = refseq[i:end].count('-')
				if (dash_diff-xx)%3 != 0:
					tag = 'frameshift ' + tag
				'''
		#res.append(tmp+[tag]+details)
		if not dcnt:
			tag = 'WT'

		ntmut.tag = tag
		ntmut.mutations = details
		res.append(ntmut)
		name2tag[name] = tag

	#print >>Fout, "\t".join(['seqname', 'align_start', 'align_end', 'align_len', 'tag', 'mutation_count', 'mutation_rate/1000nt)', 'mutations'])
	#_write_output(Fout, res, '1000nt')
	filename = 'nt_%s' % outfile_sufix
	_write_output(filename, res, ntfasta)

	# process protein sequence
	# read nt file, remove gap, do translation, then do stat
	if not os.path.isfile(aalign):
		#aalign = 'aa_align.fas'
		aalign = ntalign.replace('_cds_', '_aa_')
		print "NT:", ntalign
		print "AA:", aalign
		if aalign == ntalign:
			aalign = ntalign.replace('.', '_aa.')
		F = open(aalign, 'w')
		allnts = Fasta.all(ntalign)
		i = 0
		for a in allnts:
			i += 1
			#seq = re.sub('[^atgcmrwsykvhdbxnATGCMRWSYKVHDBXN]', '', a.sequence)
			seq = a.sequence
			# need to make sure we are translating in the right frame
			#print a.name, a.sequence
			n = len(seq) % 3
			l = len(seq)-n
			d = DNA.DNA(seq[:l])
			aa = d.translate()
			#print "\t", a.name, len(seq), "nts", aa
			print a.name, aa
			if i > 2:
				sys.exit()
			print >> F, '>%s\n%s' % (a.name, aa)
		F.close()
	return _do_aa(aalign, name2tag, outfile_sufix=outfile_sufix)

def _do_aa(aalign, name2tag={}, outfile_sufix='mutation_summary.txt'):
	allaas = Fasta.all(aalign)
	nt0 = allaas[0]
	refname = nt0.name
	refseq = nt0.sequence
	res = []
	for nt in allaas[1:]:
		name = nt.name; seq = nt.sequence
		
		m = re.search('[A-Z]', seq, re.IGNORECASE)
		start = m.start()+1
		i = len(seq)-1
		while i >= 0 :
			if re.match('[A-Z]', seq[i], re.IGNORECASE ):
				end = i+1
				break
			i -= 1

		#seq = re.sub('^[\~\-]', '.', seq)
		#seq = re.sub('[\~\-]+$', '', seq)

		checkend = min([end, len(refseq)])
		ntmut = Mutation(nt.name, start, checkend)
		tag = ''

		#print nt.name ,  start, end2, end2-start-icnt1
		#need to figure out how to detect mutation here
		details = []
		indexs = []
		tag = ''
		for i in  range(start, end):
			icnt = refseq[:i].count('-')
			if len(refseq) <= i or len(seq) <= i:
				continue
			if refseq[i] != seq[i]:
				_r = refseq[i]
				_m = seq[i]
				if _r != '*' and _m  == '*':
					tag = 'nonsense'
				elif _m in ['x', 'X']:
					tag = 'mixed bases'
				x =  "%s%s%s" % (refseq[i], i-icnt+1, seq[i])
				details.append(x)
				indexs.append(i)

		print name, details
		# check for frameshift here
		indexs.reverse()
		#print indexs
		cnt = 0
		if not details:
			tag = 'WT'
		if indexs:
			d0 = indexs[0]
			for x in indexs[1:]:
				if d0-x == 1:
					d0 = x
					cnt += 1
				else:
					break
		#print cnt 	
		try:
			ntmut.tag = name2tag[name]
		except:
			ntmut.tag = ''
		if tag:
			if ntmut.tag and ntmut.tag != tag:
				ntmut.tag += ' %s' % tag
			else:
				ntmut.tag = tag

		if cnt > 3:
			ntmut.tag = 'frameshift %s' % ntmut.tag

		ntmut.mutations = details
		res.append(ntmut)
	filename = 'aa_%s' % outfile_sufix
	#print >>Fout, "\t".join(['seqname', 'align_start', 'align_end', 'align_len', 'tag', 'mutation_count', 'mutation_rate/1000aa', 'mutations'])
	_write_output(filename, res)
	#Fout.close()
	return aalign
	
def _write_output(filename, res, ntfasta=''):
	if not res:
		print "No mutation found"
		sys.exit()
	Fout = open(filename, 'w')
	print >>Fout, "\t".join(res[0].header())

	mutratios = []
	alignlens = []

	# detect frameshift here
	frameshift = []

	doned  = {}
	for t in res:
		doned.setdefault(t.name, 1)
		if t.tag.find('frameshift') != -1:
			frameshift.append(t.name)
			#print t[0]
		alignlens.append(t.align_len)
		mutratio = 1000.0* len(t.mutations)/t.align_len
		mutratios.append(round(mutratio))
		#t.insert(4, mutcnt)
		#t.insert(5, mutratio)
		#t = [str(x) for x in t]
		print >>Fout, "\t".join(t.data()) 

	# write failed clones or any clone that does not match to reference
	if ntfasta and os.path.isfile(ntfasta):
		allseqs=Fasta.all(ntfasta)
		for a in allseqs:
			if doned.has_key(a.name):
				continue
			else:
				print >>Fout, "\t".join([a.name, 'Not Used'])
	seqcnt = len(mutratios)
	minlen = min(alignlens)	
	maxlen = max(alignlens)	
	meanlen = stats.mean(alignlens)
	unit = '1000%s' % t.seq_type

	print >>Fout, "\nCompare %s sequence, length of sequence compared ranging from %s to %s, average length is %s" % (seqcnt, minlen, maxlen, meanlen)
	print >>Fout, "Mutation ratio per 1000 bases for nucleotide sequence (per 1000 aa for amino acid sequence)"

	print >>Fout, "Minimal mutation rate (count/%s): %s" % (unit, min(mutratios) )
	print >>Fout, "Maximal mutation rate (count/%s): %s" % (unit, max(mutratios))
	print >>Fout, "Average mutation rate (count/%s): %s" % (unit, round(stats.mean(mutratios)) )
	print >>Fout, "Meddian mutation rate (count/%s): %s" % (unit, stats.lmedianscore(mutratios))
	print >>Fout, "Most frequent mutation rate(frequency, mutation rate):", stats.lmode(mutratios)
	print >>Fout, "Frameshift mutation: %s" % ", ".join(frameshift)
	
	Fout.close()

def do_all(args):
	from qblib import util
	usage = """%s [option]
	optios:
	-r <reference fastafile>
	-N <refname for nt>
	-G <refname for CDS>, required
	-a <protein alignment file (fasta)>
	-n <necleotide alignment file <fasta>>
	-p <prefix> outfile will be nt_<prefix>.txt or aa_<prefix>.txt>

	SherryLi.py -n 073110_BDC_cds_align.fas
        SherryLi.py -r p:\\tools\\ref\\x264.fas -G x264 -p 081010_BDC_mutation.txt
	""" % (__file__)

	optd, ars = util.parse_args(args, "a:n:r:N:G:p:", usage)

	if optd.has_key('n'):
		# has alignment file to start with
		try:
			nt = optd['n']
		except:
			nt =''

		try:
			aa = optd['a']
		except:
			aa = ''
		try:
			pref = optd['p']
		except:
			pref = 'mutation_summary.txt'

		if nt:
			return dostats(nt, pref, aa)

	elif optd.has_key('a'):
		nt = ''
		aa = optd['a']
		try:
			pref = optd['p']
		except:
			pref = 'mutation_summary.txt'
		return _do_aa(aa)

	elif not optd.has_key('r'):
		print usage
		return

	# get sequence read into fasta file
	curd = os.path.abspath(os.path.curdir)

	#os.chdir(datadir)
	files = glob.glob("*.seq")
	if not files:
		print "No *.seq file in cwd" 
		sys.exit()

	dname = os.path.dirname(__file__)
	cat = os.path.join(dname, 'cat.py')

	mydir = os.path.basename(curd)

	seqf1 = "%s.fas" % mydir
	os.system("python %s -o %s ." % (cat, seqf1))
	print "%s -o %s ." % (cat, seqf1)

	seqf2 = CloneAssembler.trim2ref(optd['r'], refname=optd['G'], contig_file=seqf1, reftype='CDS', platename=mydir)
	cds_html = align.do_muscle(seqf2, 'html')
	cds_align_fas = align.do_muscle(seqf2, 'fas')

	aa_align_fas = dostats(cds_align_fas, optd['p'], ntfasta='')
	aa = aa_align_fas.replace('_align', '')
	if os.path.isfile(aa):
		os.remove(aa)
	os.rename(aa_align_fas, aa)
	aa_html = align.do_muscle(aa, 'html')



if __name__ == '__main__':
	do_all(sys.argv[1:])
	sys.exit()
	'''
	do_all(refile=sys.argv[1], datadir=os.path.curdir)

	nt = sys.argv[1]; 
	try:
		outfile = sys.argv[2]
	except:
		outfile = ""

	try:
		aa = sys.argv[3]
	except:
		aa =""
	dostats(nt, outfile, aa)
	'''
