#!/usr/local/bin/python



import glob, sys, os, re, pprint, time, getopt

from qblib import ABISeq

def get_trim_sequence(seqfile):
	# if this is a ab1 file
	if seqfile.endswith('.ab1'):
		ab1 = ABISeq(seqfile)
		ab1.trim_by_quality()
		return ab1.trimseq()
	else:
		seq = open(seqfile).read()
		lines = seq.split('\n')
		if re.match('>', lines[0]):
			seq = "\n".join(lines[1:])
		# remove spave, newline
		seq = re.sub('[\s+\\n]', '', seq)
		return seq
			
def cat2(pat="", long='', remove=0, outfile='', score=0, datadirs=[]):

	if not datadirs:
		datadirs = [os.path.abspath(os.path.curdir)]

	files = []
	for d in datadirs:
		files += glob.glob(os.path.join(d, ("*%s*.ab1" % pat) ) )

	if not files:
		files = glob.glob('*.seq')
	files.sort()



	print "File counts:", len(files)



	curdir = os.path.abspath(os.curdir)

	y, m, d = time.localtime()[:3]

	if outfile:

		pass

	elif pat:

		outfile = "all_%s_%02d%02d%d.fas" % (pat, m,d,y)

	else:

		#outfile = "all_%02d%02d%d.fas" % (m,d,y)

		dname = os.path.basename(os.path.abspath(os.curdir))
		outfile = "%s.fas" % dname
	print "Output file:", outfile





	fout = open(outfile, 'w')

	ext = '.seq'

	for f in files:

		basename = os.path.basename(f).replace('.ab1', '')

		seqfile = f.replace('.ab1', ext)

		#print "getting sequence from %s" % seqfile


		if not os.path.isfile(seqfile):

			print "  .. missing sequence file:", seqfile

			continue



		# if filename format is long, then keep current filename 

		name = basename

		if not long:

			# search for date string, remove everything afterward

			datepat = '\_\d{4}\-\d{2}\-\d{2}'

			m = re.search(datepat, basename)

			if m:

				name = basename[:m.start()]



		seq = get_trim_sequence(seqfile)
		seq = re.sub('\s+', '', seq)

		#seq = open(seqfile).read().strip()

		#print name, len(seq), 'remove:', remove
		if remove:

			if len(seq) < remove:

				print ".. skip %s" % name

				continue



		#print seq[:30]

		print >>fout, '>%s' % name

		for i in range(0, len(seq), 60):
			print >>fout, seq[i:i+60] 



	fout.close()

	return outfile



def do(args):

	try:

		opts, ars = getopt.getopt(args, 'p:lr:o:')

	except:

		print """Usage: cat.py [option] 

		-p: filename pattern to glob for

		-l: filename format in output fasta file

		-r <len>: skip sequence if length is less than "len"

		-o:  output file

		"""

		sys.exit()



	print opts



	long=''; pat=outfile=''; remove=0

	score = 0
	for o, v in opts:

		if o == '-l':

			long=1

		if o == '-p':

			pat = v

		if o == '-r':

			remove = int(v)

		if o == '-o':

			outfile = v

		if o == '-s':

			score = int(v)




	print "File name pattern:", pat

	print "File name format: ", long



	if not pat:

		print "Create a fasta file for all 'seq' files in current directory"

	else:

		print "Create a fasta file for all 'seq' files with %s in filename " % pat



	cat2(pat, long, remove, outfile=outfile, score=score)



if __name__=='__main__':

	do(sys.argv[1:])



