#!python

import os, sys, glob, shutil, time, re, pprint, platform
import move2folder, upload_goddy
from qblib import util 

# A=amur, B=BO, W=Bengal, J=Javan, M=MJ, P=Pu, T=T, V=Vena 
SRCDIRS = ['A:', 'B:', 'W:', 'J:', 'M:', 'P:', 'T:', 'V:']

# A=amur, BE=Bengal, BO=BO, J=Javan, MJ=MJ, Pu=Pu, T=T, V=Vena
SUFFIX = re.compile('\_((BO)|A|J|T|V|(MJ)|(Pu)|(BE))$')

# default sample sheet directory
SSDIR = "O:\\"
DESTDIR = "O:"
TIMEWINDOW = 24 # last 24 hours
keyfile = "O:\\tools\\keys\\ssh2_rsa.ppk"
ssh = "O:\\tools\\bin\\windows\\plink.exe"

def do_update(srcdir, datafolder,  destdir):
	srcd = os.path.join(srcdir, datafolder)

	destd = os.path.join(destdir, datafolder)
	if not os.path.isdir(destd):
		os.mkdir(destd)

	for root, dirs, files in os.walk(srcd):
		files.sort(lambda x,y: cmp(os.path.getmtime(x), os.path.getmtime(y)))
		for f in files:
			f2 = os.path.join(destd, f)
			if not os.path.isfile(f2):
				shutil.copy(f, os.path.join(destd, f2))
		for d in dirs:
			shutil.copytree(os.path.join(root, d), destdir)

def do_copy(srcd, srcdata, destd):
	#curdir = os.path.abspath(os.path.curdir)
	# check to make sure there are 96 data files before copying, 
	# data collection software migth be writing the data file

	src = os.path.join(srcd, srcdata)
	print "\twithin do_copy: src=%s" % src
	count = 0

	loop = 0
	while 1:
		files = glob.glob(os.path.join(src, "*.ab1"))
		c = len(files)
		if c < 96 and count < c:
			print "file count: %s, wait for 5 minutes for 3730 to finish writing ab1 files" % c
			time.sleep(5*60) # wait for 5 minutes
			count = c
		else:
			break

	shutil.copytree(os.path.join(srcd, srcdata), srcdata)

	# within DESTDIR (root directory)
	destdir = srcdata
	# get plateid
	pid = re.sub('^20\d\d\-\d\d\-\d\d\_', '', destdir)
	pid = SUFFIX.sub('', pid)
	ssheet = os.path.join(SSDIR, "%s.txt" % pid)
	
	scoretxt = ''
	print "--->Sample Sheet:", ssheet
	if os.path.isfile(ssheet):
		scoretext = move2folder.separate_files(destdir, ssheet)		
		# make it group writable
		os.system('chmod -R 775 %s' % destdir)
		upload2godaddy(ssheet, destdir, pid)
	else:
		print "!!! WARNING: no sample sheet (%s) available" % ssheet
	return scoretext

def upload2godaddy(ss,datadir,platename):
	print "====> upload (%s, %s) to godaddy" % (ss, datadir)
	datastr = " ".join([str(ss), str(datadir)])
	upload_goddy.upload_data(datastr, platename)
	
def cpdata(srcdirs = SRCDIRS, destdir=DESTDIR, timewindow=TIMEWINDOW):
	destd = get_dirs(destdir)
	#pprint.pprint(destd)

	# need to do the copy within DESTDIR?
	os.chdir(destdir)

	scorefile = 'autoscore_%s.txt' % util.today()
	SCORE = open(scorefile, 'a')

	datad = {}
	copy = 0
	for src in srcdirs:
		print "==>CHK %s" % src
		srcd = get_dirs(src, timewindow=timewindow)
		datad[src] = srcd
		for srcdata, srctime in srcd.items():
			if not destd.has_key(srcdata):
				print "  copy %s\%s to %s" % (src, srcdata, destdir)
				try:
					scoretext = do_copy(src, srcdata, destdir)
					copy += 1 
					print "\twrite score file", scorefile
					print >>SCORE, "\n================ %s ===================" % srcdata
					print >>SCORE, "\n".join(scoretext)
					SCORE.flush()
				except:
					continue
			else:
				dtime = destd[srcdata]
				if dtime >= srctime:
					print " .. skip %s/%s, newer data in %s" %  (src, srcdata, destdir)
				else:
					print "update %s/%s" % (destdir, srcdata)
					#do_copy(src, srcdata, destdir)
					#srcd = os.path.join(src, srcdata)
					#do_update(src, srcdata, destdir)
	SCORE.close()
	return (datad, copy)

# check within 24 hours
def get_dirs(destdir, timewindow=24):
	try:
		allfiles =  os.listdir(destdir)
	except:
		print "ERROR connecting to %s" % destdir
		return {}
	datpat = re.compile('\d{4}\-\d\d\-\d\d')
	curtime = time.time()
	
	# only check for data folder generated in the past 24 hours
	datadirs = []

	datad = {}
	for d in allfiles:
		# data folder start with yyyy-dd-mm
		if not re.match(datpat, d):
			continue
		d2 = os.path.join(destdir, d)
		if not os.path.isdir(d2):
			continue
		mtime = os.path.getctime(d2)
		hour = (curtime-mtime)/60/60
		if hour < timewindow:
			#print " folder modified within last %s hours:" % timewindow, d
			datad[d] = mtime

	return datad

def check_host(destdir):
	# make sure we only run 1 copy and run at O:
	cmd = os.popen('query process')
	x = cmd.readlines()
	for y in x:
		p = y.find('cp3730data')
		if p>=0:
			print "ERROR: one process is already running, please terminate it first, then relaunch"
			sys.exit()
	return destdir
	"""
	# if it is running on Chess, make sure the current directory is o:	
	if platform.uname()[1] == 'Chess':
		destdir = "O:"
	if platform.uname()[1] == 'Chess':
		destdir = "O:"
	return destdir
	"""

def auto_copy(destdir=DESTDIR, timewindow=TIMEWINDOW): 
	destdir = check_host(destdir)	
	stime = time.time()

	print "DEST:%s, TIME WINDOW: %s" % (destdir, timewindow)

	while 1:
		datad,cnt = cpdata(destdir=destdir, timewindow=timewindow)

		# get the lastest timestamp for each data source
		#pprint.pprint(datad)

		curtime = time.time()
		last_plate_timestamps = []
		for datasrc, srcd in datad.items():
			if len(srcd.keys()) == 0:
				continue
			timestamps = srcd.values()
			ltime = max(timestamps)
			min_pass = int((curtime - ltime)/60)
			last_plate_timestamps.append([datasrc, min_pass] )

		# sort by last modified time, earliest first
		hr = 5
		runtime = time.time()-stime
	
		if runtime < 5 * 60 * 60:
			sleep_time = 10 # 10 minutes
		elif min_pass > hr * 60:
			print "Done copying data, exiting"	
			break
		else: 
			sleep_time = 20 # 30 minutes
		x = time.ctime(sleep_time * 60 + time.time())
		print "Waiting for %d min for next checking, will check at '%s'" % (sleep_time, x)
		# connect to qb636-m2 to change files to group writable before sleeping
		cmd = "%s -i %s sue.zhao@qb636-m2.quintarabio.com 'sudo chmod -R 755 /home/quintara/Quintara_data/201*_%s*'" % (ssh, keyfile, util.today())
		#print cmd
		#if cnt:
		#	os.system(cmd)
		time.sleep(sleep_time*60)


def do(args):
	optd , ars = util.parse_args(args, "i:d:", "")
	w = 24

	d = DESTDIR
	if ars:
		d = os.path.abspath(ars[0])
	elif optd.has_key('d'):
		d = optd['d']
	if not os.path.isdir(d):
		d = DESTDIR
	if optd.has_key('i'):
		w = int(optd['i'])			
	auto_copy(destdir=d, timewindow=w)

def test(destdir='D:\\test'):
	cpdata(destdir=destdir)

if __name__ == '__main__':
	do(sys.argv[1:])
	sys.exit()
	
