#!python

import sys, os, re, glob, getopt
import cat
from qblib import Fasta, BlastParser, util
from qblib.config import *

def get_default_db():
	dname = os.path.dirname(__file__)
	rootdir = os.path.dirname(dname)
	db = os.path.join(rootdir, "blastdb", "QBprimers.fas")
	if not os.path.isfile(db):
		print "ERROR: cannt locate %s" % db
		sys.exit()
	return db

opts = '-e 0.1 -m 8'

def do(files=[]):
	print files
	if not files:
		outfile = cat.cat2()

		cmd = "%s -W 8 -p blastn -i %s -d %s -e 0.1 -o primers.out"  % (blastall, outfile, db)
		os.system(cmd)

		print "Primer hits are in primers.out"
	else:
		for file in files:
			i = file.rfind('.')
			#outfile = file[:i] + '.out'
			outfile = 'primers.out'
			cmd = '%s -W 8 -p blastn -e 0.1 -i %s -o %s -d %s' % (blastall, file, outfile, db)
			print cmd

			os.system(cmd)

			print "blast output in %s" % outfile


def find_primer(file=''):
	seqd = {}
	if not file or not os.path.isfile(file):
		# process all sequences within current directory
		seqfiles = glob.glob('*.seq')
		dname = os.path.basename(os.path.abspath(os.curdir))
		file = '%s.fas' % dname
		fin = open(file, 'w')
		for f in seqfiles:
			seq = open(f).read()
			seq = re.sub('\s+', '', seq)
			#print "key:", f
			print >>fin, '>%s\n%s' % (f, seq)
			seqd[f] = seq
		fin.close()
	else:
		seqs = Fasta.all(file)
		for x in seqs:
			n = x.name
			s = x.sequence
			s = re.sub('\s+', '', s)
			seqd[n] = s
			#print "key:",n

def find_primers(db, files, opts=""):
	if type(files) == type(''):
		files = [files]
	primerfiles = []
	for f in files:
		primerfiles.append(_find_primers(db, f))
	return primerfiles

def _find_primers(db, file):
	seqd = {}
	seqs = Fasta.all(file)
	for x in seqs:
		n = x.name
		s = x.sequence
		s = re.sub('\s+', '', s)
		seqd[n] = s

	# run blastn here
	d = os.path.basename(db).split('.')[0]
	outfile = file+'_%s.out' % d
	cmd = '%s -W 8 -p blastn -i %s -d %s -o %s %s' % (blastall, file, db, outfile, opts)
	print cmd
	os.system(cmd)

	# load primer
	allprimers = Fasta.all(db)
	name2primer= {}
	for p in allprimers:
		name2primer[p.name] = p.sequence

	blast_results = BlastParser.parse(outfile)
	primer_file = file.replace('.fas', '_%s.pr' % d )
	O = open(primer_file, 'w')

	blast_results.sort(lambda x,y: cmp(x.query, y.query))
	hsps = []

	#print "seqd:", seqd.keys()
	for bltres in blast_results:
		# sort hit but coordinates
		all_hsps = []

		for hit in bltres.hits:
			all_hsps.extend( hit.get_best_HSPs() )
		#print bltres.query, ", HIT COUNT:", len(bltres.hits), ", HSP count:", len(all_hsps)

		for hsp in all_hsps:
			#print "  ", hsp.query, "IDENT:", hsp.ident
			if hsp.ident < 100:
				continue
			subj = hsp.subj

			if name2primer.has_key(subj):
				primerseq = name2primer[subj]
				#print "  ", subj , primerseq, len(primerseq)
				qstart  = min([hsp.qend, hsp.qstart]); qend = max([hsp.qstart, hsp.qend])

				if hsp.sstart < hsp.send:
					strand = '+'
					s = hsp.sstart
					e = hsp.send
				else:
					strand = '-'
					e = hsp.sstart
					s = hsp.send

				if len(name2primer[subj]) == e and s == 1:
					qseq = seqd[hsp.query][qstart-1:qend]
					p = [hsp.query, qstart, qend, strand, hsp.subj, '%s' % primerseq]
					#p = [str(x) for x in p]
					hsps.append(p)
					#print >> O, "\t".join(p)


	hsps.sort(lambda x,y: cmp((x[0],x[1]), (y[0], y[1])))
	print >>O,  "\t".join(['Query', 'Start', 'End', 'Strand', 'PrimerName', 'Sequence'])
	for hsp in hsps:
		hsp = [str(x) for x in hsp]
		print >>O, '\t'.join(hsp)

	O.close()
	print "\n========================================================="
	print "Primer informations are in %s" %  primer_file
	print "========================================================="
	return primer_file

def do2(args):
	prog = args[0]
	usage = "%s [option] fastafile, for option: -d <database>" % prog

	optd, ars = util.parse_args(args[1:], "d:", usage)
	print optd, ars
	
	try:
		db = optd["d"]
	except:
		db = get_default_db()
	if not os.path.isfile(db):
		print usage
		sys.exit()
	find_primers(db, ars)

def get_primer_by_name(name):
	all = Fasta.all('C:\\tools\\blastdb\\QBprimers.fas')
	founds = []
	for a in all:
		if re.search(name, a.name, re.IGNORECASE) :
			founds.append(a)
	for a in founds:
		print a.name, a.sequence


	
if __name__ == "__main__":
	#get_primer_by_name(sys.argv[1])
	#sys.exit()
	do2(sys.argv)
	sys.exit()
	if len(sys.argv) > 1:
		print "UUUUUUUUUUUUUUUUUUUUUUUUUUUUUUUU"
		files = sys.argv[1]
		#do(files)	
		find_primer(files)
	else:
		print "XCCCCCCCCCCCCCCCCCCCCCCCCCCCCCCC"
		do()
		for r in self.rows:
			for c in self.cols:
				well = '%s%02d' % (r, c)
