#!python

import os, sys, re, glob
from qblib import Fasta

"""
This script read gb file, convert to fasta format
option:
	trimstart: trim the sequence to start
	trimend:  trim the sequence to end
"""
class Location:
	def __init__(self, locstr):
		self.complement = ''
		if re.match('complement', locstr):
			self.complement = 1
			locstr = re.sub('complement\(', '', re.sub('\)$', '', locstr) )
		locations = re.findall('\d+\.\.\d+', locstr)
		self.locs = []
		for loc in locations:
			start, end = loc.split('..')
			self.locs.append([int(start), int(end)])
		self.start = min([start for [start,_] in self.locs])
		self.end = max([end for [_,end] in self.locs])

	def __str__(self):
		t = '%s..%s' % (self.start, self.end)
		if self.complement:
			t = 'complement(%s)' % t
		return t

class Feature(object):
	def __init__(self, name, location, **kw):
		self.__dict__['name'] = name
		self.__dict__['location'] = location
		for k, v in kw.items():
			self.__dict__[k] = v
	def __str__(self):
		tmp = ['%s=%s' % (k, v) for (k, v) in self.__dict__.items()]
		return "\n".join(tmp)

def gb2fasta(gbfile, startfeat='', endfeat=''):
	lines = open(gbfile).readlines()
	id = seq = ''
	startfeature = 0
	startseq = 0
	features = []
	for l in lines:
		if re.match('LOCUS', l):
			#print l.strip()
			try:
				id = l.split()[1]
			except:
				id = os.path.basename(gbfile)
			id = re.sub('\s+', '', id)
			continue

		if re.match('FEATURES', l):
			startfeature = 1
			continue
		elif re.match('ORIGIN', l):
			startfeature = 0
			startseq = 1
			continue
		if startfeature and l[5:6] != ' ':
			name, loc = l[5:].strip().split()
			f = Feature(name=name, location=Location(loc))
			features.append(f)
			continue
		elif re.match('\/', l[21:]):
			if startfeature:
				k,v = l[22:].strip().split('=') 
				setattr(features[-1], k, v)
			continue			
		if startseq:
			seq += re.sub('[\/\d+\s+]', '', l.strip())
	
	#id = seq = ''
	start = 0
	end = len(seq)
	if startfeat:
		for f in features:
			if f.label == startfeat:
				print f.name, f.location.start
				start = f.location.start-1
			elif f.label == endfeat:
				end = f.location.end
	print id, len(seq), start, end
	#print seq[start:end]
	fas = Fasta.Fasta(name="%s(%s..%s)" % (id, start+1, end), sequence=seq[start:end])
	return fas
				

def do_all():
	files = glob.glob('*.ape') + glob.glob('*.gb') + glob.glob('*.gbk')
	refile = open('ref.fas', 'w')
	for f in files:
		print "\n", f
		fas=gb2fasta(f, 'Tb', '6x\His-Tag')
		print >>refile, fas
	refile.close()

if __name__ == '__main__':
	do_all()
