#!python

import os, sys, re, glob, pprint
from qblib import util, ABISeq

score_cutoff = 40
seqlen_cutoff = 800
machines = ['T', 'Pu', 'MJ' , 'BO', 'V', 'J', 'BE', 'A']
verbose = 0

def process_samplesheet(samplesheet):
	print "process_samplesheet():", samplesheet
	if not os.path.isfile(samplesheet):
		print "!!! WARNING: invalid sample sheet (%s), do nothing to data" % samplesheet
		return None
	lines = open(samplesheet).readlines()
	
	line1 = lines[0]
	#print "LINE1:", line1
	# second line has plateid
	line2 = lines[1]
	cols = line2.split('\t')
	headers = lines[0].split('\t')
	d = dict(zip(headers,  cols))

	try:
		plateid = d['Plate ID']
	except:
		print "ERROR: plate ssheet %s is invalid" % samplesheet
		return None	
		
	if not plateid:
		print "ERROR: plateid is invalid" 
		return None

	container = d['Container Name']
	if not container:
		container=plateid
	
	operator = d['Operator']
	if not operator:
		operator = "None"
	cd = {}
	for line in lines[5:]:
		cols = [x.strip() for x in line.split('\t')]
		well, sample, customer = cols[:3]
		customer = re.sub('[\'\"]', '', customer)
		tmp = customer.replace(' ', '_')
		cols = re.split('[,;]', tmp)	
		notes = ""
		customer = cols[0]
		if len(cols) > 1:
			notes = cols[-1]	
		if not customer:
			continue
		if re.match('\#REF', customer):
			continue
		if not sample:
			continue
		if re.match('_', sample):
			continue
		if re.match('none', sample, re.IGNORECASE):
			continue
		cd.setdefault(customer, []).append((sample, notes))
	return (plateid, operator, cd)

def separate_files(datadir, samplesheet):
	datadir = os.path.abspath(datadir)
	if not os.path.isdir(datadir):
		print "ERROR: invalid datadir (%s)" % datadir
		sys.exit()
	
	x = process_samplesheet(samplesheet)
	if not x:
		return
	print ".separate_file() for DATADIR:", datadir
	(plateid, operator, cd)  = x

	dname = os.path.basename(datadir)
	if not re.search(plateid, dname):
		print "ERROR: dname %s does not match plateid %s" % (dname, plateid)

	cdir = os.path.abspath(os.path.curdir)
	os.chdir(datadir)

	today = util.today()
	files = glob.glob('*ab1') + glob.glob("*fsa")
	d = dict([(x.split('_')[0],1) for x in files])
	if len(d.keys()) == 1:
		cmd = "O:\\tools\\qbpy\\rename.py -f %s -c 1 -F" % samplesheet
		print ".RENAME: ", cmd
		os.system(cmd)

	scored = {}
	for customer, samples in cd.items():
		dname = '%s_%s' % (today, customer.lower())
		if not os.path.isdir(dname):
			os.mkdir(dname)
			print "... create %s" % dname

		scored.setdefault(customer,  {'good':0, 'bad':[], 'check':[]})

		for x in samples:
			(s, notes) = x
			p = '%s_*.ab1' % s
			ab1s = glob.glob(p)
			for ab1 in ab1s:
				a = ABISeq.ABISeq(ab1, qscore=score_cutoff, seqlen=seqlen_cutoff)
				a.trim_by_quality(quality_cutoff=score_cutoff)
				if not a.trimscore:
					scored[customer]['bad'].append(a)
				elif a.trimscore >= 40:
					l = a.trimend-a.trimstart
					if l >= seqlen_cutoff:
						scored[customer]['good'] += 1
					else:
						scored[customer]['check'].append(a)
				else:
					scored[customer]['check'].append(a)
			os.system('mv %s_* %s' % (s, dname))

		# make sure it is group writable
		os.system('chmod -R 0775 .')

	customers = scored.keys()
	customers.sort()

	scoretext = []
	summary = []
	for customer in customers:
		sd = scored[customer]
		good = sd['good']
		bad  = len(sd['bad'])
		check = len(sd['check'])
		total = good+bad+check
		clen = len(customer[:12])
		cpat = (12 - clen) * ' '
		ccc = "%s %s" % (customer, cpat)
		#print '%s total: %2d, good: %2d, bad: %2d, tocheck: %2d' % (ccc, total, good, bad, check)
		scoretext.append( '%s total: %2d, good: %2d, bad: %2d, tocheck: %2d' % (ccc, total, good, bad, check) )
		summary.append( '%s total: %2d, good: %2d, bad: %2d, tocheck: %2d' % (ccc, total, good, bad, check) )
		for c in sd['check']:
			scoretext.append( '\ttocheck: %s, score=%s, len=%s' % (c.sample, c.trimscore, c.trimend-c.trimstart) )
		for c in sd['bad']:
			scoretext.append( '\tbad: %s, score=%s' % (c.sample, c.trimscore) )

	# write scoring file
	"""
	f = 'score_%s.txt' % plateid
	F = open(f, 'w')
	print >>F, "\n".join(scoretext)
	F.close()
	"""
	#print "Scoring file: ", f
	os.chdir(cdir)
	return summary

exclude = re.compile('(out)|(not)|(tmp)')
def process_datadir(datadir, plateid, FOUT=None):
	total = 0
	customerd = {}
	total = 0
	for root, dirs, files in os.walk(datadir):
		basename = os.path.basename(root)
		if exclude.match(basename):
			if verbose:
				print 'Skipping %s' % basename
			continue
		files2 = []
		for f in files:
			if re.match('_', f):
				continue
			if f.endswith('ab1') or f.endswith('fsa'):
				files2.append(f)
	
		count = len(files2)
		if not count:
			continue
		customerd.setdefault(root, count)
		total += count

	print "\tTOTAL", total
	return customerd

def collect_daily_summary(rootdirs, pat=''):
	# get datadirs within rootdir
	mapat   = '_(T|V|(BO)|(MJ)|(Pu))$'


	for rdir in rootdirs:
		datepat = '20\d\d\-\d\d\-\d\d_'
		print "ROOTDIR:", rdir, datepat
		pattern = os.path.join(rdir, '2011*%s*' % pat)
		print "pattern:", pattern
		dirs = glob.glob(pattern)
		print dirs
		if not dirs:
			break
		F = open('%s_daily_summary.txt' % pat, 'w')
	
		headers = ['plateid', 'operator', 'customer', 'rnx_data', 'rnx_ss', 'To Check', 'SAP', 'MINPREP', 'PCR', 'Data Location']
		print >>F, "\t".join(headers)
		for datadir in dirs:
			operator = soperator = splateid = ''
			cols = datadir.split(os.sep)
			print cols
			plateid = re.sub(datepat, '', cols[-1])
			plateid = re.sub(mapat, '', plateid)
			scd = {}
			ss = glob.glob(os.path.join(rdir, '%s.txt' % plateid))
			try:
				ssheet, = ss
				if  os.path.isfile(ssheet):
					splateid, soperator, scd = process_samplesheet(ssheet)	

			except:
				print "ERROR: get sample sheet %s for plate %s error" % (plateid, ss)


			print "DATADIR: %s, plateid: %s" % (datadir, plateid)
			customerd = process_datadir(datadir, plateid, F)
			base_dirname = os.path.basename(datadir)

			if soperator:
				operator = soperator
			
			for dname, count in customerd.items():
				folders = dname.split(os.sep)
				for i in range(0, len(folders)):
					if folders[i] == os.path.basename(datadir):
						i += 1
						break
				print i, folders
				if i < len(folders):
					customer_dname = folders[i]
				else:
					customer_dname = base_dirname

				cname = re.sub('\d+\_', '', customer_dname)
				try:
					sscnt = len(scd[cname])
				except:
					sscnt = ''
				tocheck = ''
				if sscnt != count:
					tocheck = 'X'
				data = [plateid, operator, cname, count, sscnt, tocheck, '', '','', dname ]
				data = [str(x) for x in data]
				print >>F, '\t'.join(data)
		
		F.close()

def collect_data(ddir, pat='2011-03'):
	# get all sample sheets
	ssheets = glob.glob(os.path.join(ddir, '\%s*.txt' % pat))
	print ssheets
	return
	file = '%s-2011_summary.txt' % pat
	F = open(file, 'w')

	header = ['plateid', 'operator', 'customer', 'seqcount']
	print >>F, "\t".join(header)
	for s in ssheets:
		x = process_samplesheet(s)
		if not x:
			continue
		(plateid, operator, cd ) = x
		print s, plateid, operator
		customers = cd.keys()
		customers.sort()
		for c in customers:
			#print "\t", c, len(cd[c])
			tmp = [plateid, operator, c, len(cd[c])]
			tmp = [str(x) for x in tmp]
			print >>F, "\t".join(tmp)

	F.close()

def do(args):
	usage = """
	%s [option] datadir
	option:
		-s <samplesheet>
		-I <do invoice summary>
		-M <separate user data>
		-p <file patten>

		# summarize reaction for 030111
	ex. %s -I -p 030111 
	o:\\tools\\qbpy\\move2folder.py -M -s ..\\050311H.txt ..\\2011-05-04_050311H_MJ

	""" % (__file__, __file__)

	optd, ars = util.parse_args(args, "s:p:IMA", usage)
	if not ars:
		print usage
		sys.exit()
	if optd.has_key('A'):
		p = optd['p']
		for i in range(1, 31):
			pat = '%s%02d11' % (p, i)
			collect_daily_summary(ars, pat)

	elif optd.has_key('I'):
		collect_daily_summary(ars, optd['p'])
	elif optd.has_key('M'):
		separate_files(ars[0], optd['s'])
		
			
if __name__ == '__main__':
	do(sys.argv[1:])
	#collect_data("O:\\2011-01", pat="01")
	sys.exit()

	#def separate_files(datadir, samplesheet):
	if len(sys.argv) < 2:
		print "Usage: %s datadir, samplesheet" % __file__
		sys.exit()
	separate_files(sys.argv[1], sys.argv[2])


