#!python
import sys, os, re, glob

import DNA, align
class Read:
	def __init__(self, name, rc, start_pos, sequence=''):
		self.name  = name
		self.rc = rc
		self.start_pos = start_pos
		self.sequence = sequence

	def __str__(self):
		tmp = [str(x) for x in [self.name, self.rc, self.start_pos]]
		tmp =  ",".join(tmp)+ self.sequence
		return tmp

	def align_pos(self):
		return [self.start_pos, self.start_pos+len(self.sequence)]


class Contig:
	def __init__(self, **kw):
		self.reads = []
		for k, v in kw.items():
			self.__dict__[k] = v

		# name:
		# length
		# read_count
		# segment_count
		# sequence
		# reads

	def add_read_sequence(self, readname, readseq):
		for r in self.reads:
			if r.name == readname:
				r.sequence = readseq


	def __str__(self):
		tmp = []
		for k, v in self.__dict__.items():
			if type(v) == type([]):
				v2 = [str(x) for x in v]
				v = "\n\t".join(v2)
			tmp.append('%s=%s' % (k, v))
	
		return "\n".join(tmp)

class AceParser:
	def __init__(self, acefile=''):
		self.file = acefile
		self.contigs = []
		self.contig_count = '' 
		self.read_count = ''
	
	def parse(self, ref, acefile):
		self.file = acefile
		In = open(acefile)
		contig_seq = ''
		read_seq = ''
		getseq = 0
		while 1:
			l = In.readline()
			if not l:
				break
			l = l.strip()
			if re.match('AS', l):
				cols = l.split()
				_, contig_count, read_count = cols[:3]
				self.contig_count = int(contig_count)
				self.read_cont = int(read_count)

				#print "READ count:", read_count
			elif re.match('CO',l):
				cols = l.split()
				_, name, bcount, rcount, scount, _ = cols[:6]
				bcount = int(bcount)
				rcount = int(rcount)
				scount = int(scount)
				c = Contig(name=name, length=bcount, read_count = rcount, segment_count = scount)
				self.contigs.append(c)
				getseq = 1
				continue

			elif re.match('BQ', l):
				#print c.name, contig_seq , len(contig_seq)
				getseq = 0
				self.contigs[-1].sequence = contig_seq
				contig_seq = ''

			elif re.match('AF', l):
				cols = [x.strip() for x in l.split()]
				_, name, rc, start_pos = cols[:4]
				start_pos = int(start_pos)
				read = Read(name, rc, start_pos)
				self.contigs[-1].reads.append(read)

			elif re.match('RD', l):
				cols = [x.strip() for x in l.split()]
				_, name, bases = cols[:3]
				bases = int(bases)
				read_seq = ''
				while 1:
					l2 = In.readline()
					if not l2 or  re.match('QA', l2):
						break
					read_seq += re.sub('\s+','', l2)
				self.contigs[-1].add_read_sequence(name, read_seq)
			if getseq:
				contig_seq += l.strip()

	def process(self, ref, acefile, outdir=''):
		# process ace file to see whether it 
		# 1. match reference
		# 2. done sequencing
		# 3. mutation?
		# 4. get contig nt and aa sequence
		self.parse(ref, acefile)

		match= mutation= contig_nt= contig_aa = ''
		doneseq = 1

		# match to reference? the reference will be included one of the contigs if reads match
		if len(self.contigs) > 1:
			doneseq = 0
			return (match, mutation, doneseq, contig_nt, contig_aa)

		for c in self.contigs:
			match = 0
			print c.name, c.length, c.read_count, c.segment_count, len(c.reads)

			align_regions = [] # align_start and align_end position for each read
			alignments = []
			ref_pos = ''
			ref_read = ''
			for r in c.reads:
				pos = r.align_pos()
				if r.name == ref.name:
					match = 1
					ref_pos = pos
					ref_read = r
					continue
				align_regions.append(pos)

				# pat sequence to the same length as the contig with '.'
				length = len(r.sequence)
				seq = (r.start_pos-1)  * '.' + r.sequence + (c.length-r.start_pos+1- length)*'.'
				alignments.append([r, seq])

			align_regions.sort()
			readcnt = len(align_regions)-1

			gaps = [] # gap has the position related to contig sequence
			for i in range(0, readcnt):
				(x1,y1) = align_regions[i]
				(x2,y2) = align_regions[i+1]
				if x2>y1:
					doneseq = 0
					gaps.append([y1, x2])

			# convert position in related to reference
			gap2 = []
			for g in gaps:
				rs,re = ref_pos
				s,e = g
				gap2.append([s-rs, e-rs])
			print "Match %s, donesequencing %s" % ( match, doneseq), gaps, gap2

			# get contig sequence
			if not doneseq:
				return (match, mutation, doneseq, contig_nt, contig_aa)

			contig_nt = ''
			if doneseq:
				alignments.sort(lambda x,y: cmp(x[0].start_pos, y[0].start_pos))
				Fal = open(acefile.replace('.fas.qb.ace', '.aln'), 'w')
				for i in range(0, len(alignments)-1):
					m = alignments[i]; n = alignments[i+1]
					print >>Fal, '>%s' % m[0].name
					print >>Fal, m[1]
					ms = m[0].start_pos; me = m[0].start_pos+ len(m[0].sequence)
					ns = n[0].start_pos; ne = n[0].start_pos+ len(n[0].sequence)
					print m[0].align_pos(), n[0].align_pos()
					e = min(me, ns)
					s = ms
					if s == 1:
						s = 0
					
					contig_nt += m[1][s:e]
				(last, seq) = alignments[-1]
				print >>Fal, '>%s' % last.name
				print >>Fal, seq
				sss = seq[last.start_pos:]
				contig_nt += sss

			# trim to the right start and end
			rs, re = ref_pos
			nt = contig_nt[rs-1:re]
			print " get seq: %s-%s" % (rs, re)
			aa = DNA.DNA(nt).translate()
			name = acefile.replace('.fas.qb.ace', '')
			ntfile = name + '_nt.fas'
			aafile = name + '_aa.fas'
			Fnt = open(ntfile, 'w')
			Faa = open(aafile, 'w')
			print "writing ", ntfile
			print >>Fnt, '>%s' % name
			print >>Fnt, nt
			print >>Faa, '>%s' % name
			print >>Faa, aa
			Fnt.close()
			Faa.close()

			# do alignment 
			align.do_muscle(ntfile, 'html')
			align.do_muscle(ntfile, 'fas')
			align.do_muscle(aafile, 'html')
			align.do_muscle(aafile, 'fas')

			# done sequencing?
			return (match, mutation, doneseq, ntfile, aafile)

	def create_alignment(self):
		alignments = []
		for c in self.contigs:
			print >>F, '>%s\n%s' % (c.name, c.sequence)
			alignments.append([c.name, c.sequence])

			align_regions = []
			for r in c.reads:
				base_count = len(r.sequence)
				if r.rc == 'U':
					tag = '(+)'
					seq = (r.start_pos-1) * '.' + r.sequence + (c.base_count-base_count-r.start_pos) * '.'
				else:
					tag = '(-)'
					seq = r.start_pos * '.' + r.sequence + (c.base_count-base_count-r.start_pos) * '.'
				name = r.name + tag
				align_regions.append([r.start_pos, r.start_pos+base_count])
				print "Read start: %s, read length: %s" % (r.start_pos, base_count)
				print >>F, '>%s\n%s' % (name, seq)
				alignments.append([name, seq])

			align_regions.sort()


		F.close()
		return  alignments

	def get_contig_sequence(self):
		pass


if __name__ == '__main__':

	ace = AceParser(sys.argv[1])
	a = ace.create_alignment()
	print a
