#!python

import string, sys, re, os
import Fasta
point = 'point'
insertion = 'insertion'
deletion = 'deletion'
incomplete = 'incomplete'
NT_MUTATION_TYPES = ['point', 'insertion', 'deletion']

# silent: is the same amino acid
# missense: to different amino acid, sub-type: neutral mutation - different but chemically similar amino acid
# nonsense: to stop codon 
AA_MUTATION_TYPES = ['silent', 'frameshift', 'nonsense', 'missense']
silent = 'silent'
frameshift = 'frameshift'
nonsense = 'nonsense'
missense = 'missense'


def report_mutations(nt_fasta, aa_fasta, nt_refname, aa_refname):
	nt_typed =  {}
	nt_mutd = {}
	if os.path.isfile(nt_fasta):
		(nt_typed, nt_mutd) = do_nt_mutation(nt_fasta, nt_refname)

	if os.path.isfile(aa_fasta):
		(aa_typed, aa_mutd) = do_aa_mutation(aa_fasta, aa_refname)

def do_nt_mutation(nt_fasta, nt_refname):
	window	= 5

	# process nt_fasta alignment file
	seqs = Fasta.all(nt_fasta)
	seqd = {}
	names = []
	RSTART = ''
	REND = ''
	for nt in seqs:
		nt_seqd.setdefaut(nt.name, nt.sequence)
		if nt.name == nt_refname:
			RSTART = nt.sequence[:window].upper()
			REND = nt.sequence[-window:].upper()
			continue
		names.append(nt.name)

	if not RSTART or not REND:
		print "ERROR: cannot figure out reference sequence from %s" % nt_fasta
		sys.exit()

	# get nt mutation type
	refseq = seqd[nt_refname].upper()
	rlen = len(refseq)

	mtyped = {}
	mutd = {}
	for name in names:
		seq = seqd[name].upper()

		mtype = 'WT'
		# figure out mutation types
		mstart = seq[:5].upper()
		mend   = seq[-5:].upper()
		mlen = len(seq)
		muts = []

		# make sure start and end of sequence are the same
		if mlen < rlen:
			if mstart == RSTART and mend == REND:
				mtype = deletion
			else:
				mtype = 'incomplete'
		elif mlen > rlen:
				mtype = insertion
		else:
			rseq, mseq = align_2seq(refseq, mseq)

			muts = []
			mlen = len(rseq)
			for i in range(0, mlen):
				if mseq[i] != rseq[i]:
					muts.append('%s%03d%s' % (refseq[i], i, seq[i]))	
				if not muts:
					mtype = 'WT'
		mtyped[name] = mtype
		if muts:
			mutd[name] = muts

			
	return (mtyped, mutd)
	aaseqs = []
	aa_seqd = {}
	AA_RSTART = ''
	AA_REND = ''
	if os.path.isfile(aa_fasta):
		aaseqs = Fasta.all(aa_fasta)
		aa_seqd.setdefault(aa.name, aa.sequence)
		if aa.name == aa_refname:
			AA_RSTART = nt.sequence[:windows].upper()
			AA_REND = nt.sequence[-windows:].upper()
		else:
			aa_names.append(aa.name)

class Mutation:
	def __init__(self, name, align_start, align_end, seq_type='nt'):
		self.name = name
		self.align_start = align_start
		self.align_end = align_end
		self.align_len = align_end-align_start+1
		self.seq_type = seq_type
		self.tag = ''
		self.mutations = []

def get_nt_mutation(nt_align):
	res = []
	if not os.path.isfile(nt_align):
		return res

	# read alignment fasta file
	
	allnts = Fasta.all(nt_align)

	nt0 = allnts[0]
	refname = nt0.name
	refseq = nt0.sequence

	# get gap characters
	refseq = re.sub('[^GATCNgatcn]', '-', refseq)
	#icnt0 = refseq[:start].count('-')
	#icnt1 = refseq[:end].count('-')

	name2tag = {}

	for nt in allnts[1:]:
		name = nt.name; seq = nt.sequence

		# find alignment start and end
		m = re.search('[GATCN]+', seq, re.IGNORECASE)
		if m:
			start = m.start()+1
		i = len(seq)-1
		while i >= 0 :
			if re.match('[AGTCN]', seq[i], re.IGNORECASE ):
				end = i+1
				break
			i -= 1

		seq = re.sub('^[\~\-]', '.', seq)
		seq = re.sub('[\~\-]+$', '', seq)

		#end2 = min(end, len(seq))

		#print nt.name ,  start, end2, end2-start-icnt1
		#align_len = end-start +1
		#tmp = [nt.name, start, end, align_len]

		checkend = min([end, len(refseq)])
		ntmut = Mutation(nt.name, start, checkend)
		tag = ''
		details = []
		dcnt = 0
		for i in  range(start, checkend):
			icnt = refseq[:i].count('-')
			if refseq[i] != seq[i]:
				x =  "%s%s%s" % (refseq[i], i-icnt+1, seq[i])
				details.append(x)
				dcnt += 1
			if i == checkend -1:
				my_dashcnt = seq[:i].count('-')
				dash_diff = my_dashcnt - icnt
				if dash_diff < 0:
					tag = 'insertion'
				elif dash_diff > 0:
					tag = 'deletion'

				if dash_diff%3 != 0:
					tag = 'frameshift ' + tag
		#res.append(tmp+[tag]+details)
		if not dcnt:
			tag = 'WT'
		ntmut.tag = tag
		ntmut.mutations = details
		res.append(ntmut)
		name2tag[name] = tag
	return res	
