#!python

import os, sys, re, pprint
import Fasta
from config import *

insertion = 'insertion'
deletion = 'deletion'
point = 'point'
silent = 'silent'
missense = 'missense'
nonssense = 'nonssense'

def identify_mutation(fasta, format='fasta'):
	# try to figure out it is a DNA or protein alignment file
	seqs = Fasta.all(alignment_file)

	r = seqs[0]
	rname = r.name
	rseq = r.sequence
	rlist = [x.upper() for x in list(rseq)]

	mutd = {}
	names = []
	seqd = {}

	# identify mutation index
	for s in seqs[1:]:
		slist = [x.upper() for x in list(s.sequence)]
		seqd[s.name] = slist
		names.append(s.name)

		mutd.setdefault(s.name, [])
		#print s.name, len(rlist), len(slist)
		for i in range(0, len(rlist)):
			if rlist[i] != slist[i]:
				mutd[s.name].append(i)
	
	# create mutation report
	for name in names:
		mutlocs = mutd[name]

		if not mutlocs:
			continue
		#print name, mutlocs
		# get reference and mutation for each location
		slist = seqd[name]
	
		locs = mutd[name]
		'''
		muts = []
		for loc in locs:
			muts.append( '%s%s%s' % (rlist[loc], loc+1, slist[loc]) )

		print name,  muts
		'''

def format_mutations(rseq, mseq, mtype, mutpos):
	refseq = rseq
	#print "MTYPE", mtype, mutpos
	mutations = []
	mutpos.sort()
	if mtype == point:
		for i in mutpos:
			_dcnt = refseq[:i].count('-')
			r = rseq[i]
			m = mseq[i]
			mutation.append('%s%03d%s' % (r, i+1-_dcnt, m))

	elif mtype in [deletion, insertion]:
		# do range 
		ranges = []
		s = min(mutpos)
		e = s
		last = mutpos[0]

		for m in mutpos[1:]:
			if m - last == 1:
				s = min(s, m)
				e = max(e, m)
			else:
				ranges.append([s,e])
				#print "RANGE: %s-%s" % (s+1,e+1)
				s = m
				e = m
			#print "Cur=%s, s=%s, e=%s, last=%s" % (m,s, e, last)
			last = m

		if e ==last:
			ranges.append([s,e])
		#print "OUT Cur=%s, s=%s, e=%s, last=%s" % (m,s, e, last)
		for x in ranges:
			s, e = x
			_dcnt = refseq[:s].count('-')
			s = s - _dcnt
			e = e - _dcnt

			if s == e:
				mutations.append('%s%s' % (s+1, mtype[:3]))
			elif mtype == deletion:
				mutations.append('%s%s_%s%s%s' % (rseq[s], s+1, rseq[e], e+1, mtype[:3]))
			elif mtype == insertion:
				#print "S: %s, E:%s" % (s,e)
				rs = refseq[s-1]
				re = refseq[e]
				ins = mseq[s:e]
				mutations.append('%s%s-%s%sins%s' % (rs, s, re, s+1, ins))
				
	return mutations

# new as 3/1/2011
def do_aa_mutation(aafile, aa_refname=''):
	mutation_d = {}
	if not os.path.isfile(aafile):
		return mutation_d

	# get all sequences
	seqs = Fasta.all(aafile)

	if not aa_refname:
		aa_refname = seqs[0].name

	refname = refseq = ''
	seqd = {}
	for s in seqs:
		seq = s.sequence.upper()
		if s.name == aa_refname:
			refname = s.name
			refseq = seq
		else:
			seqd[s.name] = seq
	
	# do pairwise alignment
	for name, seq in seqd.items():
		#print name
		tmp = "tmp_x.fas"
		F = open(tmp, 'w')
		print >>F, '>%s\n%s\n' % (refname, refseq)
		print >>F, '>%s\n%s\n' % (name, seq)
		F.close()
		align = 'a_%s' % tmp
		cmd = "%s -in %s -out %s -quiet"  % (MUSCLE, tmp, align)
		print cmd
		os.system( cmd)
		all = Fasta.all(align)

		r, m = all
		mseq = m.sequence.upper()
		rseq = r.sequence.upper()
		alen = len(rseq)
		_md = {}
		for i in range(0, alen):
			m = mseq[i]
			r = rseq[i]
			# adjust i for insertion 
			j = i #-_dcnt
			if m == r:
				continue
			if not re.search('[A-Z]', r):
				_md.setdefault(insertion, []).append(j)

			elif not re.search('[A-Z]', m):
				_md.setdefault(deletion, []).append(j)
			else:
				dcnt = rseq[:i].count('-')
				_md.setdefault(point, []).append(j-dcnt)
		
		# what type of mutation?
		mtype= 'WT'
		try:
			ic = len(_md[insertion])
		except:
			ic = 0
		try:
			dc = len(_md[deletion])
		except:
			dc = 0

		mutations = []
		#print _md.keys()
		if _md.has_key(point):
			mtype = ''
			for i in _md[point]:
				r = rseq[i]
				m = mseq[i]
				mutations.append('%s%03d%s' % (r, i+1, m))
		
		if ic:
			y = format_mutations(rseq, mseq, insertion, _md[insertion])
			mutations += y
			print name, "Insertion:", _md[insertion], y

		elif dc:
			y = format_mutations(rseq, mseq, deletion, _md[deletion])
			print name, "Deletion:", name, y
			mutations += y

		if ic > dc and len(refseq) < len(seqd[name]):
			mtype = insertion
		elif ic < dc and len(seqd[name]) < len(refseq):
			mtype = deletion
			# is this deletion or incomplete?
			pos = _md[deletion]
			pos.sort()
			# first index is not 0
			if pos[0] == 0 or pos[-1] == len(rseq)-1:
				mtype = 'incomplete'

		# is there a frameshift?
		for x in [insertion, deletion, point]:
			if not _md.has_key(x):
				continue
			pos = _md[x]
			pos.sort()
			if pos[-1] < len(refseq)-2:
				continue

			pos.reverse()
			l0 = pos[0]
			cnt = 0 
			for a in pos:
				if l0-a <= 1:
					l0 = a
					cnt += 1
				else:
					break
		
				#print "\t", a, cnt
			print "LAST INDEX: %s, cnt:%s" % (l0, cnt)
			if cnt >= 3:
				mtype = 'frameshift %s' % mtype
		mutation_d.setdefault(name, {})
		mutation_d[name]['tag'] = mtype
		mutation_d[name]['mutations'] = mutations
		#print name, mtype, mutations
				
	try:				
		os.system('rm -rf %s' % tmp)
		os.system('rm -rf %s' % align)
	except:
		pass

	names = mutation_d.keys()
	names.sort()
	O = open('out.txt', 'w')
	for n in names:
		d = mutation_d[n]
		tag = d['tag']
		mutations = d['mutations']
		tmp = [n, tag, "\t".join(mutations)]
		print >>O, "\t".join(tmp)

	O.close()
	return mutation_d
		
if __name__ == '__main__':
	if len(sys.argv) < 3:
		print "Usage: %s fastafile refname" % __file__
		sys.exit()
	do_aa_mutation(sys.argv[1], sys.argv[2])
	#identify_mutation(sys.argv[1])
