#!python

import os, sys, re, glob, pprint, types,  time
import ABISeq, process_dir, Fasta, DNA, align, BlastParser, util, MutationDetection
from config import *

_20AA=['I', 'L', 'V', 'F', 'M', 'C', 'A', 'G', 'P', 'T', 'S', 'Y', 'W', 'Q', 'N', 'H', 'E', 'D', 'K', 'R', '*']

LABELD = {  'no_sample':"No Sample", 
		'perfect':"perfect", 
		'other_mutation':"Other Mutation", 
		'not_done':"Not Done", 
		'no_match':"No Match", 
		'match_other':"Match Other",
		'framestop': "Frame/Stop"
		}

v1 = 0
v2 = 0
class Plate:
	def __init__(self, name, well2ref={}):
		self.name = name
		self.data = {}
		self.rows = ['A', 'B', 'C', 'D', 'E', 'F', 'G', 'H']
		self.cols = [1,2,3,4,5,6,7,8,9,10,11, 12]
	
		self.well2ref = well2ref
		for r in self.rows:
			for c in self.cols:
				well = '%s%02d' % (r, c)
				# need to change FanYanPlate so each well contains more than 1 clone
				#self.data[well] = [] 
				self.data[well] = None 

	def setvalue(self, well, value):
		'''
		r = well[0]
		if mutd.has_key(well):
		continue

		c = int(well.replace(r, ''))
		w = '%s%02d' % (r.upper(), c)
		'''
		w = util.format(well)
		self.data[w] = myvalue
		#self.data.setdefault(w, []).append( myvalue )

	def get_well(self, r, c):
		c = int(c)
		w = '%s%02d' % (r.upper(), c)
		return w

	def getvalue2(self, r, c):
		w = self.get_well(r,c)
		try:
			return self.data[w]
		except:
			return ''

	def getvalue(self, well):		
		r = well[0]
		c = int(well.replace(r, ''))
		w = '%s%02d' % (r.upper(), c)

		if self.data.has_key(w):
			return self.data[w]
		else:
			print "self.data has no %s" % w

	def do_muscle(self, infile, format=''):
		return align.do_muscle(infile, format)

	def items(self):
		return self.data.items()

	def __str__(self):
		table = []
		tmp = [str(x) for x in ['col'] + self.cols]
		table.append('\t'.join(tmp))

		for r in self.rows:
			row = [r]
			for c in self.cols:
				w = '%s%02d' % (r.upper(), c)
				mydata = self.getvalue(w)
				if mydata:
					row.append(mydata)
				else:
					row.append('.')
			table.append('\t'.join(row))
		return '\n'.join(table)	

	def set_all(self, value):
		"""
		set all well to value
		"""
		for r in self.rowindex:
			for c in self.colindex:
				self.__dict__[r][c] = value
		
	def set_seqdata(self, sep, dirlist):
		"""
		sep: seperator between name and primer 
		"""
		# seperator for template_primer
		cloned = process_dir.get_allclones(sep, dirlist)	

		#print "Clone Count:", len(cloned.keys())
		clonenames = cloned.keys()
		clonenames.sort()
		for c in clonenames:
			clone = cloned[c]
			#print clone.name, clone.well(), len(clone.reads)
			self.data[clone.well()] = clone
			#self.setvalue( clone.well(), clone)
class SingleMutationPlate(Plate):
	def __init__(self, name, fPrimer='', rPrimer='', minDistance=500, maxDistance=5000, nt_reffile='', clean_start='', col_order=0):
		Plate.__init__(self, name)
		self.col_order = col_order
		self.fprimer = fPrimer
		self.rprimer = rPrimer
		self.min_distance = minDistance
		self.max_distance = maxDistance

		#curdir = os.path.abspath(os.path.curdir)
		curdir = os.path.curdir

		self.tmp_dir = os.path.join(curdir, 'tmp_%s' % self.name)
		if not os.path.isdir(self.tmp_dir):
			os.mkdir(self.tmp_dir)

		self.report_dir = os.path.join(curdir, 'report_%s_%s' % (self.name, util.today() ))
		if not os.path.isdir(self.report_dir):
			os.mkdir(self.report_dir)

		self.clean_start = clean_start
		if clean_start:
			os.system('rm -rf %s/*' % self.tmp_dir)
			os.system('rm -rf %s/*' % self.report_dir)
		if os.path.isfile(nt_reffile):
			self.reffile = nt_reffile

		self.cds_fasta = os.path.join(self.report_dir, '%s_cds.fas' % self.name)
		self.aa_fasta = os.path.join(self.report_dir, '%s_aa.fas' % self.name)
		self.nt_fasta = os.path.join(self.report_dir, '%s_nt.fas' % self.name)
		self.cds_align = self.cds_fasta.replace('.fas', '_align.fas')
		self.aa_align = self.aa_fasta.replace('.fas', '_align.fas')


	def do_assembly(self, trimlength=0, prog="cap3", trimtags=[]):
		contigfile = os.path.join(self.report_dir, '%s_contig.fas' % self.name)
		F = open(contigfile, 'w')
		for well, clone in self.data.items():
			if clone is None:
				continue
			clone.set_dir(self.report_dir, self.tmp_dir)
			clone.do_assembly(self.fprimer, self.rprimer, self.min_distance, self.max_distance, notrim=trimlength, trimtags=trimtags, assembly_prog=prog)
			contigs = clone.get_contig_seqs()

			# be default, expect 1 contig only
			if len(contigs) != 1:
				print 'XXX:', well, len(contigs)
				continue

			for s in contigs:
				print >>F, s
			self.contig_count = len(contigs)

		F.close()
		self.contig_fasta = contigfile
		if trimtags:
			stag, etag = trimtags
			self.contig_fasta = align.trim1_by_tags(contigfile, stag, etag)
			return self.contig_fasta
		else:
			return contigfile

	def set_sequencing_status(self):
		for c in self.items():
			if c is None:
				continue
			print c.name, len(c.reads), c.sequencing_status

	def trim2ref(self, reffile, nt_refname, cds_refname, fastafile, debug=0):
		print "==> trim to reference"
		"""
		trim all sequences within fastafile according to reffile
		"""
		# first trim to nt (including coding and non-coding sequence)
		nt_trim_fasta = os.path.join(self.report_dir, '%s_nt.fas' % self.name)
		cds_trim_fasta = os.path.join(self.report_dir, '%s_cds.fas' % self.name)
		aa_fasta = os.path.join(self.report_dir, '%s_aa.fas' % self.name)
		
		trim_nt = 1
		trim_cds = 1
		if os.path.isfile(nt_trim_fasta) and os.path.getmtime(nt_trim_fasta) > os.path.getmtime(fastafile):
			trim_nt = 0
		
		if os.path.isfile(cds_trim_fasta) and os.path.getmtime(cds_trim_fasta) > os.path.getmtime(fastafile):
			trim_cds = 0

		'''	
		if not self.clean_start:
			if os.path.isfile(nt_trim_fasta) and os.path.isfile( cds_trim_fasta) and os.path.isfile(aa_fasta):
				self.nt_fasta = nt_trim_fasta
				self.cds_fasta = cds_trim_fasta
				self.aa_fasta = aa_fasta

				return 
		'''

		refs = Fasta.all(reffile)
		nt_refseq = cds_refseq = ''
		for a in refs:
			if nt_refname and a.name == nt_refname:
				nt_refseq = a.sequence
			if cds_refname and a.name == cds_refname:
				cds_refseq = a.sequence

		#print "REFSEQ:", nt_refseq, cds_refseq
		if trim_nt and nt_refname and nt_refseq:
			reffile = os.path.join(self.report_dir, 'ref_%s.fas' % nt_refname)
			F = open(reffile, 'w')
			print >>F, '>%s\n%s' % (nt_refname, nt_refseq)
			F.close()
			align.trim2ref(reffile, fastafile, nt_trim_fasta, debug=debug)	
			self.nt_fasta = nt_trim_fasta
	
		if trim_cds and  cds_refname and cds_refseq:
			reffile = os.path.join(self.report_dir, 'ref_%s.fas' % nt_refname)
			F = open(reffile, 'w')
			print >>F, '>%s\n%s' % (cds_refname, cds_refseq)
			F.close()
			align.trim2ref(reffile, fastafile, cds_trim_fasta, order_by_row=1, debug=debug )	
			self.cds_fasta = cds_trim_fasta

			# get a translation file for all cds
			F = open(aa_fasta, 'w')
			cds_seqs = Fasta.all(self.cds_fasta)	

			cds_refseq = cds_refseq.upper()
			stag = cds_refseq[:9]
			etag = cds_refseq[-9:]

			for a in cds_seqs:
				aseq = a.sequence.upper()
				if aseq[:9] == stag and aseq[-9:] == etag and self.contig_count == 1:
					self.sequencing_status = 'done'
				else:
					self.sequencing_status = 'notdone'

				print >>F, '>%s\n%s' % (a.name, a.translate())
			F.close()
			self.aa_fasta = aa_fasta
			print "Sequence count: ", len(cds_seqs)

		print "TRIM NT FILE:", self.nt_fasta
		print "TRIM CDS FILE:", self.cds_fasta

	def do_align(self, refile, nt='FNA:Plat-Tlat', cds='FNA', debug=0):
		print "==> do alignment"
		if debug:
			print "refile: %s" % refile
			print "NT: %s, CDS: %s" % (nt, cds)

		
		# do vector triming here
		#contig_fasta = align._trim_by_tags(fasta=self.contig_fasta, upstream='aagaaggagatatacat', downstream='ggatccgaattcgag')
		#self.contig_fasta = contig_fasta
		self.trim2ref(refile, nt, cds, self.contig_fasta, debug=debug)
		
		'''
		if not self.clean_start:
			aa_align_fasta = self.aa_fasta.replace('.fas', '_align.fas')
			cds_align_fasta = self.cds_fasta.replace('.fas', '_align.fas')
			#if os.path.isfile(aa_align_fasta) and os.path.isfile(cds_align_fasta):
			#	return (cds_align_fasta, aa_align_fasta)
		'''

		aa_align_fasta = cds_align_fasta = cds_align_html = ''
		# TO DO
		if cds:
			aa_align_html = self.do_muscle(self.aa_fasta, 'html')
			aa_align_fasta = self.do_muscle(self.aa_fasta )

		if nt:
			nt_align_html = self.do_muscle(self.nt_fasta, 'html')
			#nt_align_fasta = self.do_muscle(self.nt_fasta)
			pass

		self.cds_align = cds_align_fasta
		self.aa_align = aa_align_fasta

		return (cds_align_fasta, aa_align_fasta)

	def _get_mutation_pos(self, fasta_align, refname=''):
		name2mutpos = {}
		refseq = ''
		cds_seqs = Fasta.all(fasta_align)

		if not refname:
			refname = cds_seqs[0].name

		for x in cds_seqs:
			if x.name == refname:
				refseq = x.sequence
				continue
			reflist = list(refseq)
			mylist = list(x.sequence)

			for i in range(0, len(reflist)):
				if reflist[i] != mylist[i]:
					name2mutpos.setdefault(x.name, []).append([i, reflist[i], mylist[i]])
		return name2mutpos
		
	def id_mutation(self, aa_fasta, mut_pos, refname=''):
		print "MUT POS: %s" % mut_pos, refname
		named = self._get_mutation_pos(aa_fasta, refname)
		name2mutd = {}

		names = named.keys()
		names.sort()

		for name in names: 
			positions = named[name]
			blocks = []
			for p in positions:
				if not blocks:
					blocks.append([p, p])
				else:
					last_block = blocks[-1]
					ls, le = last_block
					if p-le == 1:
						last_block[-1] = p
						blocks[-1] = last_block
					else:
						blocks.append([p, p])
	
			muts = []
			for b in blocks:
				s,e = b
				if e == s:
					muts.append(s)
				elif e-s < 5:
					for i in range(s, e+1):
						muts.append(i)	
				else:
					muts = ['FAILED']	

			if 'FAILED' in muts:
				muts = ['FAILED']

			name2mutd[name] = muts

		clone_mutd = {}
		for name, muts in name2mutd.items():
			clone_name = name[:name.find(':')]
			clone_mutd.setdefault(clone_name, [])
			for m in muts:
				if m not in clone_mutd[clone_name]:
					clone_mutd[clone_name].append(m)

		# get aa sequence
		clone_mutaa = {}
		clone_other = {}
		clone2aa = {}
		allaas = Fasta.all(aa_fasta)
		for a in allaas:
			clone = a.name[:a.name.find(':')]
			clone2aa[clone] = a.sequence
			if clone_mutd.has_key(clone):
				mutations = clone_mutd[clone]
				if mutations.count('FAILED') != 0:
					clone_mutaa[clone] = 'FAILED'
				else:
					other = []
					for m in mutations:
						if m == mut_pos-1:
							clone_mutaa[clone] = a.sequence[m]
						else:
							other.append(m)
					if other:
						clone_other[clone] = other
			else:
				clone_mutaa[clone] = a.sequence[mut_pos-1]

		return (clone_mutaa, clone_other)


class FNAPlate(SingleMutationPlate):

	# 3/3/2011
	#def trim_to_ref(self, refile, nt_name, cds_name, report_done=1):
	def report_done(self, create_donedir=0):
		mutd = self._do_mutation_030111()
		cnt = 0
		done = []
		for well, d in mutd.items():
			if not d['tag']:
				print "DONE: ", well
				cnt += 1
				done.append(well)

			elif d['tag'] in ['WT', 'deletion']:
				print "DONE: ", well, d['tag']
				done.append(well)
				cnt += 1
		

		donedir = 'done_%s' % self.name
		if done and create_donedir:
			if not os.path.isdir(donedir):
				os.mkdir(donedir)
		else:
			return
		wells = self.data.keys()
		wells.sort()
		for w in wells:
			if w not in done:
				continue

			clone = self.data[w]
			if clone is None:
				continue

			for r in clone.reads:
				cmd = 'mv -f %s %s' % (r.file, donedir)
				print cmd
				os.system(cmd)

	def get_clone(self, well):
		well = util.format_well(well)
		return self.getvalue(well)

	# for single mutation library, pick top 3
	def do_toppick(self, mut_pos_start, mut_pos_end):
		# for single site mutation, mut_pos_start = mut_pos_end 
		print "CDS:", self.cds_fasta
		print "AA:", self.aa_fasta

	# new as 12/08/2010
	# renames: list of name in refile
	def do_align2(self, refile, refnames=[]):
		# get all refseq
		if not os.path.isfile(refile):
			print "WARNING: invalid reference fasta file %s" % refile
			return
		allseqs = Fasta.all(refile)
		refname2seq = {}

		for a in allseqs:
			if a.name in refnames:
				refname2seq[a.name] = a.sequence
				print "\tref:", a.name, len(a.sequence)
		notfound = []
		for n in refnames:
			if not refname2seq.has_key(n):
				notfound.append(n)
		if notfound:
			print "WARNING: cannot find the following references %s" % notfound	
		ntas = []	
		for name, seq in refname2seq.items():
			# now trim my contig to refseq
			rfile = os.path.join(self.tmp_dir, "ref_%s.fas" % name)
			outfile = os.path.join(self.report_dir, "%s_%s_nt.fas" % (self.name, name))
			F = open(rfile, 'w')
			print >>F, '>%s\n%s' % (name, seq)
			F.close()
			# do vector triming here
			contig_fasta = align.trim1_by_tags(fasta=self.contig_fasta, upstream='ccctttcgtcttcaa', downstream='taagcttacataaa')
			self.contig_fasta = contig_fasta
			align.trim2ref(ref=rfile, myfasta=self.contig_fasta, outfile=outfile )
			print outfile

			# create alignment html file
			html = self.do_muscle(outfile, 'html')
			fas1 = self.do_muscle(outfile, 'fas')
			ntas.append(fas1)
			#mout = os.path.join(self.report_dir, '%s_nt_mutation_summary.txt' % self.name)
			#CloneAssembler.report_mutation(mout, fas1)
	
			# do we need to do aa alignment?
			# get the refseq, translate into amino acid
			(valid, aa) = DNA.DNA(seq).is_valid_protein()
			if valid:
				aafile = os.path.join(self.report_dir, "%s_%s_aa.fas" % (self.name, name))
				F = open(aafile, 'w')
				allseqs = Fasta.all(outfile)
				for a in allseqs:
					print >>F, '>%s\n%s' % (a.name, DNA.DNA(a.sequence).translate())
				F.close()
				html = self.do_muscle(aafile, 'html')
				alignfile = aafile
				mfile = "%s_%s" % (self.name, name)
				print "\n\n===> create mutation summary for %s using %s" % (mfile, alignfile)
				self.do_mutation_summary(aa_alignment=aafile,name=mfile)
		return ntas

	# in used as 3/1/2011
	def _do_mutation_030111(self, aa_refname='', notes=''):
		v2 = 1
		aafasta = self.aa_fasta

		if not aafasta:
			F = open(aafasta, 'w')
			all = Fasta.all(self.contig_fasta)
			for a in all:
				print >>F, '>%s\n%s' % (a.name, DNA.DNA(a.sequence).translate())
			F.close()
			
		#(well_mutation_d, stat_d, repeat_file)
		mutd2 =  MutationDetection.do_aa_mutation(self.contig_fasta, aa_refname='')

		mutd = {}
		for name, d in mutd2.items():
			well = self.get_well_from_contig(name)
			mutd[well ] = d
			d.setdefault('msg', '')

		# get other clones that do not have a protein sequence
		allnts = Fasta.all(self.contig_fasta)
		ntd = {}

		contigd = {}
		for n in allnts:
			well = self.get_well_from_contig(n.name)
			ntd[well] = n
			contigd.setdefault(well, 0)
			contigd[well] += 1

		for well, clone in self.data.items():
			if clone is None:
				continue

			if mutd.has_key(well):
				continue
	
			mutd.setdefault(well, {'msg':'', 'mutations':''})
			if not contigd.has_key(well):
				mutd[well]['tag'] = 'Not Done'
				mutd[well]['msg'] = 'No contig'
				continue

			if Contigd[well] > 1:
				mutd[well]['tag'] = 'Not Done'
				mutd[well]['msg'] = 'more than 1 contigs'
				continue
			if ntd.has_key(well):
				mutd[well]['tag'] = 'No CDS'
			else:
				mutd[well]['tag'] = 'No Contig'

		return mutd

	def _do_mutation(self, aa_alignment='', name='', well_without_growth=[]):
		print "... Plate._do_mutation()0: do mutation summary", aa_alignment
		v1=1
		# get mutation from protein alignment
		if not aa_alignment:
			aa_alignment = self.aa_fasta
		if not name:
			name = self.name
		print "... Plate._do_mutation(): do mutation summary", aa_alignment
		mutd  = self.get_mutation_d(aa_alignment)
		print "... Plate._do_mutation()2: do mutation summary", aa_alignment

		# get nt contig info
		nt_w2d = {}
		contig_fasta = self.contig_fasta
		if os.path.isfile(contig_fasta):
			nts = Fasta.all(contig_fasta)
			for n in nts:
				well = self._get_well_from_contigname(n.name)
				nt_w2d[well] = n

		# get mutation tag, i.e. annotation
		cnt = 0
		for j in self.cols:	
			col_mutations = []
			for i in self.rows:
				well = '%s%02d' % (i, j)
				msg = ""
				mutations = []
				if mutd.has_key(well):
					cnt += 1
					well_mutation = mutd[well]

					tag = ''
					msg = ""
					mixed_base = 0
					mutations = mutd[well]['mutations']
					if not mutations:
						tag = 'WT'

					for m in mutations:
						(r, pos, mut) = m
						if mut.find('X') != -1:
							mixed_base += 1	
					if mixed_base:
						# found mixed base, get quality score for all reads
						clone = self.get_clone(well)	
						if clone is None:
							continue
						print well, clone
						tag = "mixed bases"
						mymsg = []
						scores = []
						for r in clone.reads:
							if r.status in [ABISeq.read_short, ABISeq.read_failed, ABISeq.read_lqv]:
								mymsg.append('%s:%s' % ( r.sample, r.status) )
								scores.append('%s:%s' % (r.sample, r.score()))
						if mymsg:
							tag = "low quality"
							msg = "; ".join(mymsg)
						'''
						elif mixed_base >=5:
							tag = "low quality"
							msg = ";".join(scores)
						else:
							tag = "mixed base"
							msg = ""
						#print well, tag, 'mixedbase=%s' % mixed_base, msg
						'''
					# frameshift?
					f = self.detect_frameshift(mutations)
					if f:
						tag = f
						msg = ""
					#print " 1:mutd:", mutd[well]['tag']
					if not mutd[well].has_key('tag'):
						mutd[well]['tag'] = tag
					elif not mutd[well]['tag']:
						mutd[well]['tag'] = tag
					#print " 2:mutd:", mutd[well]['tag']
					mutd[well]['msg'] = msg

				else:
					#print well, "NO Contig: get read info"
					clone = self.data[well]
					msg = ""
					if clone is None:
						tag = "No Sample"
					elif nt_w2d.has_key(well):
						# has contig, but don't have coding sequence
						tag = "No CDS"
					elif not clone.reads:
						tag = "No Data"
					else:
						# get read status
						#print "clone read count:", len(clone.reads)
						mymsg = []
						for r in clone.reads:
							if r.status in [ABISeq.read_short, ABISeq.read_failed, ABISeq.read_lqv]:
								mymsg.append('%s:%s' % ( r.sample, r.status) )
						msg = "; ".join(mymsg)
						tag = "No Contig"

					mutd.setdefault(well, {})
					mutd[well]['tag'] = tag
					mutd[well]['msg'] = msg
					mutd[well]['mutations'] = mutations

				
				if well in well_without_growth:
					mutd[well]['tag'] += ', no growth'
				col_mutations.append([well, mutd[well]['tag'], mutations])
				#print well, mutd[well]['tag'], mutd[well]['msg']#, mutd[well]['mutations']

			# END: for each row
		print "YYYY"
		return mutd

	# in use 5/26/2011
	def do_mutation_summary(self, aa_alignment='', name='', well_without_growth=[]):
		print ">> Plate.do_mutation_summary():", aa_alignment
		mutd = self._do_mutation(aa_alignment=aa_alignment, name=name)
		#mutd = self._do_mutation_030111()

		# mutd contains mutation information for each well of this plate
		if not name:
			name = self.name
		report_file = os.path.join(self.report_dir, '%s_mutation_summary.txt' % name)
		print "XX1: report file =", report_file
		F = open(report_file, 'w')
		print >>F, '\t'.join(['Sample File', 'Well', 'Tag', 'Info',  'Mutations' ])

		statd = {}
		# row order or column order?
		if self.col_order:
		    for col in self.cols:
		        for row in self.rows:
				well = "%s%02d" % (row, col)
				clone = self.data[well]
				if clone is None:
					continue
				tmp = [clone.name, well, mutd[well]['tag'], mutd[well]['msg']]
				
				muts = []
				mutations = mutd[well]['mutations']
				for mutation in mutations:
					r, p, m = mutation
					p = int(p)
					muts.append('%s%03d%s' % (r, p, m))
				if mutd[well]['tag'].find('insertion') or mutd[well]['tag'].find('insertion')  :
					if len(muts) > 20:
						pass
					else:
						tmp.append("\t".join(muts[:20]))
				else:	
					tmp.append("\t".join(muts))
				print >>F, "\t".join(tmp)
				tag = mutd[well]['tag']
				statd.setdefault(tag, 0)
				statd[tag] += 1
		# row order or column order?
		else:
		    for row in self.rows:
			for col in self.cols:
				well = "%s%02d" % (row, col)
				clone = self.data[well]
				if clone is None:
					continue
				if mutd.has_key(well):
					tmp = [clone.name, well, mutd[well]['tag'], mutd[well]['msg']]
				else:
					tmp = [clone.name, well, '', '']


				if v1:
					muts = []
					mutations = mutd[well]['mutations']
					for mutation in mutations:
						r, p, m = mutation
						p = int(p)
						muts.append('%s%03d%s' % (r, p, m))

					if mutd[well]['tag'].find('insertion') or mutd[well]['tag'].find('insertion')  :
						if len(muts) > 20:
							pass
						else:
							tmp.append("\t".join(muts[:20]))
					else:	
						tmp.append("\t".join(muts))
				elif v2:
					m = mutd[well]['mutations']
					tmp.append(m)
				print >>F, "\t".join(tmp)
				tag = mutd[well]['tag']
				statd.setdefault(tag, 0)
				statd[tag] += 1


		F.close()

		# write repeat, always be in row order
		repeat_file = os.path.join(self.report_dir, '%s_repeat_%s.txt' % (self.name, util.today() ))

		# repeat directory
		repeat_dir = "to_repeat_%s" % self.name #util.today()
		if not os.path.isdir(repeat_dir):
			os.mkdir(repeat_dir)
		else:
			os.system('rm -rf %s/* ' % repeat_dir)

		F = open(repeat_file, 'w')
		print >>F, '\t'.join(['Sample File', 'Well', 'Tag', 'Info', 'Score', 'Repeat', 'Mutations' ])
		for row in self.rows:
			for col in self.cols:
				well = "%s%02d" % (row, col)
				clone = self.data[well]
				tag = mutd[well]['tag']
				if tag in ['', 'WT']:
					continue

				# get score
				scores = []
				redo = []
				clone = self.data[well]
				if clone is None:
					#scores.append('nodata')
					continue
				else:
					redo=[]
					for r in clone.reads:
						trimlen = r.trimlength()
						trimscore = r.trimscore
						if trimscore > 40 and trimlen > 700:
							pass
						else:
							redo.append(r.sample)
						scores.append('%s|%s|%s' % (r.primer, trimscore, trimlen) )

						# copy to repeat_dir
						os.system('cp %s %s' % (r.file, repeat_dir))

				_score = "; ".join(scores)
				if len(redo) == len(clone.reads):
					_redo = "redo PCR"
				else:
					_redo = "; " .join(redo)
				tmp = [clone.name, well, mutd[well]['tag'], mutd[well]['msg'], _score, _redo]
				muts = []
				mutations = mutd[well]['mutations']
				for mutation in mutations:
					r, p, m = mutation
					p = int(p)
					muts.append('%s%03d%s' % (r, p, m))
				if not tag: 
					tmp.append("\t".join(muts))
				print >>F, "\t".join(tmp)
		F.close()

		return (mutd, statd, repeat_file)

	def find_mutation_site(self, col_mutations):
		# find the intended mutated position for each column
		positiond = {}
		wt = 0
		for mut in col_mutations:
			well, tag, mutations = mut
			#print well, mutations
			if tag == 'WT':
				wt += 1
			for m_ in mutations:
				r, p, m = m_
				positiond.setdefault(p, 0)
				positiond[p] += 1


		# (pos, count) list
		pos_count_list = positiond.items()
		pos_count_list.sort(lambda x,y: cmp(y[1],x[1])) # sort in reversed order
		if not pos_count_list:
			return []
		p0, c0 = pos_count_list[0]

		mutation_pos = [p0]
		for x in pos_count_list[1:]:
			p, c = x
			if c == 1:
				break
			if c < c0:
				break
			else:
				mutation_pos.append(p)

		mutation_pos.sort()
		return mutation_pos

	def detect_frameshift(self, mutation_list):
		mutations = mutation_list
		index_diffs = {}
		begin=0
		xcnt = 0
		for i in range(1, len(mutations)):
			a = mutations[i]
			ref1,s,my1 = a
			index_diffs.setdefault(s-begin, 0)
			index_diffs[s-begin] += 1
			begin=s
		
		off1_cnt = 0
		if index_diffs.has_key(1):
			off1_cnt = index_diffs[1]
	
		if off1_cnt > 6:
			return 'frameshift'
		else:
			return ""

	def get_mutation_d(self, aa_alignment):
		# take amino acide file, and do the stat and mutation here
		aaseqs = Fasta.all(aa_alignment)

		# first one is reference
		ref = aaseqs[0]
		#print ref.name, ref.sequence, len(ref.sequence)
		ref_list = list(ref.sequence)
		ref_len = len(ref.sequence)

		mutd = {}
		for a in aaseqs[1:]:
			well = self.get_well_from_contig(a.name)
			mutd.setdefault(well, {'tag':'', 'mutations':[]})

			mylist = list(a.sequence)
			my_len = len(mylist)

			if my_len < ref_len:	
				mutd[well]['tag'] = 'deletion'
			elif my_len > ref_len:
				mutd[well]['tag'] = 'insertion'

			for i in range(0, min(ref_len, my_len)):
				r = ref_list[i].upper()
				m = mylist[i].upper()
				if r != m:
					mutd[well]['mutations'].append([r, i+1, m])
					
			if len(mutd[well]['mutations']) == 0:
				mutd[well]['tag'] = 'WT'
		return mutd

	def get_well_from_contig(self, contig_name):
		# format : V36-81-H01:Contig1
		tmp = contig_name.split(':')[0]
		m = re.search('[A-H]\d\d?$', tmp)
		well = tmp
		if m:
			well = tmp[m.start():m.end()]
		return util.format_well(well)

	def id_mutation(self, ref=''):
		# get <plate>_cds_nt_align.fas
		cds_align = os.path.join(self.report_dir, '%s_cds_nt_align.fas' % self.name)
		if not os.path.isfile(cds_align):
			print "ERROR: %s_cds_nt_align.fas does not exist in the %s" % (self.name, self.report_dir)
			sys.exit()

		# locate outliner
		name2mutpos = self._get_mutation_pos(cds_align, refname='FNA')

		muts = name2mutpos.items()
		muts.sort(lambda x,y: cmp(len(y[1]), len(x[1])))
		outliners = []
		for mut in muts:
			name, mut_pos = mut
			print  name, len(mut_pos)
			if len(mut_pos) > 30:
				print "  OUTLINER: %s" %name
				outliners.append(name)
		if len(outliners) > 15:
			print "TOO many outliners:", outliners
			sys.exit()

		# remove outliner from cds_nt.fas
		cds_fas = os.path.join(self.report_dir, '%s_cds_nt.fas' % self.name)
		if not os.path.isfile(cds_fas):
			print "ERROR: cannot file cds_nt.fas:%s" % cds_fas
			sys.exit()

		cdsseqs = Fasta.all(cds_fas)
		os.system('mv -f cds_fas, %s_1' % cds_fas)
		F = open(cds_fas, 'w')

		aafile = cds_fas.replace('_nt.fas', '_aa.fas')
		os.system('mv -f aafile, %s_1'% aafile)
		Fa = open(aafile, 'w')

		for s in cdsseqs:
			if s.name in outliners:
				continue
			print >>F, '>%s\n%s' % (s.name, s.sequence)
			print >>Fa, '>%s\n%s' % (s.name, s.translate())

		F.close()
		Fa.close()
		self.aafile = aafile
		self.cdsfile = cds_fas

		# redo alignment
		cds_align_fasta=self.do_muscle(cds_fas)
		cds_align_html=self.do_muscle(cds_fas, 'html')

		#aa_align_fasta = self.do_muscle(aafile)
		aa_align_html = self.do_muscle(aafile, 'html')

		# report mutation here
		self.report_mutation(cds_align_fasta, aa_file, 'FNA')
	

	"""
	FNA report specific
	"""
	def _get_mutation_pos(self, aa_align, start, end):
		seqs = Fasta.all(aa_align)
		ref = seqs[0]
		refname = ref.name; refseq = ref.sequence; refseq_list = list(refseq)
		length = len(refseq_list)

		refaas = refseq_list[start-1:end]
		while refaas.count('-') > 0:
			refaas.remove('-') 

		mut_d = {}
		for s in seqs[1:]:
			myseq_list = list(s.sequence)
			myname = s.name
			mut_d.setdefault(myname, [])

			myaas = myseq_list[start-1:end]
			while myaas.count('-') > 0:
				myaas.remove('-')

			# adjust for '-'
			dashcnt1 = s.sequence[:start].count('-')
			dashcnt2 = s.sequence[:end].count('-')
			mut_d[myname].append([start-dashcnt1, end-dashcnt2, "".join(refaas), "".join(myaas)])

			# too short?
			myseq = s.sequence.replace('-','')
			if len(myseq) < 0.7 * length:
				mut_d[myname].append('Short')
				continue

			# get mutation at the mutated region between start and end
			for i in range(0, length):
				if refseq_list[i] != myseq_list[i]:
					if start-1 <= i <= end:
						continue	
					else:
						dashcnt = s.sequence[:(i+1)].count('-')
						mut_d[myname].append([i+1-dashcnt, i+1-dashcnt, refseq_list[i], myseq_list[i] ])
			#print myname, mut_d[myname]
		return ("".join(refaas), mut_d)

	def _get_well_from_contigname(self, name):
		sample_well = name.split(':')[0]
		x = sample_well.split('-')
		if len(x) == 2:
			return x[-1]
		else:
			return sample_well.replace('%s-' % self.name, '')

	# *****************************
	# this is the version inused
	# *****************************
	def report_mutation(self, cds_align_fasta, aa_align_fasta, mut_block_start, mut_block_end):
		if not cds_align_fasta:
			cds_align_fasta = self.cds_align
		if not aa_align_fasta:
			aa_align_fasta = self.aa_align

		refaa, name2pos_aa= self._get_mutation_pos(aa_align_fasta, mut_block_start, mut_block_end)
		#name2pos_nt= self._get_mutation_pos(cds_align_fasta, mut_block_start, mut_block_end)

		# refseq
		ref = Fasta.all(aa_align_fasta)[0]
		dashcnt1 = ref.sequence[:mut_block_start].count('-')
		dashcnt2 = ref.sequence[:mut_block_end].count('-')

		report_file = os.path.join(self.report_dir, '%s_mutation_summary.txt' % self.name)
		F = open(report_file, 'w')
		if mut_block_start-dashcnt1 == mut_block_end-dashcnt2:
			print >>F, '\t'.join(['Sample File', 'Well', 'Tag', 'Mutation Loc: %s' % (mut_block_start-dashcnt1), 'Other Mutations'])
		else:
			print >>F, '\t'.join(['Sample File', 'Well', 'Tag', 'Mutation Loc: %s-%s' % (mut_block_start-dashcnt1, mut_block_end-dashcnt2), 'Other Mutations'])
		print >>F, '\t'.join( ['Ref', '', 'WT', refaa] )

		names = name2pos_aa.keys()
		names.sort()
		wells = []
		for n in names:
			sample_well = n.split(':')[0]
			well = self._get_well_from_contigname(n)

			mutations = name2pos_aa[n]
			main_mutation = mutations[0]
			s, e, refaa, mutaa = main_mutation
			tmp = [sample_well, well]

			tag = ''
			if refaa == mutaa and len(mutations) == 1:
				tag = 'WT'
			tmp.append(tag)
			tmp.append(mutaa)

			if len(mutations) == 2 and mutations[1] == 'Short':
				#tmp.append(mutations[1])
				continue
			
			index_diffs = {}
			begin=0
			xcnt = 0
			for i in range(1, len(mutations)):
				a = mutations[i]
				s, e, ref1, my1 = a
				s = int(s)
				tmp.append('%s%03d%s' % (ref1, s, my1))
				if my1 == 'X':
					xcnt += 1
					tag = 'mixed bases'
				index_diffs.setdefault(s-begin, 0)
				index_diffs[s-begin] += 1
				begin=s
		
			if xcnt >=5 :
				tag = 'low quality'

			off1_cnt = 0
			if index_diffs.has_key(1):
				off1_cnt = index_diffs[1]
	
			if len(tmp) > 10 and off1_cnt > 6:
				tag = 'frameshift'
			tmp[2] = tag
			wells.append(well)

			if len(tmp) > 10:
				tmp = tmp[:10]
				tmp[-1] = tmp[-1] + '...'
			print >>F, "\t".join(tmp)

		all_wells= self.data.keys()
		all_wells.sort()

		# get nt info
		nt_w2d = {}
		if self.nt_fasta:
			nts = Fasta.all(self.nt_fasta)
			for n in nts:
				well = self._get_well_from_contigname(n.name)
				nt_w2d[well] = n

		for w in all_wells:
			if w not in wells:
				clone = self.data[w]
				if clone is None:
					tmp = ['', w, 'No Sample']
				# w not in cds file, but in nt file?
				elif nt_w2d.has_key(w):
					tmp = [clone.name, w, 'No CDS']
				else:
					print "GET read and contig status for: %s" % w
					tmp = [clone.name, w, 'No Contig']
			
					# get read status
					msg = []
					for r in clone.reads:
						if r.status in [ABISeq.read_short, ABISeq.read_failed, ABISeq.read_lqv]:
							msg.append('%s:%s' % ( r.sample, r.status) )

					tmp.append("; ".join(msg) )

				print >>F, '\t'.join(tmp)
		F.close()
		self.report_file = report_file
		return wells


class FanYanPlate(Plate):
	"""
	each well might have more than 1 clone
	"""
	def __init__(self, name, fprimer='M13F', rprimer='M13R', min_distance=200, max_distance=5000, outdir='', well2ref={}, ref_fasta='', cleanup=''):
		print "FanYanPlate(%s): "% name

		self.platenum = name.replace('plate', '')

		if os.path.isfile(ref_fasta):
			all = Fasta.all(ref_fasta)
			for a in all:
				m = re.match('\d+', a.name)
				if not m:
					continue
				platenum = a.name[:m.end()]
				if platenum!= self.platenum:
					continue
				well, = re.findall('[A-H]\d\d?', a.name)
				well = util.format_well(well)
				well2ref[well] = a

		Plate.__init__(self, name=name, well2ref=well2ref)

		for r in self.rows:
			for c in self.cols:
				well = '%s%02d' % (r, c)
				self.data[well] = [] 
		self.fprimer = fprimer
		self.rprimer = rprimer
		self.min_distance = min_distance
		self.max_distance = max_distance

		if not outdir:
			outdir = os.path.join(os.getcwd())

		self.tmp_dir = os.path.join(outdir, 'tmp_%s' % self.name)
		if not os.path.isdir(self.tmp_dir):
			os.mkdir(self.tmp_dir)

		self.report_dir = os.path.join(outdir, 'report_%s' % self.name)
		if not os.path.isdir(self.report_dir):
			os.mkdir(self.report_dir)

		if cleanup:
			print "!! cleanup old temp directory"
			os.system('rm -rf %s/*' % self.report_dir)
			os.system('rm -rf %s/*' % self.tmp_dir)

		self.ref_fasta = ref_fasta

	def set_seqdata(self, sep, datadirs):
		cloned = process_dir.get_allclones(sep, datadirs)
		for clonename, clone in cloned.items():
			#print clonename, clone.name, len(clone.reads)
			# get well from clonename
			well, = re.findall('[A-H]\d+', clonename)
			well = util.format_well(well)
			self.data[well].append(clone)

	def setvalue(self, well, clone):
		well = util.format_well(well)
		for c in self.data[well]:
			if c.name == clone.name:
				return
		self.data[well].append(clone)

	"""
	def do_assembly(self, trimlength=0, prog="cap3", trimtags=[]):
		print "==> DO assembly"
		for well, clones in self.data.items():
			#print well, clones
			if type(clones) == type(None) :
				continue
			if type(clones) not in [type([]), type(())]:
				clones = [clones]	
			for clone in clones:
				#print '\n==>well: %s, clone:%s' % ( well, type(clone ))
				clone.set_dir(self.report_dir, self.tmp_dir)
				try:
					ref = self.well2ref[well]
				except:
					pass
				clone.do_assembly(self.fprimer, self.rprimer, self.min_distance, self.max_distance, notrim=trimlength, assembly_prog=prog, trimtags=trimtags)

	"""
	def _get_ref_detail(self, refname):
		# format: 1A1, 1A12
		platewell = re.findall('^\d[A-H]\d\d?', refname)[0]
		m = re.match('\d', platewell)	
		pid = platewell[:m.end()]
		well, = re.findall('[A-H]\d\d?', platewell)
		well = util.format_well(well)
		return (pid, well)

	def set_reference(self, use_default_ref=0):
		print "--->FanYanPlate: set reference"  
		# self.well2ref, self.ref_fasta
		# do blast to find out the best reference hit, it might be different from what intend to be
		for well, clones in self.data.items():
			if type(clones) not in [type(()), type([])]:
				clones = [clones]

			for clone in clones:
				if clone is None:
					continue

				print "XXX: %s, type:%s" % (well, type(clones))
				if self.well2ref.has_key(well):
					clone.ref = self.well2ref[well]
				
				#if use_default_ref:
				#	clone.match_ref = clone.ref
				#	continue

				clone.match_ref=clone.find_match(self.ref_fasta)


	def align2ref(self, use_default_ref=0):
		print "==>ALIGN to REFERENCE" 
		self.set_reference(use_default_ref=use_default_ref)

		# either well2ref or refile has to present
		for well, clones in self.data.items():
			for clone in clones:
				clone.align2ref(do_translation=1, fix_stop=0)	
				clone.set_status(starttag='AGGCTTC', endtag='AGCTTTC')
		
	def __str__(self):
		table = []
		tmp = [str(x) for x in ['col'] + self.cols]
		table.append('\t'.join(tmp))

		for r in self.rows:
			rowdata = [r]
			for c in self.cols:
				clones = self.getvalue2(r,c)
				if not clones:
					rowdata.append('.')
				else:
					rowdata.append(",".join([x.name for x in clones]) )
			table.append('\t'.join(rowdata))
		return "\n".join(table)

	def html_header(self):
		return '''
	<HTML>
	<HEAD>
	<TITLE>plate report for PLATE %s </TITLE>
	</HEAD>
	<BODY>
	<h2 style="background-color:#ccccff">Plate %s Layout</h2>
	''' % (self.name, self.name)

	def html_footer(self):
		return '</BODY></HTML>'

	def build_html_report(self, refile='', id="clonename", do_translation=1):
		print "\n!!!!!!!!!!%s building report" % self.__class__.__name__
		plate_htmlfile = os.path.join(self.report_dir, 'plate_%s.htm' % self.name)
		FHTML = open(plate_htmlfile, 'w')

		print >>FHTML, self.html_header()
		print >>FHTML, self.html_plate(refile, id, do_translation=do_translation)
		print >>FHTML, self.html_stat()
		print >>FHTML, self.html_footer()
		FHTML.close()
		return plate_htmlfile

	def html_stat(self):
		"""
		print out some basic stat for this plate
		"""
		tmps = ['<h2 style="background-color:#ccccff">Basic Stat</h2>']
		total_clone_cnt = 0
		total_read_cnt = 0

		clone_matchedto_ref = []
		clone_notmatchedto_ref = []
		clone_need_sequencing = []
		clone_perfect_match = []
		clone_mutation = []

		statd = {'match': {}, 'notmatch':{}}
		match = nmatch = 0
		statd['match'].setdefault('mutation', 0)
		statd['match'].setdefault('perfect', 0)
		statd['match'].setdefault('notdone', 0)
		statd['notmatch'].setdefault('notdone', 0)
		statd['notmatch'].setdefault('vector', 0)
		for well, clones in self.data.items():
			if clones is None:
				continue
			if type(clones) not in [types.TupleType, types.ListType]:
				clones = [clones]
			for clone in clones:
				total_clone_cnt += 1
				for r in clone.reads:
					total_read_cnt += 1

				# get clone status here
				if clone.match_ref:
					match += 1
					if clone.has_mutation :
						statd['match']['mutation'] += 1

						clone_mutation.append(clone)
					else:
						statd['match']['perfect'] += 1
						clone_perfect_match.append(clone)
					if not clone.done_sequencing:
						statd['match']['notdone'] += 1
						clone_need_sequencing.append(clone) 
				else:
					nmatch += 1
					clone_notmatchedto_ref.append(clone)
					if clone.noinsert:
						statd['notmatch']['vector'] += 1
					elif not clone.done_sequencing:
						statd['notmatch']['notdone'] += 1
					

		
		match_m = statd['match']['mutation']
		match_p = statd['match']['perfect']
		match_n = statd['match']['notdone']
		nmatch_nd = statd['notmatch']['notdone']
		nmatch_v = statd['notmatch']['vector']

		tmps.append('<ul> Total %s clones, %s reads ' % (total_clone_cnt, total_read_cnt) )
		tmps.append('<li> %s match to reference: %s perfect match, %s has mutation, %s not done sequencing' % (match, match_p, match_m, match_n) )
		tmps.append('<li> %s do not match to reference: %s not done sequencing' % (nmatch, nmatch_nd) )
		return "\n".join(tmps)

	def html_plate(self, refile='', id='clonename', do_translation=1):
		tmps = ['<TABLE WIDTH=900 BORDER=0 CELLSPACING=1 CELLPADDING=1>' ]

		# get table header cells		
		th = []
		for i in self.cols:
			th.append('\t<TH width=66>%s</TH>' % i)

		# create header row
		tmps.append('<TR bgcolor="#ffffcc">')
		tmps.append('\t<TH width=10></TH>')
		tmps += th
		tmps.append('</TR>')

		# create table data r	
		# perfect: 0,176,80 = #00B050 
		# Frame/Stop: 255,0,0 = #FF0000 
		# Other Mutation: 55,96,146,= #376092 
		# Not Done: 255,255,0 = #FFFF00 
		# No Match: 217,150,248 = #D996F8 
		# Match Other: 75,172,198 = #4BACC6ow
		bgcolor_no_sample = "#cccccc"
		bgcolor_perfect = "#00B050"
		bgcolor_other_mutation = "#376092"
		bgcolor_not_done = "#ffff00"
		bgcolor_no_match = "#D996F8"
		bgcolor_match_other = "#4BACC6"
		bgcolor_framestop = "#FF0000"

		tmps.append("""
<style type="text/css">
	span{
		width: 100%;
		display:-moz-inline-block;
		display:-moz-inline-box;
		display:inline-block;
	}
	td {
		text-align: center;
	}


</style> """)
		for row in self.rows:
			tmp2 = ['<TR bgcolor="#dddddd">']
			tmp2.append('\t<TD bgcolor="#ffffcc"> %s </TD>' % row)
			
			for col in self.cols:
				well = self.get_well(row, col) 
				clones = self.getvalue2(row, col)
				if clones is None:
					tmp2.append('\t<TD bgcolor="#CCCCCC">.</TD>' )
				else:
					text_temp = [] 
					if type(clones) not in [type(()), type([])]:
						clones = [clones]

					match_tag = ''
					for clone in clones:
						displayid = ''
						clone_html_file = clone.build_html_report(do_translation=do_translation)
						if id == 'well':
							displayid = well
						elif id == 'clonename':
							displayid = clone.name
							displayid = re.sub('^%s\-' % well, '', displayid)
			

						match_tag = ''

						# don't match
						print "SUMMARY: name=%s, form_contig=%s, done_sequencing=%s"% (clone.name, clone.form_contig, clone.done_sequencing)
						if not clone.match_ref:
							bgcolor = bgcolor_no_match
						else:
							# match, done?, flag match_other first
							match_tag = 'M(1)'
							if not clone.ref:
								bgcolor = bgcolor_match_other
								match_tag = 'M(2)'
							elif clone.match_ref.name != clone.ref.name:
								bgcolor = bgcolor_match_other
								match_tag = 'M(2)'
							elif not clone.done_sequencing:
								bgcolor= bgcolor_not_done
							elif clone.framestop:
								bgcolor= bgcolor_framestop
							elif clone.has_mutation:
								bgcolor = bgcolor_other_mutation
							else:
								bgcolor = bgcolor_perfect

                                                name = clone.name.replace('%s-' % well, '')
						#text_temp.append('<span style="background-color:%s"><a href="./%s">%s</a>  %s<br><font size=-1>%s</font></span>' % (bgcolor, clone_html_file, well, match_tag, name ) )

						text_temp.append('<span style="background-color:%s"><a href="./%s">%s</a><br>%s</span>' % (bgcolor, clone_html_file, displayid, match_tag ) )
					text = '<br>\n'.join(text_temp)

					bgcolor = "#99FFcc"
					tmp2.append('\t<TD>%s</TD>' % ( text) )
			tmp2.append('</TR>')

			tmps.append('\n'.join(tmp2))

		tmps.append('</TABLE>')

		# print legend
		tmps.append('<table align="center" cellspacing=12><tr>')
		for label in [ 'no_sample', 'perfect', 'framestop', 'other_mutation', 'not_done', 'no_match', 'match_other']:
			bgcolor ='bgcolor_%s' % label
			if label == 'not_done':
				tmps.append('<td style="background-color:%s; color=#cc0000">%s</td>' % (eval(bgcolor), LABELD[label]))
			else:
				tmps.append('<td style="background-color:%s">%s</td>' % (eval(bgcolor), LABELD[label]))

		tmps.append('</tr></table>')

		return '\n'.join(tmps)

class VMPlate(FanYanPlate):
	#def __init__(self, name, fprimer='M13F', rprimer='M13R', min_distance=200, max_distance=5000, outdir='', well2ref={}, ref_fasta='', cleanup=''):
	def __init__(self, well2clone, optd, well2ref):
		FanYanPlate.__init__(self, name=optd['p'], fprimer=optd['x'], rprimer=optd['y'], well2ref=well2ref)
		for well, clone in well2clone.items():
			self.data[well] = clone
		#self.refd = well2ref
		self.do_translation=0
	
	def align2ref(self):
		self.set_reference(use_default_ref=1)

		# pick the right sequence to align with reference
		for well, clones in self.data.items():
			if type(clones) not in [type(()), type([])]:
				clones = [clones]
			for clone in clones:
				if clone is None :
					continue
				print "===>", well, clone.name, len(clone.reads), "reads, DO ALIGNMENT"
				#print clone.ref
				clone.align_contig_or_read_2ref(do_translation=self.do_translation)
				print "===>", well, clone.name, "SET STATUS"
				clone.set_status_VM(do_translation=self.do_translation)


	def build_report(self):
		self.do_assembly()
		self.align2ref()
		self.build_html_report(do_translation=self.do_translation)



class PMPlate(FanYanPlate):
	def __init__(self, name, well2ref='', fprimer="M13F", rprimer="M13R", min_distance=200, max_distance=5000, cleanup=0, ref_fasta='' ):
		name = "plate%s" % name
		FanYanPlate.__init__(self, name=name, ref_fasta=ref_fasta, well2ref=well2ref, cleanup=cleanup)

	def _get_ref_detail(self, refname):
		# format: A01|At1213.1:1472 bp 
		platewell = re.findall('^[A-H]\d\d?\|', refname)[0]
		pid = ''
		well = platewell.replace('|', '')
		well = util.format_well(well)
		return  well

	def set_reference(self, use_default_ref=0):
		# self.well2ref, self.ref_fasta
		# do blast to find out the best reference hit, it might be different from what intend to be
		for well, clones in self.data.items():
			if not clones:
				continue

			print "######## set_reference WELL:", well, len(clones), os.path.abspath(os.path.curdir)
			for clone in clones:
				if clone is None:
					continue

				if self.well2ref.has_key(well):
					clone.ref = self.well2ref[well]
				clone.match_ref = ''

				# first match to specific reference
				print "Plate: clone.match2ref", clone.ref.name
				matchto = clone.match2ref(clone.ref)

				# if not match, try to find a match on this plate
				if not matchto:
					matchto=clone.find_match(self.ref_fasta)

				if matchto:
					mwell = self._get_ref_detail(matchto)		
					clone.match_ref = self.well2ref[mwell]

				print "=====>WELL %s: matchto=%s" % (well, matchto)

	def build_html_report(self, refile='', do_translation=1):
		plate_htmlfile = os.path.join(self.report_dir, 'plate_%s.htm' % self.name)
		print "!!!!!!!!!! build html report:", plate_htmlfile
		FHTML = open(plate_htmlfile, 'w')

		print >>FHTML, FanYanPlate.html_header(self)
		print >>FHTML, self.html_plate(refile, do_translation=do_translation)
		print >>FHTML, FanYanPlate.html_stat(self)
		print >>FHTML, FanYanPlate.html_footer(self)
		FHTML.close()
		return plate_htmlfile

	def html_plate(self, refile='', id='clonename', do_translation=1):
		return FanYanPlate.html_plate(self, refile=refile, id=id, do_translation=do_translation)

	def create_repeat_file(self, internal_primers):
		# create repeat file
		rptfile = 'repeats_%s_%s.txt' % (self.name, util.today())
		print "REPEAT FILE:", rptfile
		R = open(rptfile, 'w')
		print >>R, self.name, time.ctime()
		rdir = self.tmp_dir

		lines = open(internal_primers).readlines()
		d = {}
		for l in lines:
			cols = l.split()
			sample = cols[0]
			primer = l.replace(sample, '').strip()
			well = sample.split('|')[0]
			d[well] = primer

		all = []
		# get internal primers
		for well, clones in self.data.items():
			if type(clones) == type(None):
				continue
			for clone in clones:
				if clone.match_ref and not clone.done_sequencing:
					if d.has_key(clone.well()):
						print well, d[well]
						all.append('%s %s' % (well, d[well]) )

		for (r, dirs, files) in os.walk(rdir):
			for f in files:
				if f == 'repeat.txt':
					text = open(os.path.join(r, f)).read().strip()
					if not text:
						continue
					all.append(text)
		all.sort()
		for x in all:
			print >>R, x.strip()
		R.close()

		return rptfile

	def process_plate(self, min_ref_length=300, use_default_ref=0, do_translation=1):
		# do assembly
		self.do_assembly()

		# align to reference
		self.align2ref(use_default_ref=use_default_ref)
		
		# build html report
		self.build_html_report(do_translation=do_translation)

	


class List:
	def __init__(self, name, ref_fasta, wkdir=''):
		self.name = name
		self.refd = {}
		self.data = {}
		all = Fasta.all(ref_fasta)
		for a in all:
			self.refd.setdefault(a.name, a.sequence)

		self.tmp_dir = os.path.join(wkdir, 'tmp_dir')
		if not os.path.isdir(self.tmp_dir):
			os.mkdir(self.tmp_dir)

		self.report_dir = os.path.join(wkdir, 'out_dir')
		if not os.path.isdir(self.report_dir):
			os.mkdir(self.report_dir)

	def set_data(self, sep="_", datadirs=[]):
		self.data.update(process_dir.get_allclones(sep, datadirs))

	"""
	def do_assembly(self):
		contig_file = os.path.join(self.report_dir, '%s_contig.fas' % self.name)
		F = open(contig_file, 'w')
		for clonename, clone in self.data.items():
			if clone is None:
				continue
			clone.set_dir(self.report_dir, self.tmp_dir)
			clone.do_assembly()
			contigs = clone.get_contig.seqs()
			for s in contigs:
				print >>F, s
		F.close()
		self.contig_fasta = contig_file
		return self.contig_fasta
	"""

	def align2ref(self, use_default_ref=1, clone2ref={}):
		for clonename, clone in self.data.items():
			try:
				ref = clone2ref[clonename]
			except:
				ref = ''

			# find a match
			matchto = clone.match2ref(ref)

			if use_default_ref:
				clone.ref = ref
			else:
				clone.ref = matchto
			self.match_ref = matchto	
			clone.align2ref(do_translation=1)
			clone.set_status()

	def build_html_report(self):
		report_file = os.path.join(self.report_dir, 'report_%s.html' % self.name)
		HTML = open(report_file, 'w')
		print >>HTML, '<html><head><title>report</title></head>'
		print >>HTML, '<body>'
		print >>HTML, '<ul>'
		for clone in self.data.values():
			clone_report_file = clone.build_html_report()
			tmp = '<li><a href="%s">clone.name</a></li>' % clone_report_file
		print >>HTML, '</UL></body></html>'
		HTML.close()


if __name__ == '__main__':
	test(sys.argv[1])
