#!python

import os, sys, re, glob, pprint
import util, ABISeq

score_cutoff = 40
seqlen_cutoff = 800
machines = ['T', 'Pu', 'MJ' , 'BO', 'V']
verbose = 0

	
def process_samplesheet(samplesheet):
	print "   SS:", samplesheet
	lines = open(samplesheet).readlines()
	if len(lines) < 100:
		print "ERROR: sample sheet format error"
		return None
			
	line1 = lines[0]
	print "LINE1:", line1
	# second line has plateid
	line2 = lines[1]
	cols = line2.split('\t')
	headers = lines[0].split('\t')
	d = dict(zip(headers,  cols))

	try:
		plateid = d['Plate ID']
	except:
		print "ERROR: plate ssheet %s is invalid" % samplesheet
		return None	
		
	if not plateid:
		print "ERROR: plateid is invalid" 
		return None

	container = d['Container Name']
	if not container:
		container=plateid
	
	operator = d['Operator']
	if not operator:
		operator = "XZ"
	#print "OPERATOR: ", operator
	operator = re.split('[^A-Za-z]', operator)[0]
	operator = operator.upper()
		
	cd = {}
	for line in lines[5:]:
		line = line.strip()
		if not line:
			continue
		cols = [x.strip() for x in line.split('\t')]
		try:
			well, sample, customer = cols[:3]
		except:
			print line
			sys.exit()
		customer = re.sub('[\'\"]', '', customer)
		tmp = customer.replace(' ', '_')
		cols = re.split('[,;]', tmp)	
		notes = ""
		customer = cols[0].lower()
		customer = re.sub('[\-\_\.\;\,]((rpt)|(repeat)|(rr)|(rerun)|(rer))$', '', customer)

		if len(cols) > 1:
			notes = cols[-1]	
		if not customer:
			continue
		if re.match('\#REF', customer):
			continue
		if not sample:
			continue
		if re.match('_', sample):
			continue
		if re.match('\(?none\)?', sample, re.IGNORECASE):
			continue
		cd.setdefault(customer, []).append((well, sample, notes))
		#print customer, sample
	return (plateid, operator, cd)

class SampleSheet:
	def __init__(self, file):
		if not os.path.isfile(file):
			print "ERROR: FILE '%s' invalid" % file
			sys.exit()

		s = process_samplesheet(file)
		if s: 
			(self.plateid , self.operator, cd) = s
			
		else:
			cd = {}
			self.plateid = None
			self.operator = None

		self.customerd = {}
		for c, samples in cd.items():
			self.customerd.setdefault(c, [])
			for s in samples:
				well, name = s[:2]
				cols = name.split('_')
				if len(cols) > 1:
					primer = cols[-1]
					template="_".join(cols[:-1])
					if len(template) > 30:
						template = cols[0]
						primer = "_".join(cols[1:])
				else:
					primer = ''
					template = name
				self.customerd[c].append([well, template, primer])	

def test(file):
	ss = SampleSheet(file)

	print ss.plateid, ss.operator
	for customer, samples in ss.customerd.items():
		print "Customer:", customer
		for s in samples:
			print "\t",s

if __name__ == '__main__':
	test(sys.argv[1])	

