#!python

"""
align sequences
"""
import glob, os, re, sys

from config import *
from util import *
import AceParser, Fasta, BlastParser, DNA

def align(seqd, program, workdir):
	if not os.path.isdir(workdir):
		os.mkdir(workdir)

	for clone, seqs in seqd.items():
		fasta = os.path.join(workdir, '%s.fas'% clone)
		fas = open(fasta, 'w')
		for s in seqs:
			print clone, type(s)

			if isinstance(s, AceParser.Contig):
				print s.name
				for r in s.reads:
					seq =''
					for i in range(0, r.alignment_start):
						seq += '.'
					seq += r.seq.replace('*', '')
					print >>fas, '>%s\n%s' % (r.name, seq)

				name = '%s:%s' % (clone, s.name)
				print >>fas, '>%s\n%s' % (name, s.seq)


			elif type(s) == type(()):
				name, seq = s
				if not name:
					continue
				print >>fas, '>%s\n%s'% (name, seq)
		fas.close()
		cmd = ''
		if program == 'muscle':
			cmd = '%s -in %s -out %s.html -html -stable -quiet'% (MUSCLE, fasta, fasta)
			print cmd
			os.system(cmd)
		


def align2ref(ntref, cdsref, fastafile):
	allseqs = Fasta.all(fastafile)

	cmd = '%s -i %s -p F' % (formatdb, fastafile)
	print "CWD:" , os.path.abspath(os.path.curdir)
	print cmd
	print "----------------"
	os.system(cmd)

	ntname, ntseq = ntref
	cdsname, cdsseq = cdsref

	cdsfile = 'tmp_%s.fas' % cdsname
	Fcds = open('tmp_%s.fas' % cdsname, 'w')
	print >>Fcds, '>%s\n%s' % (cdsname, cdsseq)
	Fcds.close()

	# run blast to figure out orientation 
	cmd = '%s -p blastn -i "%s" -d "%s" -e 1d-5 -m 8 -o x.out' % (blastall, cdsfile, fastafile)
	print cmd
	os.system(cmd)
	return 'x.out'

def blast_ref2db(infile, dbfile, format='-m 8', debug=0):
	if os.path.getsize(dbfile) == 0:
		print "ERROR: blastdb file size is 0: %s" % dbfile
		return ''
	cdir = os.getcwd()

	# get directory name
	db_path = os.path.abspath(dbfile)
	db_dname = os.path.dirname(db_path)
	dbfile_name = os.path.basename(dbfile)

	print ">>>>>chdir %s" % db_dname

	# do blast related staff in db_dname, due to the space in the path
	# blast did not work well with that
	os.chdir(db_dname)

	cmd = '%s -i "%s" -p F' % (formatdb, dbfile_name)
	if debug:
		print ">>>>>", cmd
	os.system(cmd)
	
	infile_name = os.path.basename(infile)
	outfile_name = '%s.out' % infile_name
	outfile = os.path.join(db_dname, outfile_name)

	if not os.path.isfile(infile_name):
		os.system('cp -f %s %s' % (infile, infile_name))

	cmd = '%s -p blastn -i "%s" -d "%s" -o "%s" -e 1e-5 %s' % (blastall, infile_name, dbfile_name , outfile_name, format)
	print cmd
	os.system(cmd)

	## clean up, remove files created by formatdb
	if os.path.isfile('formatdb.log'):
		os.remove(os.path.join(db_dname, 'formatdb.log'))
	'''
	files = glob.glob('%s.*' % dbfile)
	for f in files:
		if f != dbfile and f != outfile:
			os.remove(f)
	'''
	os.chdir(cdir)
	return outfile

def trim_by_tags(fastas, upstream='ttaagaaggagatatacat', downstream='ggatccgaattcgagctc'):
	if not upstream:
		upstream='ttaagaaggagatatacat'
	if not downstream:
		downstream='ggatccgaattcgagctc'

	for fasta in fastas:
		trim1_by_tags(fasta, upstream, downstream)

def trim1_by_tags(fasta, upstream='', downstream=''):

	if not upstream and not downstream:
		return fasta
	if not os.path.isfile(fasta):
		print "ERROR: invalid fasta file %s" % fasta
		sys.exit()

	fname = os.path.basename(fasta)
	dname = os.path.dirname(fasta)
	outfile = os.path.join(dname, 'trim_%s' % fname)

	# get start index
	stag = ''
	etag = ''
	if upstream:
		stag = upstream.upper()
	if downstream:
		etag = downstream.upper()

	# read fasta file
	all = Fasta.all(fasta)
	F = open(outfile, 'w')
	for a in all:
		print >>F, ">%s" % a.name
		ms = _find(a.sequence, stag)
		me = _find(a.sequence, etag)
		if not ms or not me:
			print "Both Tags not found:",a.name
		# both best matchs should be in the same orientation
		trimseq = a.sequence
		if ms and me:
			so, sm = ms
			eo, em = me

			assert( so == eo )
			#print "\t", so, sm.span(), em.span()
			coord = [sm.span(), em.span()]
			coord.sort()
			start = coord[0][1]; end = coord[-1][0]
			trimseq = a.sequence[start:end]	
			if so == -1:
				trimseq = DNA.DNA(trimseq).reverse_complement()
		elif ms:
			so, sm = ms
			if so == 1:
				trimseq = a.sequence[sm.end():]
			else:
				trimseq = DNA.DNA(a.sequence[:sm.start()]).reverse_complement()
		elif me:
			eo, em = me
			if eo == 1:
				trimseq = a.sequence[em.start():]
			else:
				trimseq = DNA.DNA(a.sequence[:em.end()]).reverse_complement()
		
		print >>F, trimseq
	F.close()
	print "Trimfile:", outfile
	return outfile

def _find(seq, tag):
	seqtxt = seq.upper()
	ms = None
	if not tag:
		return ms

	results = []
	rctag = DNA.DNA(tag).reverse_complement()
	for (r,t) in [(1, tag.upper()), (-1, rctag.upper())]:
		# require minimal 8 bases matched
		while len(t) >= 8:
			ms =	re.findall(t, seqtxt)
			if ms:
				break
			# try to shorten the tag
			l = len(t)-1
			t = t[:l]

		if ms:
			results.append((r, ms))

	# get start point
	best =  ''
	lastmatch = 0
	if results:
		for x in results:
			(st, matchseqs) = x
			mseq = matchseqs[0]
			if len(mseq) > lastmatch:
				lastmatch = len(mseq)
				best = (st, mseq)
	# get index of the match
	if best:
		st, mseq = best
		m = re.search(mseq, seqtxt)
		best = (st, m)
	return best


def trim2ref(ref, myfasta, outfile='', order_by_row=0, debug=0):
	error = 0
	if not os.path.isfile(ref):
		print "ERROR: invalid ref file:", ref
		error +=1
	if not os.path.isfile(myfasta):
		print "ERROR: invalid myfasta file:", myfasta
		error +=1
	if error:
		sys.exit()

	blastout = blast_ref2db(infile=ref, dbfile=myfasta, debug=debug)
	if debug:
		print """  DEBUG: 
	refdb: %s
	myfasta: %s
	blastout: %s
	""" % (ref, myfasta, blastout)

	trimseqs =  BlastParser.get_hit_seqs(blastout, ref, myfasta)
	if not outfile:
		outfile = 'trim_contigs.fas'
	os.system('rm -rf %s' % blastout)

	# write reference sequence
	F = open(outfile, 'w')
	print >>F, open(ref).read()

	# write trimmed sequences
	if order_by_row:
		seqs = []
		for s in trimseqs:
			name = s.name.split(':')[0]
			m = re.search('[A-H]\d\d?$', name)
			if m:
				well = name[m.start():m.end()]
				row = well[0]
				col = int(well.replace(row, ''))
			else:
				row = col = name

			seqs.append([col, row, s])
		seqs.sort()
		trimseqs = [z for [x,y,z] in seqs]
		
	else:
		trimseqs.sort(lambda x,y : cmp(x.name, y.name))
	for s in trimseqs:
		name = s.name
		seq = s.sequence
		#print ',,, writing %s, length %s' % ( name, len(seq))
		print >>F, '>%s\n%s' % (name, seq)
	F.close()
	return outfile

def rewrite_cap3(cap3out, refname=''):
	lines = open(cap3out).readlines()
	print "... rewrite cap3 file for ref %s" % refname

	i = 0
	match_to_ref = 0
	for line in lines:
		i += 1
		if refname and re.match(refname, line):
			match_to_ref = 1

		if re.match('DETAILED DISPLAY OF CONTIGS', line):
			break

	# always form contig is it is the right reference sequence, need to implement something else
	form_contig = 0

	x = re.sub('\.fas\.out$', '', cap3out)
	outfile = '%s.html' % x
	print "... writing %s" % outfile
	F = open(outfile, 'w')
	print >>F, ''' <HTML>
<BODY BGCOLOR="#FFEEE0">
<PRE>
	'''
	i = 0
	for l in lines[i:]:
		l2 = l.rstrip()
		if l2.endswith(':'):
			i += 60
			l2 = '%s %02d' % (l2, i)
		print >>F, l2
	#print >>F, "".join(lines[i:])
	print >>F, ''' 
	</PRE>
</BODY>
</HTML>'''
	F.close()

	return (form_contig, match_to_ref, outfile)

def rewrite_muscle_html( htmlfile, rewrite=1):
	"""
	add coordinates
	"""
	lines = open(htmlfile).readlines()
	#os.system('mv -f %s %s_1' % htmlfile)
	F = open('%s' % htmlfile, 'w')

	i = 0
	n = 0
	coordline1 = coordline2 = ''
	for l in lines:
		# add coordinates information
		if re.match('\<SPAN', l):
			if not i:
				i += 1
				start = l.find(">")
				end = l.find('<SPAN', start)-1
				namefield = l[start:end]
				#print namefield, len(namefield)
				seqfield = ''
				n += 1
				n,line1,line2 = _line(n)
				coordline1 ='<SPAN STYLE="background-color:#FFEEE0">%s</SPAN><SPAN STYLE="background-color:#FFFFFF">%s</SPAN>'% ( len(namefield) * ' ', line1) 
				coordline2 ='<SPAN STYLE="background-color:#FFEEE0">%s</SPAN><SPAN STYLE="background-color:#FFFFFF">%s</SPAN>'% ( len(namefield) * ' ', line2) 
				print >>F, coordline2
				print >>F, coordline1
			if rewrite:
				l2 = l.replace('>-<', '>*<')
			else:
				l2 = l
			print >>F, l2.strip()
		elif l.strip() == '':
			print >>F, l.strip()
			n,line1,line2 = _line(n)
			coordline1 ='<SPAN STYLE="background-color:#FFEEE0">%s</SPAN><SPAN STYLE="background-color:#FFFFFF">%s</SPAN>'% ( len(namefield) * ' ', line1) 
			coordline2 ='<SPAN STYLE="background-color:#FFEEE0">%s</SPAN><SPAN STYLE="background-color:#FFFFFF">%s</SPAN>'% ( len(namefield) * ' ', line2) 
			print >>F, coordline2
			print >>F, coordline1
		else:
			l2 = l.strip()
			if rewrite:
				l2 = l2.replace('>-<', '>*<')
			print >>F, l2

	F.close()
	return htmlfile 

def _line(n=1):
	tmp = ''
	tmp2 = []
	for i in range(n, n+60):
		if i%5 == 0:
			tmp +='|'
		else:
			tmp += '.'

		if i%10 == 0:
			tmp2.append( ' ')
			s = str(i)
			le = len(s)
			tmp2[-le:] = list(s)

		else:
			tmp2.append( ' ')
	return (n+60, tmp, "".join(tmp2))

def do_muscle(infile, format='', rewrite=0):
	if not os.path.isfile(infile) or os.path.getsize(infile) == 0:
		return

	align_file = infile.replace('.fas', '_align.')
	f = ''
	if format == 'html':			
		align_file += format
		f = '-html'
	else:
		align_file += 'fas'

	cmd = '%s -in "%s" -out "%s" -quiet %s' % (MUSCLE, infile, align_file, f)
	print cmd
	os.system(cmd)
	if format == 'html' :
		# add coordinates
		return rewrite_muscle_html(str(align_file), rewrite)
	return align_file
	

def do(args):
	usage = """
	%s option <fasta or cap3_alignmefile> <format>
	options:
	-w: rewrite alignment file
	-a: do alignment
	-t: do trim according to tag, need either or both -x and -y
	-x: upstream tag
	-y: downstram tag
	"""
	optd, ars = parse_args(args, "w|a|t|x:|y:", usage)

	if not optd and not ars:
		print usage

	if optd.has_key('t'):
		u = d = ''
		try:
			u = optd['x']
		except:
			pass
		try:
			d = optd['y']
		except:
			pass

		return trim_by_tags(ars, u, d)

	infile = format = ''
	if len(ars) == 2:
		infile, format = ars
	elif len(ars) == 1:
		infile = ars[0]
		format = 'html'

	if optd.has_key('a'):
		do_muscle(infile=infile , format=format)
	elif optd.has_key('w'):
		rewrite_cap3(infile, refname='')
	else:
		if infile:
			do_muscle(infile=infile , format=format)


if __name__ == '__main__':
	do(sys.argv[1:])
	sys.exit()

	do_muscle(infile = sys.argv[1], format=sys.argv[2])
	sys.exit()
	ref = sys.argv[1]
	myfasta = sys.argv[2]
	outfile = sys.argv[3]
	trim2ref(ref, myfasta, outfile=outfile)
