#!/usr/bin/python

import os, sys, re, glob, pprint
from qblib import ABISeq, util

def process_files(files):
	files.sort()
	data=[]
	for f in files:
		ab1 = ABISeq.ABISeq(f)

		basename = os.path.basename(f)
		name = basename.replace('.ab1', '')
		m = re.search('[A-H]\d\d\.ab1', basename)
		# get sample, well info from file name
		if m:
			s = m.start()
			well = basename[s:s+3]
			name = basename[:s]
		else:
			well = ''
		
		# remove  date info from file name
		name = re.sub('\_*\d{4}\-\d{2}\-\d{2}\_', '', name)

		#print basename, ab1.plateid, ab1.sample, ab1.well, well, name
		data.append( "\t".join([ab1.plateid, ab1.sample, ab1.well, name, well] ) )
		if name != ab1.sample or well != ab1.well:
			data.append('X')
		#print "\t".join(data)	
	return "\n".join(data)

def test():
	d = '/Quintara_data/2012-02/2012-02-05_020412C_A/020512_ccliu'
	files = glob.glob(os.path.join(d, '*.ab1'))
	process_files(files)
	
def do(args):
	usage = """
  %s [options] datadir
	-o <output file>
	""" % __file__

	optd, ars = util.parse_args(args, "o:", usage)
	if not ars:
		print usage

	OUT = sys.stdout

	if optd.has_key('o'):
		OUT = open(optd['o'], 'w')
	for d in ars:
		print "processing:", d
		files = glob.glob(os.path.join(d, '*.ab1'))
		print >> OUT, process_files(files)
	OUT.close()

if __name__ == '__main__':
	#test()
	do(sys.argv[1:])

