#!python
"""
For changing file name
"""

import sys, os, re, glob, pprint, getopt
from qblib.util import *

"""
get well2name mapping from a mapping file
each line contains 'well\tname' or 'name\twell'
"""
def get_d(mapfile, force=''):
	"""
	try to figure out format of map file here
	"""
	lines = open(mapfile).readlines()

	x2y = {}
	y2x = {}
	well2name = {}
	# check the first 2 lines to see whether it is in samplesheet format
	line1 = lines[0].strip(); line2 = lines[1].strip()
	if line1 == 'Container Name	Plate ID	Description	ContainerType	AppType	Owner	Operator	PlateSealing	SchedulingPref':
		cols = line2.split('\t')
		plateid = cols[1]	

		# get current working dir name
		dname = os.path.basename(os.path.abspath(os.curdir))

		# plateid and directory name should match
		if not force and not re.search('%s$' % plateid, dname, re.IGNORECASE):
			print "DIRECTORY name %s and plateid %s don't match, continue ?" % (plateid, dname)
			print " to use current mapping file to rename, please run with -F option"
			sys.exit()

		# start well to name mapping from line6
		for l in lines[5:]:
			cols = l.split('\t')
			well, name = cols[:2]
			well2name[well] = name

		return well2name

	for l in lines[5:]:
		l = l.strip()
		if not l:
			continue

		if re.search('well', l, re.IGNORECASE):
			continue
		cols = l.split()

		try:
			x, y = [n.strip() for n in cols[:2]]
		except:
			print "ERROR: line does not contain 2 columns", l
			continue	

		x2y.setdefault(x, []).append(y)
		y2x.setdefault(y, []).append(x)


	# try to figure out which column has well information
	done = 1
	myd = {}
	for well, value in x2y.items():
		if not re.match('[A-H]\d\d?$', well):
			done = 0	
	if not done:
		done = 1
		for well, values in y2x.items():
			if not re.match('[A-H]\d\d?$', well):
				done = 0	

	else:
		print 'DONE: x2y', done
		for k, vs in x2y.items():
			myd[k] = vs[0] 
		return myd

	if not done:
		print "ERROR in mapping file, cannot figure out which column is well column"
		sys.exit()
	else:
		myd = y2x

	for k, vs in myd.items():
		myd[k] = vs[0] 
	return myd

# 12/28/2010
# sample sheet using well as sample name, need to get name from order form, replace well with sample name
# in the sample sheet
def rename_well2name(orderform, samplesheet):
	# process order form, setup well2name mapping
	well2name = {}
	lines = open(orderform).readlines()
	for l in lines[21:]:
		cols = [x.strip() for x in l.split('\t')]
		name = cols[1]
		primer = cols[7]
		well = cols[9]
		well2name[well] = '%s_%s' % (name, primer)
	# process sample sheet
	#Well	Sample Name	Comment	Results Group 1	Instrument Protocol 1	Analysis Protocol 1
	lines = open(samplesheet).readlines()

	nfile = samplesheet.replace('old', 'new')
	F = open(nfile, 'w')
	for l in lines:
		cols = [x.strip() for x in l.split('\t')]
		if len(cols) >= 4:
			sswell, oldname, _, qb = cols[:4]
			assert(qb == 'Quintara')
		else:
			sswell, oldname = cols[:2]
		if not re.match('[A-H]\d\d', well):
			continue
		x = oldname.split('_')
		oname, primer = x
		if not oname or oname == '':
			continue
		well = format_well(oname)
		nname = well2name[well]
		#print sswell, nname, well, oldname
		print >>F, '%s\t%s' % (sswell, nname)
	F.close()

ABIEXT = re.compile('(\.)((seq)|(phd\.1)|(ab1))$')
def rename_old2new(old, new, change, optd={}):
	print "change old value to new value"
	files = os.listdir(os.curdir)
	for f in files:
		#if re.search(ABIEXT, f):
		f2 = f.replace(old, new)
		lines = []
		"""
		if f.endswith('phd.1'):
			# 39BM01H09H.phd.1
			oldname = f.split('.')[0]
			newname = f2.split('.')[0]
			lines = open(f).readlines()
			line0 = lines[0]
			cols = line0.split()
			cols[1] = newname+"\n"
			line0 = " ".join(cols)

			line4 = lines[4]
			cols = line4.split()
			cols[1] = "%s.ab1" % newname + "\n"
			line4 = " ".join(cols)
			lines[0] = line0
			lines[4] = line4
		"""
		
		if f != f2:
			print f, f2
		if change:
			try:
				os.rename(f,f2)
				# also change content of phd.1 file

			except:
				print "ERROR renaming %s to %s" % (f, f2)


def rename_phd1():
	files = glob.glob('*.phd.1')
	for f in files:
		# 39BM01H09H.phd.1
		newname = os.path.basename(f).replace('.phd.1', '')
		lines = open(f).readlines()
		line0 = lines[0]
		cols = line0.split()
		cols[1] = newname+"\n"
		line0 = " ".join(cols)

		line4 = lines[4]
		cols = line4.split()
		cols[1] = "%s.ab1" % newname + "\n"
		line4 = " ".join(cols)
		lines[0] = line0
		lines[4] = line4
	
		f2 = f
		
		F = open(f2, 'w')
		print >>F, "".join(lines).rstrip()
		F.close()	

def rename_new(format, change=0, old='', new='', optd=''):
	print "rename_new:foramt=%s, change=%s, old=%s, new=%s" % ( format, change, old, new)
	if old :
		return rename_old2new(old=old, new=new, change=change, optd=optd)

	# decode format
	format = format.lower()
	if format == 'short':
		return rename(format, change)

	allows = ['name', 'well', 'primer']
	tmps = re.split('[\-\_\;]', format)
	tmps = [x for x in tmps if x in allows]

	formatd = dict([(x,1) for x in tmps])

	files = glob.glob('*.seq') + glob.glob('*.ab1')+glob.glob("*.phd.1")
	datepat = '_\d{4}\-\d\d\-\d\d_'
	for f in files:
		# decode name
		cols = re.split(datepat, f)
		if len(cols) != 2:
			continue
		namestr, wellstr = cols

		name = well= primer = ''
		if formatd.has_key('well'):
			m = re.match('[A-H]\d\d', wellstr, re.IGNORECASE) 
			if m:
				well=wellstr[m.start():m.end()]

		if formatd.has_key('primer'):
			cols = re.split('[\_\;]', namestr)
			primer = cols[-1] 
			name = namestr.replace(primer, '')
			name = re.sub('[\_\;]$', '', name)

		else:
			# no need to get primer informat, so name is the whole namestr
			name = namestr


		newname = format.replace('name', name).replace('primer', primer).replace('well', well)
	
		oldname = re.sub(ABIEXT, '', f)

		f2 = f.replace(oldname, newname)
		#f2 = 'FNA-P40-%s' % f2

		if f != f2:
			print f, f2

		if change:
			try:
				os.rename(f,f2)
			except:
				pass

	
def rename(format, change=0, old='', new=''):
	files = glob.glob('*.seq') + glob.glob('*.ab1')+glob.glob("*.phd.1") + glob.glob('*.qual')

	print "File count to be renamed:", len(files)

	for f in files:

		if format == 'short':
			m = re.search('\_+\d{4}\-\d\d\-\d\d', f)
			if not m:
				continue
			s = m.start()
			em = re.search('\.((seq)|(ab1)|(phd\.1)|(qual))$', f)
			e = em.start()
			n = f[:s] + f[e:]
			#print f, s, e, n
			if change:
				if n != f:
					try:
						os.rename(f, n)
						print "rename %s to %s" % (f,n)
					except:
						pass
					#print n
			else:
				pass
				#print f,n
			continue
		elif format in ['well' , 'Well']:
			base = re.sub('\.((seq)|(ab1)|(phd.1))$', '', f)
			old = base
			m = re.search('\_[A-H]\d\d\_\d{3}$', old)
			well = old[m.start()+1:m.start()+4]
			new = well	
		
		if old and new:
			f2 = f.replace( old, new)
			#print f, f2
			if change:
				os.rename(f,f2)
			continue

		basename = os.path.basename(f)
		b = re.sub('\.((seq)|(ab1)|(phd\.1))$', '', basename)
		ext = basename.replace(b, '')

		# try to get well
		cols = re.split('_', b)
		name, primer, date, well = cols[:4]
		name = name.lower()	
		f2 = '%s-%s_%s%s' % (name, well, primer, ext)

		if change:
			os.rename(f, f2)


def rename_with_mapfile(mapfile='', change=0, verbose=0, force=''):
	d = {}
	if mapfile:
		d = get_d(mapfile, force=force)
		#print d.keys()

	files = glob.glob('*.seq') + glob.glob('*.ab1') + glob.glob('*.phd.1')+glob.glob('*.fsa')
	files.sort()
	print "file count:", len(files)
	print "well:", d.keys()[0]

	datepat	 = re.compile('_\d{4}\-\d\d\-\d\d_')

	for f in files:
		#if re.match('_', f):
		#	continue
		f2=''
		#print f	
		m = datepat.search(f)
		if not m: 
			continue

		oldname = f[:m.start()]
		well = f[m.end():m.end()+3]
		
		if not d.has_key(well):
			print "Well %s not in mapping file" % well
			continue

		newname = d[well]
		#print well, oldname, newname

		#if re.match('_', newname):
		#	continue
		# remove illegal char from name
	 	newname = re.sub('[\\\|\/]', '-',  newname)

		if oldname == newname:
			continue

		#print "well: %s, OLD: %s, NEW: %s" % (well, oldname, newname)

		f2 = f.replace(oldname, newname, 1)

		if change:
			try:
				os.rename(f, f2)
				print "RENAME %s to %s" % (f, f2)
			except:
				print "rename error, skip"
def rename_well(optd):
	change = 0
	if optd.has_key('c'):
		change = int(optd['c'])

	files = glob.glob("*.ab1") + glob.glob("*.seq")
	for f in files:
		cols = f.split('_')
		name = cols[0]
		cols2 = name.split('-')
		well = cols2[-1]
		cols2[-1] = format_well(well)
		name = "-".join(cols2)
		cols[0] = name
		f2 = "_".join(cols)
		print f, f2
		if change:
			os.rename(f, f2)

def do(params):
	usage ="""Usage: rename.py -f mapfile -n formatformat -c 0|1
		-a : rename well from F8 to F08 for all files in cwd
		-M old2new_mapping.txt 0|1: to change name using mapping file"""
	optd, args = parse_args(params[1:], 'f:n:c:O:N:vFM:a', usage)
	if not optd:
		print usage
		sys.exit()

	file = option = format = ''
	c = 0

	if optd.has_key('a'):
		rename_well(optd)
		rename_seq()
		rename_phd1()
		return
	elif optd.has_key('M'):
		old2new(optd['M'], int(optd['c']))
		rename_seq()
		rename_phd1()

	oldv = newv = verbose = force=''
	if optd.has_key('f'):
			file = optd['f']
			if not os.path.isfile(file):
				print "ERROR: file %s does not exist" % file
				sys.exit()

			try:
				format = args[0]
			except:
				format =''
	if optd.has_key('n'):
			format = optd['n']
	if optd.has_key('c'):
			o = optd['c']
			try:
				c = int(o)
			except:
				c = 0
	if optd.has_key('O'):
			oldv = optd['O']
	if optd.has_key('N'):
			newv = optd['N']

	if optd.has_key('v'):
			verbose = 1

	if optd.has_key('F'):
			force = 1

	print "mapfile %s, format %s" % (file, format)
	if file:
		#rename(mapfile=file, format=format, change=c)
		rename_with_mapfile( mapfile=file, change=c, verbose=verbose, force=force)
		rename_seq()
		rename_phd1()
		return

	#rename(format=format, change=c, old=oldv, new=newv)
	rename_new(format=format, change=c, old=oldv, new=newv, optd=optd)
	rename_seq()
	rename_phd1()

def swap(c=0):
	_254B = [('H01', 'D06'),  ('C03', 'D07'), ('A12', 'B01'), ('H12', 'C01')]	

	for a in _254B:
		x,y = a
		xfiles = glob.glob('*%s*' % x)
		yfiles = glob.glob('*%s*' % y)

		tmp = []
		for xfile in xfiles:
			x2 = 't%s' % xfile
			os.rename(xfile, x2)
			print 'rename %s %s' % (xfile, x2)
			tmp.append(x2)

		for yfile in yfiles:
			y2 = yfile.replace(y, x)
			os.rename(yfile, y2)
			print 'rename %s %s' % (yfile, y2)

		for xfile in tmp:
			x2 = xfile.replace(x, y).replace('t', '')
			os.rename(xfile, x2)
			print 'rename %s %s' % (xfile,x2)
	
def rename_seq():
	# get all .seq files
	files = glob.glob('*.seq')
	for f in files:
		name = os.path.basename(f).replace('.seq', '')
		lines = open(f).readlines()
		lines2 = []
		hasname = 0
		for l in lines:
			if re.match('>', l):
				hasname=1
				l = '>%s' % name
			else:
				l = l.strip()
			lines2.append(l)

		F = open(f,'w')
		if not hasname:
			lines2 = ['>%s' % name] + lines2

		print >>F, "\n".join(lines2)
		F.close()

def old2new(mapfile, c=0):
	d = {}
	lines = open(mapfile).readlines()
	for l in lines:
		cols = [x.strip() for x in l.split('\t')]
		on,nn = cols[:2]
		#d['28FR104'+on] = nn
		d[on] = nn
	
	files = glob.glob('*.ab1') + glob.glob('*.seq') + glob.glob('*.phd.1')
	print "File to process:", len(files)
	for f in files:
		#if not re.match('28FR104', f):
		#	continue
		#o = f.split('FRc')[0]
		#o = f.split('.')[0]
		o = re.sub('\.((ab1)|(seq)|(phd.1))', '', f)
	
		if not d.has_key(o):
			cols = o.split('_')
			o = cols[0]

		try:
			n = d[o]
		except:
			print "Error: cannot find old file with name  %s" % o
			continue

		f2 = f.replace(o,n)
		if f != f2 and c:
			try:
				os.rename(f,f2)
				print "rename %s to %s, " % (f, f2)
			except:
				pass
		

def qa_rename(c=0):
	files = glob.glob('*.ab1')
	for f in files:
		nf = '34FR12' + f
		print f, nf
		if c:
			os.rename(f, nf)

if __name__ == '__main__':
	#qa_rename(sys.argv[1])
	#sys.exit()
	#swap()
	do(sys.argv)	
	#rename_well2name(sys.argv[1], sys.argv[2])
