#!/usr/bin/python

import sys, os, re, glob, pprint

from qblib import ABISeq, util

def get_stat(ab1f):
	ab1 = ABISeq.ABISeq(ab1f, qscore=20)
	ab1.trim_by_quality(quality_cutoff=20)
	lor = ab1.LOR()
	q20 = ab1.Q20(score=20)
	q40 = ab1.Q20(score=40)
	s2n = ab1.s2n
	score = ab1.score()

	if not q20:
		q20 = 0
	if not q40:
		q40 =0
	if not lor:
		lor = 0
	if not s2n:
		s2n = 0
	return (ab1.plateid, ab1.well, lor, q20, q40, s2n, score)

def process_plate(pdir, optd):
	# get all ab1 files within pdir
	files = glob.glob(os.path.join(pdir, '*.ab1'))
	if not files:
		return None

	qscore = 40
	if optd.has_key('s'):
		qscore = int(optd['s'])

	seqlen = 600
	if optd.has_key('l'):
		seqlen = int(optd['l'])

	_q40 = 500 
	if optd.has_key('q'):
		_q40 = optd['q']
	to_repeats = []
	for ab1f in files:
		ab1 = ABISeq.ABISeq(ab1f, qscore=qscore, seqlen=seqlen)
		ab1.trim_by_quality(quality_cutoff=qscore)
		namef = os.path.basename(ab1.file)
		namef = namef.replace('.ab1', '')
		if not ab1.trimscore:
			to_repeats.append(namef + ', failed')
		else:
			lor = ab1.LOR()
			q40 = ab1.Q20(score=40)
			if q40 < _q40 or ab1.trimscore < qscore or lor < seqlen:
				to_repeats.append(namef + ', score:%s, q40:%s, lor:%s' % (ab1.trimscore, q40, lor))

	
	pat = re.compile('\d{4}\-\d\d\-\d\d\_')
	based = os.path.basename(os.path.abspath(pdir) )
	m = pat.match( based)
	if m:
		platename = based[m.end():]
	else:
		platename = based
	res = "\n".join(to_repeats)
	
	return res

def process_all(dirs, optd):
	O = open(optd['o'], 'w')
	for d in dirs:
		res= process1(os.path.abspath(d), optd)
		print >>O, res
		print >>O, "\n"
	O.close()

def process1(d, optd):
	# get all data directory within curdir
	pat = '\d{4}\-\d{2}\-\d{2}'
	name = os.path.basename(os.path.abspath(d))
	#LOG = open('score_%s.txt' % name , 'w')
	#sys.stdout = LOG
	results = ''
	for root, dirs, files in os.walk(d):
		basename = os.path.basename(root)
		if re.match('bad', basename) or re.match('notuse', basename) or re.match('tmp', basename):
			continue
		if re.match('out', basename):
			continue
		dirs = [x for x in dirs if re.match(pat, os.path.basename(x)) ]
		files = [f for f in files if f.endswith('.ab1')]
		if not files:
			continue
		print "... process %s" % root
		results += '...process %s\n' % root
		res = process_plate(root, optd)
		results += res
		#continue?
		#files = [f for f in files if f.endswith('ab1')]
		#if not files:
		#	break
		#print "\n"
	return results

def do_qcstat(datadirs):
	qcd = {}
	for d in datadirs:
		for root, dirs, files in os.walk(d):
			basename = os.path.basename(root)
			#print d, root, basename
			if re.match('bad', basename) or re.match('notuse', basename) or re.match('tmp', basename) or re.match('\(?NONE\)?', basename, re.I):
				continue
	
			for f in files:
				if not f.endswith('ab1'):
					continue
				if re.match('_', f):
					continue
				if re.match('\(?none\)?', f, re.I):
					continue
				m = re.search('\d{4}\-\d\d\-\d\d', f)
				if m:
					name = f[:m.start()]
				else:
					name = f.replace('.ab1', '')
				(plateid, well, lor, q20, q40, s2n, score) = get_stat(os.path.join(root,f)	)
				#print plateid, name
				qcd.setdefault(plateid, []).append([name, well, lor, q20,q40, s2n, score])
	#pprint.pprint(qcd)
	return qcd

def do(ars):
	optstr = "d:|o:|s:|l:|Q|a:"
	usage = """
	%s [option] dirlist
		options:
		-s <score cutoff>: default is 40
		-l <length cutoff>: default is 700
		-d <output directory>: default is curdir
		-o <output filename>: default is score_<dirname>.txt
		-Q: get QC stat
	""" % __file__
	optd, ars = util.parse_args(ars, optstr, usage)
	if optd.has_key('Q'):
		if optd.has_key('a'):
			ars = glob.glob('%s*'% optd['a'])
		return do_qcstat(ars)

	if not ars:
		print usage
		return
	optd.setdefault('s', 40)
	optd.setdefault('l', 600)
	optd.setdefault('d', os.path.abspath(os.path.curdir))
	outfile = 'score_' + os.path.basename(os.path.abspath(os.path.curdir)) + '.txt'
	optd.setdefault('o', outfile)

	process_all(ars, optd)

if __name__ == '__main__':
	do(sys.argv[1:])
	#process_plate(sys.argv[1])
