#!/usr/local/bin/python

import os, sys, re, glob

import DNA


class Fasta:
	def __init__(self, name, sequence):
		self.name = name
		self.sequence = sequence

	def __repr__(self):
		return self.__str__()

	def __str__(self):
		tmp = [">%s" % self.name, "\n"]
		for i in range(0, len(self.sequence), 100):
			tmp.append( self.sequence[i:i+100])
			tmp.append("\n")
		return "".join(tmp)

	def translate(self):
		d = DNA.DNA(self.sequence.upper())
		return d.translate()

	def match(self, seqstr):
		return DNA.DNA(self.sequence).match(seqstr)

def make_fasta(text):
	# remove anything that is not ATCG
	text  = text.split('\n')
	text = "".join(text)
	seq = text.upper()
	seq = re.sub('[^ATCG]', '', seq)
	seq = re.sub('\s+', '', seq)
	return seq

def fasta2text(fasta):
	seqs = all(fasta)
	f = fasta.replace('.fas', '.txt')
	F = open(f, 'w')
	for s in seqs:
		i = re.search('\:Contig\d+', s.name)
		if i:
			name = s.name[:i.start()]
		else:
			continue
			name = s.name
		print >>F, '%s\t%s' % (name, s.sequence)
	F.close()
	return f

def all(file):
	items = []
	
	lines = open(file).readlines()
	d = {}
	allnames = []
	name = ''
	for line in lines:
		if line.startswith(">"):
			name = line[1:].strip().split()[0]
			allnames.append(name)

			d.setdefault(name, '')
		else:
			if not name:
				return make_fasta("".join(lines))

			d[name] += line.strip()


	for name in allnames:
		items.append(Fasta(name, d[name]))

	return items


def test(file):
	#seqs = all(file)
	dirs = glob.glob('report_*')

	rdir = "text_out"
	if not os.path.isdir(rdir):
		os.mkdir(rdir)

	for d in dirs:
		if not os.path.isdir(d):
			continue

		print ".. processing %s" % d
		files = glob.glob(os.path.join(d, "*.fas"))
		for f in files:
			if f.find('_align') != -1:
				continue
			f2 = os.path.basename(f).replace('.fas', '.txt')
			f2 = os.path.join(rdir, f2)

			seqs = all(f)
			F = open(f2, 'w')
			for s in seqs:
				name = re.sub('\:Contig\d*$', '', s.name)
				print >>F, '%s\t%s' % (name, s.sequence)
			F.close()

		
def find_notdone(file,l=1500):
	sequecnes = all(file)
	for a in sequecnes:
		if len(a.sequence) < l:
			t = "  ----> rpt:"
		else:
			t = "DONE:"
		print t, a.name, len(a.sequence)

def test2(file):
	allseqs = all(file)
	for a in allseqs:
		print a.name, len(a.sequence)
		print "\t", DNA.DNA(a.sequence).translate()

if __name__ == '__main__':
	#test2(sys.argv[1])
	find_notdone(sys.argv[1])
